!**************************************************************************!
! SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1 !
!**************************************************************************!
!**************************************************************************!
!                                                                          ! 
! SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013                     ! 
!                                                                          !
! Copyright Natural Environment Research Council, UK                       !
!                                                                          !
! Contains code derived from NEMO 3.4 (Copyright NEMO Consortium)          !
!                                                                          !
! Author: John Hemmings, National Oceanography Centre                      !
! e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013)                         !
! Alternative e-mail for author: marmot@wessexenv.mail1.co.uk              !
!                                                                          !
! This software is governed by the CeCILL  license under French law and    !
! abiding by the rules of distribution of free software.  You can  use,    ! 
! modify and/ or redistribute the software under the terms of the CeCILL   !
! license as circulated by CEA, CNRS and INRIA at the following URL        !
! "http://www.cecill.info".                                                ! 
!                                                                          !
! As a counterpart to the access to the source code and  rights to copy,   !
! modify and redistribute granted by the license, users are provided only  !
! with a limited warranty  and the software's author,  the holder of the   !
! economic rights,  and the successive licensors  have only  limited       !
! liability.                                                               ! 
!                                                                          !
! In this respect, the user's attention is drawn to the risks associated   !
! with loading,  using,  modifying and/or developing or reproducing the    !
! software by the user in light of its specific status of free software,   !
! that may mean  that it is complicated to manipulate,  and  that  also    !
! therefore means  that it is reserved for developers  and  experienced    !
! professionals having in-depth computer knowledge. Users are therefore    !
! encouraged to load and test the software's suitability as regards their  !
! requirements in conditions enabling the security of their systems and/or ! 
! data to be ensured and,  more generally, to use and operate it in the    ! 
! same conditions as regards security.                                     ! 
!                                                                          ! 
! The fact that you are presently reading this means that you have had     !
! knowledge of the CeCILL license and that you accept its terms.           !
!                                                                          !
!**************************************************************************!
!=======================================================================
! MODULE trdiffuse
! John Hemmings - 02 Apr 2013 
!
! MarMOT diffusion subroutine 
!
!  - diffusion - calculate new tracer profile applying input tendency 
!    and vertical diffusion (if any) over one time step 
!
! The diffusion scheme is a backward (or implicit) diffusion scheme 
! based on NEMO 3.4 code described in Madec et al. 2008 "NEMO ocean engine". 
! Note du Pole de modélisation de l'Institut Pierre-Simon Laplace (IPSL), 
! France, No 27 ISSN No 1288-1619. (Version 3.4: Gurvan Madec & the NEMO 
! team, January 2012)
!
!=======================================================================

  module trdiffuse

  use modconfig, only: kind_r_ext,kind_r

  implicit none

  contains

!=======================================================================

  subroutine diffusion(scheme,km,dz,dzw,dt,vdc,ddt,tracer_in,tracer)

  use trlink, only: ntracer,itracer

  implicit none

! Arguments (input only)

  integer scheme                  ! Diffusion scheme id
  integer km                      ! Maximum depth level
  real(kind_r) dz(:)              ! Level height (m)
  real(kind_r) dzw(:)             ! Level mid-point separation (m)
  real(kind_r) dt                 ! Time step (d)
  real(kind_r_ext) vdc(:)         ! Vertical diffusion coeff. (m^2/d)
  real(kind_r_ext) ddt(:,:)       ! Tendencies due to other processes (/d)
  real(kind_r_ext) tracer_in(:,:) ! Input tracer concentrations

! Output argument

  real(kind_r_ext) tracer(:,:)    ! Output tracer concentrations

!-----------------------------------------------------------------------
! Implicit diffusion scheme
!
! Solve m.x = y for x where m is a tri-diagonal n * n matrix and n 
! is the number of depth levels 
!
!     ( matd1 mats1   0     0     0   )( x1 ) ( y1 )
!     ( mati2 matd2 mats2   0     0   )( x2 ) ( y2 )
!     (   0   mati3 matd3 mats3   0   )( x3 )=( y3 )
!     (              ...              )( .. ) ( .. )
!     (   0     0     0   matin matdn )( xn ) ( yn )
!
!-----------------------------------------------------------------------

  real(kind_r) mati(km)
  real(kind_r) mats(km)
  real(kind_r) matd(km)
  real(kind_r) wk(km)

  real(kind_r) tracer_tmp(km)
  integer i,j,k,kmm1

  kmm1=km-1

! Apply input tendencies

  do i=1,ntracer
    j=itracer(i)
    if (j/=0) tracer(1:km,j)=tracer_in(1:km,j)+ddt(1:km,j)*dt
  enddo

  select case (scheme)

  case (1)  ! Implicit diffusion scheme

!   Set up tri-diagonal matrix (no diffusion at top and bottom of water column)

    mati(1)=0.0
    do k=2,km
      mati(k)=-dt*vdc(k-1)/(dz(k)*dzw(k-1))
    enddo

    do k=1,kmm1
      mats(k)=-dt*vdc(k)/(dz(k)*dzw(k))
    enddo
    mats(km)=0.0

    do k=1,km
      matd(k)=1.0_kind_r-mati(k)-mats(k)
    enddo 

!   Calc. tracer-independent intermediates for matrix inversion

    wk(1)=matd(1)
    do k=2,km
      wk(k)=matd(k)-mati(k)*mats(k-1)/wk(k-1)
    enddo

!   Do diffusion for each tracer 

    do i=1,ntracer
      j=itracer(i)
      if (j/=0) then 

        do k=2,km
          tracer(k,j)=tracer(k,j)-mati(k)/wk(k-1)*tracer(k-1,j)
        enddo

        tracer(km,j)=tracer(km,j)/wk(km)
        do k=kmm1,1,-1
          tracer(k,j)=(tracer(k,j)-mats(k)*tracer(k+1,j))/wk(k)
        enddo

      endif
    enddo

  end select

  end subroutine

!=======================================================================

  end module trdiffuse



