/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1

Includes modifications to 2013 MarMOT 1.1 code (MarMOT 1.1.0) as listed 
 - Copyright 2015 John C. P. Hemmings 
***********************************************************************/
/***********************************************************************

SOFTWARE RELEASE: MarMOT 1.1  DATE: 21 November 2013

Copyright Natural Environment Research Council, UK

Author: John Hemmings, National Oceanography Centre 
e-mail: j.hemmings@noc.ac.uk (until 30 Nov 2013) 
Alternative e-mail for author: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/***********************************************************************

 gfan.c - John Hemmings - 21 Nov 2013

 Generic Function Analyzer v1.1 main program 

 MODIFIED:

 John Hemmings - 15 Jan 2015 (GFAn v1.1.1)
 - Included upper limit check on number of bits for representing a 
 parameter in the genetic algorithm.
 - Removed option to set elitism off in genetic algorithm optimization 
 since this option is unsuitable for the micro GA.
 - Made NetCDF data handling optional allowing code to be built without 
 the NetCDF library. 
 - Fixed bug where '.nct' file read errors were not trapped.
 - Corrected error handling for items other than gridded domain items
 in the event that no item data file is found. 
 - Listed option codes in usage message. 
 - Changed handling of option codes to ensure standard behaviour if a
 'runoption' code of UNKNOWN is returned after processing command line.
 - Modified start up banner to reflect new version details.
 - Removed display of copyright message for David Carroll's genetic
 algorithm driver code that no longer applies.
 - Replaced display of copyright message for WELL1024a random number
 generator with code to optionally display a random number generator
 description provided by the external function 'randomu_descrip' (to
 allow for use of different random number generators).

***********************************************************************/

#include "gfan.h"
#include "garandom.h"
#include "genome.h"
#include <stddef.h>
#include <stdarg.h>
#include <limits.h>
#include <sys/stat.h>

#if NETCDF_ENABLED == 1
  #include <netcdf.h> 
#endif

#define GFANVER "1.1.1 (15 Jan 2015)"
#define LOG_FNAME "gfan.log"
#define NRECMAX_LOGINSTANCE 1000

#define N_CTRLTABVAR 1 + N_ITEM + (N_CASEVAR > 0 ? 1 : 0) + 2 * (N_OUTTAB) \
        + 8 * (OPT_ENABLED ? 1 : 0)
#define N_CASETABVAR N_CASEVAR + N_ITEM 
#define N_NCTABVAR 2 + 3 * (N_DIM) + N_ITEMVAR_MAX

#if N_CASETABVAR == 0
  #define N_CASETABVAR_NZ 1
#else
  #define N_CASETABVAR_NZ N_CASETABVAR
#endif

#define N_OPTCONFVAR 18 

#define RULED_LINE "--------------------------------------------------------------------------------"
#define RUNHDR_LINE "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"

struct fnamerec {
  char *name;
  int flag;
  int createfile;
  int resident;
};

struct ctrlrec {
  char *fieldlist;
  struct fnamerec logf;
  int ioptpset; 
  struct fnamerec optconfigf;
  struct fnamerec freeparmf;
  struct fnamerec parminitf;
  struct fnamerec funcgaf;
  struct fnamerec funcpowellf;
  struct fnamerec newparmf;
  struct fnamerec optreportf;
  struct fnamerec casef;
  struct fnamerec itemf[N_ITEM_NZ];
  struct fnamerec outf[N_OUTTAB_NZ];
  struct fnamerec outvarf[N_OUTTAB_NZ];
  struct ctrlrec *next;
};

struct gspecvarnamerec {
  char *first;
  char *last;
  char *step;
  char *n;
  char *min;
  char *max;
};

struct ncvarreportrec {
  int totcount;
  int gridcount;
  int countmax;
};

struct nciorec {
  int ncid;
  double fillval;
  int nvar;
  unsigned char *varactive;
  double *varscale;
  double *varoffset;
  int varid[N_ITEMVAR_MAX];
  int filecount;
  struct ncvarreportrec varreport[N_ITEMVAR_MAX];
  int dimflag[N_DIM_NZ];
  int dimpos[N_DIM_NZ];
  size_t blocklen[N_DIM_NZ];           
  struct itemrec *tabentry;
};

enum comstates {INIT,NOARG_INI,NOARG,HAVEOPTION,HAVELOGNUM,GO,QUIT};
enum runoptions {STANDARD,SKIPXREF,SKIPCHECK,SKIPOF,SKIPO,SKIPF,UNKNOWN};

static int setnames (struct objfnamerec *, char *[], char *[], char *[], 
                     struct gspecvarnamerec [], char *[], char **);
static enum comstates commandstatus (enum comstates, int *, char *[], 
                                     enum runoptions *, int *, int *, int *, 
                                     int *, int *);
static enum runoptions getoption (char *);
static int getlognum (char *);
static int maketemplates (char *[], char *[], enum itemcategory [], 
                          struct objfnamerec *, char *[],
                          struct gspecvarnamerec [], char *[]);
static int writectrltemplate (FILE *, char *[], struct itemnamerec[]);
static int writeoutvartemplate (FILE *, int, char *[], char *[]);
static int writecasetemplate (FILE *, char *[], char *[]);
static int writeitemtemplate (FILE *, enum itemcategory, char *[],
                              char *, struct itemnamerec *, 
                              struct gspecvarnamerec [], char *[]);
static int writeitemdatatemplate (FILE *, enum itemcategory, char *[], char *,
                                  struct itemnamerec *, char *[]);
static int writeitemdatavariables (FILE *, enum itemcategory,
                                   struct itemnamerec *, char *[],
                                   struct itemtabrec *);
static int writetemplate (FILE *, int, int [], char *[], char *[], char *);
static int readctrlfile (char *, char *[], struct itemnamerec[], 
                         struct ctrlrec **, struct ctrlrec **, int *, char **);
static int getinfname (char *, int, struct fnamerec *, struct fnamerec *,
                       char *, int *);
static int getoutfname (char *, struct fnamerec *, char *);
static int readoutvarfile (char *, int, char *[], unsigned char [], int *);
static int readcasefile (char *, char *[], char *[], struct casetabrec *, 
                         int *, char **);
static int readitemfile (char *, enum itemcategory, char *[], char *, 
                         struct itemnamerec *, struct gspecvarnamerec [],
                         char *[], struct itemtabrec *, int *, char **);
static int readitemdatafile (char *, enum itemcategory, struct itemnamerec *,
                             char *[], struct itemtabrec *, int *, char **);
static int grabitemspace (enum itemcategory, int [], unsigned char [],
                          struct itemrec *, int, int, unsigned long *);

#if NETCDF_ENABLED == 1

static int readitemdata_nc (char *, int, enum itemcategory, 
                            struct itemnamerec *, char *[], int [], char *[], 
                            struct itemtabrec *, double, double [], double [], 
                            double [], double [], FILE *, char **);
static void quicksort (size_t [], double [], int, int);  
static void swap (size_t [], int, int);  
static int writencvarreport (FILE *, int, int, unsigned char [], char *[],
                             struct ncvarreportrec []);
static int readitemdata_nct (char *, enum itemcategory, struct itemnamerec *, 
                             char *[], struct itemtabrec *, FILE *, char **,
                             int *, char **);
static int writenctrec (FILE *, int, char *[], struct datarec *, int []);
static int extractdata_nc (struct nciorec *, int, size_t [], size_t *[], int,
                           size_t [], size_t []); 
static int mapcuboid_nc (struct nciorec *, int, size_t *[], size_t [], 
                         size_t []); 

#endif

static int makesinglecase (struct casetabrec *);
static int crossref (struct casetabrec *, int, struct itemtabrec *, int *);
static int xrefgroup (struct casetabrec *, int, int, struct itemtabrec *, int,
                      int, int *);
static int writectrlrec (FILE *, struct ctrlrec *, struct objfnamerec *);
static int writelogcasetab (FILE *, struct casetabrec *, char *[], char *[], 
                            char *[]);
static int writelogitemtab (FILE *, struct itemtabrec *, char *, 
                            struct itemnamerec *, struct gspecvarnamerec [],
                            char *[]);
static int writelogitemdatatab (FILE *, int, struct itemtabrec *, char *, 
                                struct itemnamerec *, char *[]);
static int writeitemdatatab (FILE *, int, struct itemtabrec *, char *, 
                             struct itemnamerec *, char *[]);
static void exiterror_logged (FILE *, int, char *, char *, int);
static void clearobjfnames (struct objfnamerec *);
static void deletectrlrec (struct ctrlrec *);
static void clearcasetab (struct casetabrec *);
static void clearitemtab (struct itemtabrec *);


#if OPT_ENABLED == 1 

/* Optimization related functions */

static int writeoptconfigtemplate (FILE *, char *[]); 
static int writefreeparmtemplate (FILE *, char *[]);
static int writeparminittemplate (FILE *, struct itemnamerec *);
static int readoptconfigfile (char *, char *[], struct optdatarec *, int *, 
                              char **);
static int readfreeparmfile (char *, char *[], struct itemnamerec *, 
                             struct parmctrlrec *, int *, char **);
static int readparminitfile (char *, struct itemnamerec *, struct psettabrec *,
                             int *);
static int writelogoptconfig (FILE *, struct optdatarec *);
static int writenewparmvar (FILE *, struct parmctrlrec *, char *[]);
static int writeobjfevalvar_ga (FILE *, struct parmctrlrec *, char *[]);
static int writeobjfevalvar_powell (FILE *, struct parmctrlrec *, char *[]);
static void clearoptconfig (struct optdatarec *);
static void clearparmctrlrec (struct parmctrlrec *);
static void clearpsettab (struct psettabrec *);
static OBJF_FLOAT_TYPE *saveparmitem (struct itemtabrec *, 
                                      struct itemnamerec *);
static void restoreparmitem (OBJF_FLOAT_TYPE *, struct itemtabrec *, 
                             struct itemnamerec *); 

#endif


/* General purpose buffers for reading and writing data records */

static struct linebuffer {
  double x[NFIELD_MAX];
  char fieldlist[LINE_MAX+1];
  char *field[NFIELD_MAX];
} buff1,buff2; 

static struct pagebuffer {
  double x[NPAGEREC_MAX][NFIELD_MAX];
  char *field[NPAGEREC_MAX][NFIELD_MAX];
} pagebuff;


/* External flags for activating/deactivating output table variables in library 
   write functions */

static unsigned char outvaractive[N_OUTTAB_NZ][NFIELD_MAX];
static int outtabttb[N_OUTTAB_NZ];


#if OPT_ENABLED == 1 

static int newparmttb,funcgattb,funcpowellttb;

#endif


int main (int argc, char *argv[])

{ struct {
    int xref;
    int check;
    int opt;
    int func;
  } stepenabled;

  char *argvec[NFIELD_MAX];
  enum comstates comstate;
  enum runoptions runoption;
  FILE *logfile,*mainlogfile,*outfile[N_OUTTAB_NZ],*f;
  char *logfname,*mainlogfname;
  char *ctrltabvarname[N_CTRLTABVAR];
  char *optconfvarname[N_OPTCONFVAR]=
    { "ga_maxgen",
      "ga_mingen",
      "ga_npop",
      "ga_nout",
      "ga_nchild",
      "ga_initflag",
      "ga_uniflag",
      "ga_pcross",
      "ga_seed",
      "powell_max",
      "powell_maxbrac",
      "powell_maxbrac0",
      "powell_maxbr",
      "powell_fftol",
      "powell_xatol",
      "powell_fftolbr",
      "powell_xftolbr",
      "powell_xatolbr"};
  char *keyvarname[N_ITEM_NZ];
  char *itemnumvarname[N_ITEM_NZ];
  struct gspecvarnamerec gspecvarname[N_DIM_NZ];
  char *dimflagvarname[N_DIM_NZ];
  struct ctrlrec *ctrlhead=NULL,*ctrl=NULL;
  struct objfnamerec objfname;
  struct casetabrec casetab;
    /* Case table: master input data for objective function tabulated by case */
  struct itemtabrec itemtab[N_ITEM_NZ];
    /* Item table: parameter sets / item specifications, tabulated by 
       case or item key */
  int newcasetab;
  int newitemtab[N_ITEM_NZ];
  OBJF_FLOAT_TYPE *origparm;
  char logfnamebuff[LINE_MAX+1];
  char title1[LINE_MAX+1];
  char title2[LINE_MAX+1];
  char linebuff1[LINE_MAX+1];
  char linebuff2[LINE_MAX+1];
  struct stat statbuff;
  char *missingvar="",*errtxt=NULL,*ncerrfile=NULL;
  int echo,offertemplates,status,found,xrefstatus,logflag,lineno,i,j,k,num,
      errcasenum,erritemtype,errparm,firstctrl,lognum;
  int logrecmax;
  enum itemcategory itemcat[N_ITEM_NZ];
  enum {OPT,NONOPT} mode;
  char *modename,*txt1,*txt2,*txt3,*txt;
  int ttb;
  

#if OPT_ENABLED == 1 

  FILE *newparmfile,*funcgafile,*funcpowellfile,*optreportfile;
  struct optdatarec optdata;
  int freeparmno;

  txt1=NULL;

#else

  txt1="\n\n                         OPTIMIZER NOT ENABLED";

#endif

  txt2=
    " Modifications to original code - Copyright 2015 John C. P. Hemmings\n";

  txt3=randomdescrip(&logflag);  /* Get description of random number generator 
                                    which may contain copyright notice */

  sprintf(title1,
          "                  ***** Generic Function Analyzer (GFAn) *****\n\n                             Author: John Hemmings\n                        UK National Oceanography Centre\n              Copyright 2013 Natural Environment Research Council\n                       Licence: CeCILL (www.cecill.info)\n\n                          Version: %s\n\n\n",GFANVER);

  if (argc<2) 
    { printf(RULED_LINE);
      printf("\n\n%s\n\n",title1);
      if (txt1) printf("%s\n",txt1);
      if (txt2) printf("%s\n",txt2);
      if (txt3 && logflag) printf("%s\n",txt3);
      printf(RULED_LINE);
      printf("\n");
    }

  clearobjfnames(&objfname);
  status=objfun_setnames(&objfname,&errtxt);
  if (status!=OK) exiterror(status,argv[0],errtxt,0); 
  for (i=N_PSET;i<N_ITEM;i++) 
    if (objfname.item[i].nvar>N_ITEMVAR_MAX) 
      { sprintf(linebuff1,
  "item '%s' has %d variables but N_ITEMVAR_MAX=%d - check objfun_setnames and increase N_ITEMVAR_MAX if necessary",
                objfname.item[i].name,objfname.item[i].nvar,N_ITEMVAR_MAX);
        exiterror(SPECIAL,argv[0],linebuff1,0);
      }

  status=setnames(&objfname,ctrltabvarname,keyvarname,itemnumvarname,
                  gspecvarname,dimflagvarname,&errtxt);
  if (status!=OK) exiterror(SPECIAL,argv[0],errtxt,0);

  for (i=0;i<N_ITEM;i++)
    { if (i<N_PSET) itemcat[i]=PSET;
      else
        { if (i<N_PSET+N_MATITEM) itemcat[i]=MATRIX;
          else
            { if (i<N_PSET+N_MATITEM+N_GDOMITEM) itemcat[i]=GDOMITEM;
              else itemcat[i]=NGDOMITEM;
            }
        }
    }

  sprintf(title2," Application name: %s\n Version: %s",
          objfname.function,objfname.version);

  if (argc<2) 
    { printf("%s\n",title2);
      printf(RULED_LINE);
      printf("\n\n");
      if (objfname.description) 
        { printf(" --- APPLICATION DESCRIPTION ---\n\n");
          printf("%s\n\n",objfname.description);
        }
      printf(" --- GFAn APPLICATION-SPECIFIC ENVIRONMENT ---\n\n");
      if (N_DIM>0) 
        { if (N_DIM>1) printf("Domain dimensions ............... :");
          else printf("Domain dimension ................ :");
          for (i=0;i<N_DIM;i++) printf(" %s",objfname.dimension[i]);
          printf("\n");
        }
      i=0;
      if (N_PSET>0) 
        { if (N_PSET>1) printf("Input parameter set items ....... :");
          else printf("Input parameter set item ........ :");
          for (j=0;j<N_PSET;j++) printf(" %s",objfname.item[i++].name);
          printf("\n");
        }
      if (N_MATITEM>0)
        { if (N_MATITEM>1) printf("Input matrix items .............. :");
          else printf("Input matrix item ............... :");
          for (j=0;j<N_MATITEM;j++) printf(" %s",objfname.item[i++].name);
          printf("\n");
        }
      if (N_GDOMITEM>0)
        { if (N_GDOMITEM>1) printf("Input gridded domain items ...... :");
          else printf("Input gridded domain item ....... :");
          for (j=0;j<N_GDOMITEM;j++) printf(" %s",objfname.item[i++].name);
          printf("\n");
        }
      if (N_NGDOMITEM>0)
        { if (N_NGDOMITEM>1) printf("Input non-gridded domain items .. :");
          else printf("Input non-gridded domain item ... :");
          for (j=0;j<N_NGDOMITEM;j++) printf(" %s",objfname.item[i++].name);
          printf("\n");
        }
      if (N_OUTTAB>0)
        { if (N_OUTTAB>1) printf("Output tables ................... :");
          else printf("Output table .................... :");
          for (i=0;i<N_OUTTAB;i++) printf(" %s",objfname.outtab[i].name);
          printf("\n");
        }
      if (N_CASEVAR>0)
        { if (N_CASEVAR>1) printf("Case variables .................. :");
          else printf("Case variable ................... :");
          for (i=0;i<N_CASEVAR;i++) printf(" %s",objfname.casevar[i]);
          printf("\n\n");
        }
      printf(RULED_LINE);
      printf("\n");
    }

  if (N_ITEM==0)
    { printf("Running objective function (no input data)\n"); 
      printf("Objective function value = %g\n",
             objfun(NULL,NULL,&status,&errtxt));
      if (status!=OK) exiterror(status,argv[0],errtxt,0);
      exit(EXIT_SUCCESS);
    }


  /* Get GFAn command line input */

  comstate=INIT;
  if (argc>NFIELD_MAX) 
    { printf("Too many command line arguments\n");
      exit(EXIT_FAILURE);
    }
  for (i=0;i<argc;i++) argvec[i]=argv[i]; 
  runoption=STANDARD;
  lognum=-1;
  firstctrl=1;
  echo=0;
  offertemplates=0;

  do 
    { comstate=commandstatus(comstate,&argc,argvec,&runoption,&lognum,
                             &firstctrl,&echo,&offertemplates,&status);
      if (status!=OK) exiterror(status,argv[0],NULL,0);
      if (offertemplates)
        { printf("Make input file templates (y/n)?\n");
          fgets(linebuff1,LINE_MAX,stdin);
          if (linebuff1[0]=='y' || linebuff1[0]=='Y')
            { status=maketemplates(ctrltabvarname,optconfvarname,itemcat,
                       &objfname,keyvarname,gspecvarname,dimflagvarname);
              if (status!=OK) exiterror(status,argv[0],NULL,0);
            }
          else if (linebuff1[0]!='n' && linebuff1[0]!='N')
            { printf("\nTemplate for Control File:\n\n");
              writectrltemplate(stdout,ctrltabvarname,objfname.item);
              printf("\n");
            }
        }
    }
  while (comstate!=GO && comstate!=QUIT);

  if (comstate==QUIT) exit(EXIT_SUCCESS);


  /* Read sequence of control files */

  for (i=firstctrl;i<argc;i++)
    { status=readctrlfile(argvec[i],ctrltabvarname,objfname.item,&ctrlhead,
                          &ctrl,&lineno,&missingvar);
      if (status!=OK)
        { if (status==VARERR)
            { sprintf(linebuff1,"%s\\%s",missingvar,argvec[i]); 
              exiterror(VARERR,argv[0],linebuff1,0); 
            }
          exiterror(status,argv[0],argvec[i],lineno);
        }
    }

  if (ctrlhead==NULL)
    { printf("GFAn terminating - No control records to process\n");
      exit(EXIT_SUCCESS); 
    }
 
  if (echo)

    { /* print details of all analyses and request confirmation */

      num=0;
      ctrl=ctrlhead;
      while (ctrl!=NULL)
        { num++;
          printf("\n");
          printf("----- GFAn EXPERIMENT %d -----\n\n",num);
          writectrlrec(stdout,ctrl,&objfname);
          printf("\n");
          if (ctrl->next) printf("Next (y/n)? > ");
          else 
            { if (num==1) printf("Run (y/n)? > ");
              else printf("Run %d analyses (y/n)? > ",num);
            }
          fgets(linebuff1,LINE_MAX,stdin);
          if (linebuff1[0]!='y' && linebuff1[0]!='Y') exit (EXIT_SUCCESS);
          ctrl=ctrl->next;
        }
      printf("\n");
    }

  /* Set flags to enable steps according to run option */
  
  switch(runoption)
    { case STANDARD:
      case UNKNOWN:
        stepenabled.xref=1;
        stepenabled.check=1;
        stepenabled.opt=1;
        stepenabled.func=1;
        break;
      case SKIPXREF:
        stepenabled.xref=0;
        stepenabled.check=0;
        stepenabled.opt=0;
        stepenabled.func=0;
        break;
      case SKIPCHECK:
        stepenabled.xref=1;
        stepenabled.check=0;
        stepenabled.opt=0;
        stepenabled.func=0;
        break; 
      case SKIPOF:
        stepenabled.xref=1;
        stepenabled.check=1;
        stepenabled.opt=0;
        stepenabled.func=0;
        break; 
      case SKIPO:
        stepenabled.xref=1;
        stepenabled.check=1;
        stepenabled.opt=0;
        stepenabled.func=1;
        break; 
      case SKIPF:
        stepenabled.xref=1;
        stepenabled.check=1;
        stepenabled.opt=1;
        stepenabled.func=0;
        break; 
    }

  /* Start log file */

  logfname=logfnamebuff;
  strcpy(logfname,LOG_FNAME);
  if (lognum>=0) sprintf(logfname+strlen(logfname),".%d",lognum);

  if ((logfile=fopen(logfname,"w"))==NULL)
    exiterror(OPENOUTERR,argv[0],logfname,0);

  if (fprintf(logfile,"%s\n\n%s\n\n",RULED_LINE,title1)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  if (txt1 && fprintf(logfile,"%s\n",txt1)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  if (txt2 && fprintf(logfile,"%s\n",txt2)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  if (txt3 && logflag && fprintf(logfile,"%s\n",txt3)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  if (fprintf(logfile,"%s\n",RULED_LINE)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  if (fprintf(logfile,"%s\n%s\n\n",title2,RULED_LINE)<0)
    exiterror(OUTERR,argv[0],logfname,0);

  if (objfname.description) 
    if (fprintf(logfile,"%s\n\n",objfname.description)<0)
      exiterror(OUTERR,argv[0],logfname,0);
  if (objfname.notes)
    if (fprintf(logfile,"%s\n\n",objfname.notes)<0)
      exiterror(OUTERR,argv[0],logfname,0);

  if (fprintf(logfile,"%s\n",RUNHDR_LINE)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  status=writelog(logfile,0,"PROCESSING GFAn COMMAND:");
  if (status!=OK) exiterror(status,argv[0],logfname,0);
  fprintf(logfile,"\n");
  txt=linebuff1;
  for (i=0;i<argc;i++)
    { sprintf(txt,"%s ",argvec[i]);
      txt+=strlen(txt);
    }
  if (fprintf(logfile,"%s",linebuff1)<0)
    exiterror(OUTERR,argv[0],logfname,0);
  fprintf(logfile,"\n\n");

  /* Set up page buffer for table output */

  for (i=0;i<NPAGEREC_MAX;i++)
    { dpage[i].x=pagebuff.x[i];
      dpage[i].field=pagebuff.field[i];
    }

  /* Initialize master input record:
     Table pointers set to show tables are empty
     All items and item variables are activated by default but can be
     de-activated by the application */

  casetab.ndata=0;
  casetab.table=NULL;
  for (i=0;i<N_ITEM;i++) 
    { itemtab[i].ndata=0; 
      itemtab[i].table=NULL; 
      casetab.itemactive[i]=1;
    }

#if OPT_ENABLED == 1 

  /* Set table pointers for use by optimizer */

  optdata.funcindata=&casetab;
  optdata.parminittab.table=NULL;

#endif

  /* Run any set up code for the objective function and set pointers in master 
     input record for access to data generic to all cases */

  casetab.fixeddata=NULL;

#if OBJF_SETUP_REQ == 1

  status=objfun_setup(&casetab);
  if (status!=OK) exiterror_logged(logfile,status,argv[0],NULL,0); 

#endif

  casetab.objfname=&objfname;
 
  /* If no case dependencies then make a dummy case table */

  if (N_CASEVAR==0) 
    { newcasetab=1; 
      for (i=0;i<N_ITEM;i++) casetab.keyflag[i]=0;
      status=makesinglecase(&casetab); 
      if (status!=OK) exiterror_logged(logfile,status,argv[0],NULL,0); 
    }

  /* Set active variable defaults for output tables */

  for (i=0;i<N_OUTTAB;i++)
    for (j=0;j<objfname.outtab[i].nvar;j++) outvaractive[i][j]=1;

  /* Process control list */

  num=0;
  ctrl=ctrlhead;
  while (ctrl)

    { if (OPT_ENABLED && ctrl->ioptpset>=0) 
        { mode=OPT;
          modename="OPTIMIZING";
          casetab.optpsetnum=ctrl->ioptpset+1;
        }
      else 
        { mode=NONOPT;
          modename="NO OPTIMIZATION";
          casetab.optpsetnum=0;
        }
      num++;

      if (fprintf(logfile,"%s\n",RUNHDR_LINE)<0)
        exiterror(OUTERR,argv[0],logfname,0);
      sprintf(linebuff1,"GFAn EXPERIMENT %d - %s",num,modename);
      status=writelog(logfile,echo,linebuff1);
      if (status!=OK) exiterror(status,argv[0],logfname,0);
      fprintf(logfile,"\n");
      status=writectrlrec(logfile,ctrl,&objfname);
      if (status!=OK) exiterror(status,argv[0],logfname,0);
      fprintf(logfile,"\n");
      if (fflush(logfile)==EOF) exiterror(status,argv[0],logfname,0);

      mainlogfile=logfile;
      mainlogfname=logfname;
      if (ctrl->logf.name)
        { if ((logfile=fopen(ctrl->logf.name,"w"))==NULL)
            exiterror_logged(mainlogfile,OPENOUTERR,argv[0],ctrl->logf.name,0);
          logfname=ctrl->logf.name;
        }


      /* Open files for objective function's output tables
         and write headers */

      if (stepenabled.func) for (i=0;i<N_OUTTAB;i++)

        { if (ctrl->outf[i].name && ctrl->outf[i].flag)

            { if ((outfile[i]=fopen(ctrl->outf[i].name,"w"))==NULL)
                exiterror_logged(logfile,OPENOUTERR,argv[0],
                                 ctrl->outf[i].name,0);

              if (ctrl->outvarf[i].name && !ctrl->outvarf[i].resident)
                { status=readoutvarfile(ctrl->outvarf[i].name,
                           objfname.outtab[i].nvar,objfname.outtab[i].varname,
                           outvaractive[i],&lineno);
                  if (status!=OK)
                    { if (status==OPENINERR)
                        { printf("\nMissing file?\n");
                          printf(
                  "\nTemplate for Output Variables File (output table '%s')\n",
                                 objfname.outtab[i].name);
                          printf(
             "VALUE = 0 de-selects variable, VALUE = 1 selects variable:\n\n");
                          writeoutvartemplate(stdout,objfname.outtab[i].nvar,
                                              objfname.outtab[i].varname,
                                              objfname.outtab[i].vartext);
                          printf("\n");
                        }
                      exiterror_logged(logfile,status,argv[0],
                                       ctrl->outvarf[i].name,lineno);
                    }
                }

              outtabttb[i]=(ctrl->outf[i].flag==2);
              if (!outtabttb[i])
                status=writeouttabvardescrip(i,outfile[i],&objfname.outtab[i],
                                             "#",0);
              if (status!=OK) 
                exiterror_logged(logfile,status,argv[0],ctrl->outf[i].name,0);
              status=writeouttabvar(i,outfile[i],objfname.casevar,
                                    &objfname.outtab[i]);
              if (status!=OK) 
                exiterror_logged(logfile,status,argv[0],ctrl->outf[i].name,0);
            }

          else outfile[i]=NULL;
        }


      /* Read all input files specified which differ (by name) from those used 
         to create the current database */

#if OPT_ENABLED == 1 

      if (mode == OPT)

        { /* Read optimizer configuration file (if any) and set up 
             configuration table ('clearoptconfig' sets defaults) */ 

          if (!ctrl->optconfigf.resident)

            { clearoptconfig(&optdata);

              if (ctrl->optconfigf.name)              

                { status=writelog(logfile,echo,
                                  "Reading optimizer configuration file");
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  status=readoptconfigfile(ctrl->optconfigf.name,optconfvarname,
                                           &optdata,&lineno,&errtxt);
                  if (status==OPENINERR)
                    { printf("\nMissing file?\n");
                      printf(
                        "\nTemplate for Optimizer Configuration File:\n\n");
                      writeoptconfigtemplate(stdout,optconfvarname);
                      printf("\n");
                    }
                  else if (status==SPECIAL)
                    exiterror_logged(logfile,SPECIAL,argv[0],errtxt,0);
                  if (status!=OK)
                    exiterror_logged(logfile,status,argv[0],
                                     ctrl->optconfigf.name,lineno);
                }
            }

          /* Write configuration table to log (even if resident) unless
             explictly suppressed */
           
          if (logfile && (!ctrl->optconfigf.name || ctrl->optconfigf.flag))
                
            { if (ctrl->optconfigf.name) 
                sprintf(linebuff1,"OPTIMIZER CONFIGURATION (file: %s)",
                        ctrl->optconfigf.name);
              else 
                sprintf(linebuff1,"OPTIMIZER CONFIGURATION (default)");
              status=writeheading(logfile,linebuff1);
              if (status!=OK) exiterror(status,argv[0],logfname,0);
              status=writelogoptconfig(logfile,&optdata);
              if (status!=OK) exiterror(status,argv[0],logfname,0);
              writeheading(logfile,NULL);
              if (fflush(logfile)==EOF) exiterror(status,argv[0],logfname,0);


              sprintf(linebuff1,"OPTIMIZABLE PARAMETER SET (%s)",
                      objfname.item[ctrl->ioptpset].name);
              status=writeheading(logfile,linebuff1);
              if (status!=OK) exiterror(status,argv[0],logfname,0);
              for (i=0;i<objfname.item[ctrl->ioptpset].nvar;i++)
                { if (objfname.item[ctrl->ioptpset].vartext[i])
                    { if (fprintf(logfile,"%s: %s\n",
                            objfname.item[ctrl->ioptpset].varname[i],
                            objfname.item[ctrl->ioptpset].vartext[i])<0)
                        exiterror(status,argv[0],logfname,0);
                    }
                  else
                    { if (fprintf(logfile,"%s: --- undescribed parameter ---\n",
                                  objfname.item[ctrl->ioptpset].varname[i])<0)
                        exiterror(status,argv[0],logfname,0);
                    }
                }
              writeheading(logfile,NULL);
              if (fflush(logfile)==EOF) exiterror(status,argv[0],logfname,0);
            }

          /* Read free parameter file and set up free parameter
             control arrays */

          if (!ctrl->freeparmf.resident)

            { clearparmctrlrec(&optdata.freeparm);

              status=writelog(logfile,echo,"Reading free parameter file");
              if (status!=OK) exiterror(status,argv[0],logfname,0);

              status=readfreeparmfile(ctrl->freeparmf.name,
                objfname.parmctrlvar,&objfname.item[ctrl->ioptpset],
                &optdata.freeparm,&lineno,&missingvar);

              if (status!=OK)
                { if (status==OPENINERR)
                    { printf("\nMissing file?\n");
                      printf("\nTemplate for Free Parameter File:\n\n");
                      writefreeparmtemplate(stdout,objfname.parmctrlvar);
                      printf("\n");
                    }
                  else if (status==VARERR)
                    { sprintf(linebuff1,"%s\\%s",missingvar,
                              ctrl->freeparmf.name);
                      exiterror_logged(logfile,VARERR,argv[0],linebuff1,0);
                    }
                  exiterror_logged(logfile,status,argv[0],
                                   ctrl->freeparmf.name,lineno);
                }

              if (logfile && ctrl->freeparmf.flag)

                { sprintf(linebuff1,"FREE PARAMETER TABLE (file: %s)",
                          ctrl->freeparmf.name);
                  status=writeheading(logfile,linebuff1);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  status=writelogfreeparmtab(logfile,&optdata.freeparm,NULL,
                    objfname.parmctrlvar,
                    objfname.item[ctrl->ioptpset].varname);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  writeheading(logfile,NULL);
                  if (fflush(logfile)==EOF) 
                    exiterror(status,argv[0],logfname,0);
                }
            }

          if (stepenabled.opt)

            { /* Open new parameter file and write header */

              if (ctrl->newparmf.name && ctrl->newparmf.flag)
                { if ((newparmfile=fopen(ctrl->newparmf.name,"w"))==NULL)
                    exiterror_logged(logfile,OPENOUTERR,argv[0],
                                     ctrl->newparmf.name,0);
                  newparmttb=(ctrl->newparmf.flag==2);
                  status=writenewparmvar(newparmfile,&optdata.freeparm,
                                         objfname.item[ctrl->ioptpset].varname);
                  if (status!=OK) 
                    exiterror_logged(logfile,status,argv[0],
                                     ctrl->newparmf.name,0);
                }
              else newparmfile=NULL;

              /* Open function evaluation file(s) and write header(s) */

              if (ctrl->funcgaf.name && ctrl->funcgaf.flag)
                { if ((funcgafile=fopen(ctrl->funcgaf.name,"w"))==NULL)
                    exiterror_logged(logfile,OPENOUTERR,argv[0],
                                     ctrl->funcgaf.name,0);
                  funcgattb=(ctrl->funcgaf.flag==2);
                  status=writeobjfevalvar_ga(funcgafile,&optdata.freeparm,
                           objfname.item[ctrl->ioptpset].varname);
                  if (status!=OK) 
                    exiterror_logged(logfile,status,argv[0],
                                     ctrl->funcgaf.name,0);
                }
              else funcgafile=NULL; 
            
              if (ctrl->funcpowellf.name && ctrl->funcpowellf.flag)
                { if ((funcpowellfile=fopen(ctrl->funcpowellf.name,"w"))==NULL)
                    exiterror_logged(logfile,OPENOUTERR,argv[0],
                                     ctrl->funcpowellf.name,0);
                  funcpowellttb=(ctrl->funcpowellf.flag==2);
                  status=writeobjfevalvar_powell(funcpowellfile,
                           &optdata.freeparm,
                           objfname.item[ctrl->ioptpset].varname);
                  if (status!=OK) 
                    exiterror_logged(logfile,status,argv[0],
                                     ctrl->funcpowellf.name,0);
                }
              else funcpowellfile=NULL;

              /* Open optimizer report file */

              if (ctrl->optreportf.name && ctrl->optreportf.flag)
                { if ((optreportfile=fopen(ctrl->optreportf.name,"w"))==NULL)
                    exiterror_logged(logfile,OPENOUTERR,argv[0],
                                     ctrl->optreportf.name,0);
                }
              else optreportfile=NULL;
            }

          /* Read free parameter initialization file and set up
             parameter initial value table */

          if (!ctrl->parminitf.resident)

            { clearpsettab(&optdata.parminittab);  

              if (ctrl->parminitf.name)

                { status=writelog(logfile,echo,
                                  "Reading parameter initialization file");
                  if (status!=OK) exiterror(status,argv[0],logfname,0);

                  status=readparminitfile(ctrl->parminitf.name,
                           &objfname.item[ctrl->ioptpset],
                           &optdata.parminittab,&lineno);

                  if (status!=OK)
                    { if (status==OPENINERR)
                        { printf("\nMissing file?\n");
                          printf(
                           "\nTemplate for Parameter Initialization File:\n\n");
                          writeparminittemplate(stdout,
                                                &objfname.item[ctrl->ioptpset]);
                          printf("\n");
                        }
                      exiterror_logged(logfile,status,argv[0],
                                       ctrl->parminitf.name,lineno);
                    }

                  if (logfile && ctrl->parminitf.flag)

                    { sprintf(linebuff1,"FREE PARAMETER INIT. TABLE (file: %s)",
                              ctrl->parminitf.name);
                      status=writeheading(logfile,linebuff1);
                      if (status!=OK) exiterror(status,argv[0],logfname,0);
                      status=writelogparminittab(logfile,&optdata.parminittab,
                               objfname.item[ctrl->ioptpset].varname);
                      if (status!=OK) exiterror(status,argv[0],logfname,0);
                      writeheading(logfile,NULL);
                      if (optdata.gactrl.maxgen>0 && !optdata.gactrl.psetini)
                        fprintf(logfile,
              "*** WARNING: initialization data not used in this run ***\n\n");
                      if (fflush(logfile)==EOF) 
                        exiterror(status,argv[0],logfname,0);
                    }
                }
            }

          /* Find indices of free parameter values in initialization data */

          if (optdata.parminittab.ndata>0 && 
              (optdata.gactrl.maxgen==0 || optdata.gactrl.psetini))

            { for (i=0;i<optdata.freeparm.nparm;i++)

                { freeparmno=optdata.freeparm.parmno[i]; 
                  found=0;
                  j=0;
                  while (!found && j<optdata.parminittab.nparm)
                    { if (optdata.parminittab.parmno[j]==freeparmno)
                        { optdata.freeparm.parminitindex[i]=j;
                          found=1;
                        }
                      j++;
                    }

                  if (!found)
                    { sprintf(linebuff1,
                        "missing initialization data for free parameter %s",
                        objfname.item[ctrl->ioptpset].varname[freeparmno-1]);
                      exiterror_logged(logfile,SPECIAL,argv[0],linebuff1,0);
                    }
                }            
            } 

          /* If parameter initialization data to be used for GA then check
             against population size and check values are in range */

          if (optdata.gactrl.maxgen>0 && optdata.gactrl.psetini)

            { if (optdata.parminittab.ndata==0) 
                exiterror_logged(logfile,SPECIAL,argv[0],
                  "no parameter sets for explicit G.A. initialization",0);
              if (optdata.parminittab.ndata<optdata.gactrl.npopsiz)
                exiterror_logged(logfile,SPECIAL,argv[0],
                  "no. of initialization records < G.A. population size",0);

              for (i=0;i<optdata.gactrl.npopsiz;i++)
                { for (j=0;j<optdata.freeparm.nparm;j++)
                    { k=optdata.freeparm.parminitindex[j];
                      if (optdata.parminittab.table[i].parm[k]
                            <=optdata.freeparm.min[j] ||
                          optdata.parminittab.table[i].parm[k]
                            >=optdata.freeparm.max[j])
                        { freeparmno=optdata.freeparm.parmno[j];
                          sprintf(linebuff1,
        "G.A. init. data not within bounds for free parameter %s (psetkey=%d)",
                            objfname.item[ctrl->ioptpset].varname[freeparmno-1],
                            optdata.parminittab.table[i].psetno);
                          exiterror_logged(logfile,SPECIAL,argv[0],linebuff1,0);
                        }
                    }
                } 
            }

          /* If parameter initialization data to be used for Powell then
             check values for finite range parameters are in range */ 

          if (optdata.gactrl.maxgen==0)
            for (i=0;i<optdata.parminittab.ndata;i++)
              { for (j=0;j<optdata.freeparm.nparm;j++)
                  { k=optdata.freeparm.parminitindex[j];
                    if (optdata.freeparm.finitepowell[j] && 
                        (optdata.parminittab.table[i].parm[k]
                           <=optdata.freeparm.min[j] ||
                          optdata.parminittab.table[i].parm[k]
                            >=optdata.freeparm.max[j]))
                      { freeparmno=optdata.freeparm.parmno[j];
                        sprintf(linebuff1,
      "Powell init. data not within bounds for free parameter %s (psetkey=%d)",
                          objfname.item[ctrl->ioptpset].varname[freeparmno-1],
                          optdata.parminittab.table[i].psetno);
                        exiterror_logged(logfile,SPECIAL,argv[0],linebuff1,0);
                      }
                  }
              }

#if OBJF_FREEPARMCHECK_REQ == 1

          /* Perform objective function specific checks on free parameter table
             and/or initial value table */

            if (objfun_freeparmcheck(&optdata.freeparm,&optdata.parminittab,
                objfname.item[ctrl->ioptpset].varname,&errtxt)!=OK)
            exiterror_logged(logfile,SPECIAL,argv[0],errtxt,0);

#endif    
        }

#endif

 
      /* Read case file and case keys file and set up master input table */

      if (N_CASEVAR>0 && !ctrl->casef.resident)

        { newcasetab=1; 
          clearcasetab(&casetab); 

          if (ctrl->casef.name)

            { status=writelog(logfile,echo,"Reading case file");
              if (status!=OK) exiterror(status,argv[0],logfname,0);

              status=readcasefile(ctrl->casef.name,objfname.casevar,keyvarname,
                &casetab,&lineno,&missingvar);

              if (status!=OK) 
                { if (status==OPENINERR)
                    { printf("\nMissing file?\n");
                      printf("\nTemplate for Case File:\n\n");
                      writecasetemplate(stdout,objfname.casevar,keyvarname);
                      printf("\n");
                    }
                  else if (status==VARERR)
                    { sprintf(linebuff1,"%s\\%s",missingvar,ctrl->casef.name);
                      exiterror_logged(logfile,VARERR,argv[0],linebuff1,0);
                    }
                  exiterror_logged(logfile,status,argv[0],ctrl->casef.name,
                                   lineno);
                }

              if (logfile && ctrl->casef.flag)

                { sprintf(linebuff1,"CASE TABLE (file: %s)",
                          ctrl->casef.name);
                  status=writeheading(logfile,linebuff1);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  status=writelogcasetab(logfile,&casetab,objfname.casevar,
                                         keyvarname,NULL);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  writeheading(logfile,NULL);
                  if (fflush(logfile)==EOF) 
                    exiterror(status,argv[0],logfname,0);
                }
            }

          if (casetab.ndata==0)
            { status=makesinglecase(&casetab);
              if (status!=OK) exiterror_logged(logfile,status,argv[0],NULL,0);
            }
        }

      /* Set item flags in master input table for optional items */

      for (i=0;i<N_ITEM;i++) if (objfname.item[i].optional)
        casetab.itemactive[i]=(ctrl->itemf[i].name!=NULL);

      /* Read objective function input item files 
         (skip missing items or those already resident) */ 

      for (i=0;i<N_ITEM;i++) if (!ctrl->itemf[i].resident)

        { newitemtab[i]=1; 
          clearitemtab(&itemtab[i]);

          if (casetab.itemactive[i])

            { /* Read item specification file and set up item table */

              sprintf(linebuff1,"Reading item '%s'",objfname.item[i].name);
              status=writelog(logfile,echo,linebuff1);
              if (status!=OK) exiterror(status,argv[0],logfname,0);
 
              status=readitemfile(ctrl->itemf[i].name,itemcat[i],
                objfname.casevar,keyvarname[i],&objfname.item[i],gspecvarname,
                dimflagvarname,&itemtab[i],&lineno,&missingvar);
              if (status!=OK)
                { if (status==OPENINERR)
                    { printf("\nMissing file?\n"); 
                      printf("\nTemplate for Item File (item '%s'):\n\n",
                             objfname.item[i].name);
                      writeitemtemplate(stdout,itemcat[i],objfname.casevar,
                        keyvarname[i],&objfname.item[i],gspecvarname,
                        dimflagvarname);
                      printf("\n");
                    }
                  else if (status==VARERR)
                    { sprintf(linebuff1,"%s\\%s",missingvar,
                              ctrl->itemf[i].name);
                      exiterror_logged(logfile,VARERR,argv[0],linebuff1,0);
                    }
                  exiterror_logged(logfile,status,argv[0],ctrl->itemf[i].name,
                                   lineno);
                }

              if (logfile && ctrl->itemf[i].flag)

                { sprintf(linebuff1,"ITEM %s (file: %s)",
                          objfname.item[i].name,ctrl->itemf[i].name);
                  status=writeheading(logfile,linebuff1);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  status=writelogitemtab(logfile,&itemtab[i],itemnumvarname[i],
                    &objfname.item[i],gspecvarname,dimflagvarname);
                  if (status!=OK) exiterror(status,argv[0],logfname,0);
                  if (ctrl->itemf[i].flag==1 || itemcat[i]==PSET) 
                    writeheading(logfile,NULL);
                  if (fflush(logfile)==EOF) 
                    exiterror(status,argv[0],logfname,0);
                }

              /* Read item data file */

              if (itemcat[i]!=PSET)

                { lineno=0;

                  /* First try .dat file */

                  sprintf(linebuff1,"%s.dat",ctrl->itemf[i].name); 
                  status=readitemdatafile(linebuff1,itemcat[i],
                    &objfname.item[i],objfname.dimension,&itemtab[i],&lineno,
                    &errtxt);

                  if (status==OPENINERR)

                    { /* .dat file not found so try .dat.ttb file */

                      sprintf(linebuff1,"%s.dat.ttb",ctrl->itemf[i].name); 
                      status=readitemdatafile(linebuff1,itemcat[i],
                        &objfname.item[i],objfname.dimension,&itemtab[i],
                        &lineno,&errtxt);

                      if (status==OPENINERR)

                        { if (itemcat[i]!=GDOMITEM || NETCDF_ENABLED == 0) 

                            sprintf(linebuff1,"%s.dat OR %s.dat.ttb",
                                              ctrl->itemf[i].name,
                                              ctrl->itemf[i].name);
                     
#if NETCDF_ENABLED == 1

                          else 

                            { /* .dat.ttb file not found so try .nc file */

                              sprintf(linebuff1,"%s.nc",ctrl->itemf[i].name);
                              status=readitemdata_nc(linebuff1,1,itemcat[i],
                                &objfname.item[i],objfname.dimension,NULL,
                                objfname.item[i].varname,&itemtab[i],
                                OBJF_NODATA,NULL,NULL,NULL,NULL,logfile,
                                &errtxt);

                              if (status==OPENINERR)

                                { /* .nc file not found so try .nct file */
 
                                  sprintf(linebuff1,"%s.nct",
                                          ctrl->itemf[i].name);
                                  status=readitemdata_nct(linebuff1,itemcat[i],
                                    &objfname.item[i],objfname.dimension,
                                    &itemtab[i],logfile,&ncerrfile,&lineno,
                                    &errtxt);

                                  if (ncerrfile) 
                                    sprintf(linebuff1,"%s",ncerrfile);

                                  else if (status==OPENINERR)

                                    { sprintf(linebuff1,
                                              "%s.dat OR %s.nc OR %s.nct",
                                              ctrl->itemf[i].name,
                                              ctrl->itemf[i].name,
                                              ctrl->itemf[i].name);
                                    }
                                }

                              /* Common error processing for .nc or .nct read */

                              if (status==OUTERR) 
                                exiterror(status,argv[0],logfname,0);
                            }
#endif
                      }
                    }

                  /* Common error processing for .dat, .dat.ttb, .nc or .nct 
                     read */
                        
                  if (status!=OK)

                    { if (status==OPENINERR)
                        { printf("\nMissing item data file?\n");
                          printf("\nData File Variable/s (item '%s'):\n\n",
                                 objfname.item[i].name);
                          writeitemdatavariables(stdout,itemcat[i],
                            &objfname.item[i],objfname.dimension,&itemtab[i]);
                          printf("\n");
                          exiterror_logged(logfile,status,argv[0],linebuff1,0);
                        }
                      if (status==VARERR)
                        { sprintf(linebuff2,"%s\\%s",errtxt,linebuff1);
                          exiterror_logged(logfile,VARERR,argv[0],linebuff2,0);
                        }
                      if (status==SPECIAL)
                        { sprintf(linebuff2,"%s (file: %s)",errtxt,linebuff1);
                          exiterror_logged(logfile,SPECIAL,argv[0],linebuff2,0);
                        }
                      exiterror_logged(logfile,status,argv[0],linebuff1,lineno);
                    }

                  /* Write item data to log file */
              
                  if (logfile && ctrl->itemf[i].flag)

                    { if (ctrl->itemf[i].flag>2) logrecmax=-1;
                      else if (ctrl->itemf[i].flag==2) 
                        logrecmax=NRECMAX_LOGINSTANCE;
                      else logrecmax=0;
 
                      if (fprintf(logfile,
                                  "\n--- DATA FOR ITEM %s (file: %s) ---\n\n",
                                  objfname.item[i].name,linebuff1)<0)
                        exiterror(OUTERR,argv[0],logfname,0);
                      status=writelogitemdatatab(logfile,logrecmax,&itemtab[i],
                        itemnumvarname[i],&objfname.item[i],objfname.dimension);
                      if (status!=OK) exiterror(status,argv[0],logfname,0);
                      writeheading(logfile,NULL);
                      if (fflush(logfile)==EOF)
                        exiterror(status,argv[0],logfname,0);
                    }

                  /* Write item data to new .dat file */

                  if (ctrl->itemf[i].createfile)

                    { ttb=(ctrl->itemf[i].createfile==2);
                      
                      if (ttb) 
                        sprintf(linebuff1,
                                "gfan.item.%s.dat.ttb",objfname.item[i].name);
                      else
                        sprintf(linebuff1,
                                "gfan.item.%s.dat",objfname.item[i].name);
                      found=(stat(linebuff1,&statbuff)!=-1);

                      for (j=0; found && j<100; j++)

                        { if (ttb)
                            sprintf(linebuff1,
                                    "gfan.item.%s-%02d.dat.ttb",
                                    objfname.item[i].name,j);
                          else
                            sprintf(linebuff1,
                                    "gfan.item.%s-%02d.dat",
                                    objfname.item[i].name,j);
                          found=(stat(linebuff1,&statbuff)!=-1);
                        }

                      if (j==100 && found)
                        { j=strlen(linebuff1)-6-4*ttb; 
                          linebuff1[j++]='?';
                          linebuff1[j]='?'; 
                          exiterror_logged(logfile,OPENOUTERR,argv[0],
                                           linebuff1,0);
                        }

                      if ((f=fopen(linebuff1,"w"))==NULL)
                        exiterror_logged(logfile,OPENOUTERR,argv[0],
                                         linebuff1,0);

                      sprintf(linebuff2,"Writing item '%s' data -> %s",
                              objfname.item[i].name,linebuff1);
                      status=writelog(logfile,echo,linebuff2);
                      if (status!=OK) exiterror(status,argv[0],logfname,0);

                      status=writeitemdatatab(f,ttb,&itemtab[i],
                                              itemnumvarname[i],
                                              &objfname.item[i],
                                              objfname.dimension);
                      if (status!=OK) 
                        exiterror_logged(logfile,status,argv[0],linebuff1,0);

                      fclose(f);
                    }
                }
            }
        }

      if (stepenabled.xref)
 
        { /* Update cross-references in the master input table according to the
             case dependencies implied by the case/key variables for each item. 
             If the case table has changed then all cross-references must be
             updated, rather than just those for the new files.
             (Missing items are skipped.) 

             Only log a dummy case table if there are multiple item instances 
             (This is possible if item includes a default case) */

          logflag=0;
          xrefstatus=OK;
          for (i=0;i<N_ITEM && xrefstatus==OK;i++) 
            if (newcasetab || newitemtab[i])
              { if (casetab.itemactive[i])
                  { if (!logflag) 
                      { if (!ctrl->casef.name)
                          { for (j=0;j<N_ITEM;j++) 
                              if (itemtab[j].ndata>1) logflag=1; 
                          }
                        else logflag=1;
                        if (logflag)
                          { status=writelog(logfile,echo,"Cross-referencing");
                            if (status!=OK)
                              exiterror(status,argv[0],logfname,0);
                          } 
                      }
                    xrefstatus=crossref(&casetab,i,&itemtab[i],&errcasenum); 
                    if (xrefstatus!=OK) erritemtype=i;
                    newitemtab[i]=0;
                  }
                else
                  { for (j=0;j<casetab.ndata;j++) 
                      { casetab.table[j].itemnum[i]=0;
                        casetab.table[j].item[i]=NULL;
                      }
                  }
              }

          newcasetab=0;

          if (logfile && logflag)

            { if (ctrl->casef.name) 
                sprintf(linebuff1,"LINKED CASE TABLE (file: %s)",
                        ctrl->casef.name);
              else
                sprintf(linebuff1,"LINKED CASE TABLE (no input file)");
              status=writeheading(logfile,linebuff1);
              if (status!=OK) exiterror(status,argv[0],logfname,0);

              if (xrefstatus==OK) 
                status=writelogcasetab(logfile,&casetab,
                                       objfname.casevar,keyvarname,
                                       itemnumvarname);
              else
                status=writelogcasetab(logfile,&casetab,
                                       objfname.casevar,keyvarname,NULL);
              if (status!=OK) exiterror(status,argv[0],logfname,0);
              writeheading(logfile,NULL);
              if (fflush(logfile)==EOF) exiterror(status,argv[0],logfname,0);
            }

          if (xrefstatus!=OK)
            { sprintf(linebuff1,
                "cross-referencing failed for item '%s' at case number %d",
                      objfname.item[erritemtype].name,errcasenum);
              if (logfile) fprintf(logfile,"*** Error: %s ***\n",linebuff1);
              exiterror_logged(logfile,xrefstatus,argv[0],linebuff1,0);
            }
        }


#if OBJF_INPUTCHECK_REQ == 1

      if (stepenabled.check)

        { /* Perform objective function specific checks on input data
             referenced via case table */

          status=writelog(logfile,echo,
                          "Performing objective function input checks");
          if (status!=OK) exiterror(status,argv[0],logfname,0);
          if (objfun_inputcheck(&casetab,&errtxt)!=OK) 
            exiterror_logged(logfile,SPECIAL,argv[0],errtxt,0);
        }

#endif


#if OPT_ENABLED == 1 

      /* Optimize free parameters (if required) */
     
      if (mode == OPT && stepenabled.opt) 

        { /* Save original data from parameter set item table */

          origparm=saveparmitem(&itemtab[ctrl->ioptpset],
                                &objfname.item[ctrl->ioptpset]);
          if (!origparm) exiterror_logged(logfile,MEMERR,argv[0],NULL,0);

          /* Call optimizer */

          status=optimizeparameters(logfile,echo,linebuff1,optreportfile,
                   funcgafile,funcpowellfile,newparmfile,ctrl->ioptpset,
                   &optdata,&objfname,&errtxt);
          if (status!=OK) exiterror_logged(logfile,status,argv[0],errtxt,0);

          if (optreportfile) fclose (optreportfile);
          if (funcgafile) fclose (funcgafile);
          if (funcpowellfile) fclose (funcpowellfile);
          if (newparmfile) fclose (newparmfile);
        }

#endif


      if (stepenabled.func)

        { /* Evaluate objective function */
  
          status=writelog(logfile,echo,"Running objective function");
          if (status!=OK) exiterror(status,argv[0],logfname,0);
          sprintf(linebuff1,"Objective function value = %.*g\n",
                  OBJF_SIGDIG_DEF,objfun(&casetab,outfile,&status,&errtxt));
          if (status!=OK) exiterror_logged(logfile,status,argv[0],errtxt,0);
          status=writelog(logfile,echo,linebuff1);
          if (status!=OK) exiterror(status,argv[0],logfname,0);

          /* Close output files */

          for (i=0;i<N_OUTTAB;i++) 
            if (outfile[i]) fclose(outfile[i]);
          if (logfile!=mainlogfile) fclose(logfile);
        }


#if OPT_ENABLED == 1 

      /* Restore original data to parameter set item table for optimized 
         parameter set */ 

      if (mode==OPT && stepenabled.opt) 
        restoreparmitem(origparm,&itemtab[ctrl->ioptpset],
                        &objfname.item[ctrl->ioptpset]);

#endif


      /* Get next control record */

      ctrlhead=ctrl->next;
      deletectrlrec(ctrl);
      ctrl=ctrlhead;
      logfile=mainlogfile;
      logfname=mainlogfname;
    }

  status=writelog(logfile,echo,"GFAn completed");
  if (status!=OK) exiterror(status,argv[0],logfname,0);

  return EXIT_SUCCESS;
}


static enum comstates commandstatus (enum comstates instate, int *argc, 
                                     char *argvec[], enum runoptions *opt,
                                     int *lognum, int *firstctrl, int *echo,
                                     int *offertemplates, int *status)

{ static char argbuff[LINE_MAX+1];
  enum comstates comstate;
  int i;

  switch(instate)

    { case INIT:    /* Initial command line */

        if (*argc<2) comstate=NOARG_INI;
        else
          { *opt=getoption(argvec[1]);
            if (*opt==UNKNOWN) comstate=NOARG;
            else if (*opt!=STANDARD)
              { (*firstctrl)++;
                if (*argc<3) comstate=HAVEOPTION;
                else 
                  { *lognum=getlognum(argvec[2]);
                    if (*lognum>=0) (*firstctrl)++; 
                    if (*lognum>=0 && *argc<4) comstate=HAVELOGNUM;
                    else comstate=GO;
                  } 
              }   
            else
              { *lognum=getlognum(argvec[1]);
                if (*lognum>=0) (*firstctrl)++; 
                if (*lognum>=0 && *argc<3) comstate=HAVELOGNUM;
                else comstate=GO;
              } 
          }  
 
        if (comstate==GO) *echo=0;
        else 
          { *echo=1;
            printf("Usage: %s [-x|-c|-s|-o|-f] [log_file_no] control_file_1 [control_file_2 ...]\n",
                   argvec[0]);
          }

        break;

      case NOARG: case NOARG_INI:    /* No valid arguments processed */ 

        if (*argc<2)
          { if (instate==NOARG_INI) *offertemplates=1;
            comstate=QUIT; 
          }
        else
          { *opt=getoption(argvec[1]);
            if (*opt==UNKNOWN) comstate=NOARG;
            else if (*opt!=STANDARD)
              { (*firstctrl)++;
                if (*argc<3)
                  { printf("No control files\n");
                    comstate=QUIT;
                  }
                else
                  { *lognum=getlognum(argvec[2]);
                    if (*lognum>=0) (*firstctrl)++; 
                    if (*lognum>=0 && *argc<4)
                      { printf("No control files\n");
                        comstate=QUIT;
                      }
                    else comstate=GO;
                  }
              }
            else
              { *lognum=getlognum(argvec[1]);
                if (*lognum>=0) (*firstctrl)++; 
                if (*lognum>=0 && *argc<3)
                  { printf("No control files\n");
                    comstate=QUIT;
                  }
                else comstate=GO;
              }
          }
        break;

      case HAVEOPTION:    /* Valid option argument found */

        if (*argc<3)
          { printf("No control files\n");
            comstate=QUIT;
          }
        else
          { *lognum=getlognum(argvec[2]);
            if (*lognum>=0) (*firstctrl)++; 
            if (*lognum>=0 && *argc<4)
              { printf("No control files\n");
                comstate=QUIT;
              }
            else comstate=GO;
          }
        break;

      case HAVELOGNUM:     /* Valid log file number found */

        if (*argc<3)
          { printf("No control files\n");
            comstate=QUIT;
          }
        else comstate=GO;
        break;
    }

  if (comstate==NOARG) *argc=1; /* Remove arguments if invalid option found */

  *status=OK;
  if (comstate!=GO && comstate!=QUIT)
    { printf(">%s ",argvec[0]);
      for (i=1;i<*argc;i++) printf("%s ",argvec[i]);
      *status=getfieldlist(stdin,argbuff,&argvec[i],argc);
      *argc+=i;
    }

  return comstate;
}


enum runoptions getoption (char *s)

{ if (*s!='-') return STANDARD;

  switch(*(++s))

    { case 'x':  
        return SKIPXREF; 
      case 'c':  
        return SKIPCHECK; 

#if OPT_ENABLED == 1

      case 's':  
        return SKIPOF; 
      case 'o':  
        return SKIPO; 

#endif

      case 'f':  
        return SKIPF; 
      default:      
        printf("Valid options:\n");
        printf(" -x to skip cross-referencing & subsequent processing\n");
        printf(
      " -c to skip objective function's data checks & subsequent processing\n");

#if OPT_ENABLED == 1

        printf(" -s to skip parameter optimization & function evaluation\n");
        printf(" -o to skip parameter optimization\n");

#endif

        printf(" -f to skip function evaluation\n");
        return UNKNOWN;
    }
}


int getlognum (char *s)

{ int lognum,numerr;

  lognum=sgetnum(s,&numerr);
  if (numerr || lognum<0) lognum=-1; 

  return lognum;
}


static int maketemplates (char *ctrltabvarname[], char *optconfvarname[], 
                          enum itemcategory itemcat[],
                          struct objfnamerec *objfname, char *keyvarname[],
                          struct gspecvarnamerec gspecvarname[],
                          char *dimflagvarname[])

{ char fname[LINE_MAX+1];
  FILE *outfile;
  int i,j,status;

  printf("\nTemplate files written:\n\n"); 

  i=1;  /* skip gfanlogf */

#if OPT_ENABLED == 1

  i++;  /* skip optpset */

  sprintf(fname,"gfan.template.%s",ctrltabvarname[i]);
  if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
  status=writeoptconfigtemplate(outfile,optconfvarname);
  if (status!=OK) return status;
  fclose(outfile);
  printf(" %s\n",fname);
  i++;

  sprintf(fname,"gfan.template.%s",ctrltabvarname[i]);
  if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
  status=writefreeparmtemplate(outfile,objfname->parmctrlvar);
  if (status!=OK) return status;
  fclose(outfile);
  printf(" %s\n",fname);
  i++;

  for (j=0;j<N_PSET;j++)
    { sprintf(fname,"gfan.template.%s.%s",ctrltabvarname[i],
              objfname->item[j].name);
      if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
      status=writeparminittemplate(outfile,&objfname->item[j]);
      if (status!=OK) return status;
      fclose(outfile);
      printf(" %s\n",fname);
    }

  i+=5;  /* skip funcgaf,funcpowellf,parmnewf,reportf */

#endif

  for (j=0;j<N_ITEM;j++)

    { sprintf(fname,"gfan.template.%s",ctrltabvarname[i]); 
      if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
      status=writeitemtemplate(outfile,itemcat[j],objfname->casevar,
        keyvarname[j],&objfname->item[j],gspecvarname,dimflagvarname);
      if (status!=OK) return status;
      fclose(outfile);
      printf(" %s\n",fname);

      if (itemcat[j]!=PSET)
        { sprintf(fname,"gfan.template.%sdataf",objfname->item[j].name); 
          if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
          writeitemdatatemplate(outfile,itemcat[j],objfname->casevar,
            keyvarname[j],&objfname->item[j],objfname->dimension);
          if (status!=OK) return status;
          fclose(outfile);
          printf(" %s\n",fname);
        }

      i++;
    }

  if (N_CASEVAR>0) 
    { sprintf(fname,"gfan.template.%s",ctrltabvarname[i]);
      if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
      status=writecasetemplate(outfile,objfname->casevar,keyvarname);
      if (status!=OK) return status;
      fclose(outfile);
      printf(" %s\n",fname);
      i++;
    }

  i+=N_OUTTAB;  /* skip output tables */

  for (j=0;j<N_OUTTAB;j++) 
    { sprintf(fname,"gfan.template.%s",ctrltabvarname[i]);
      if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
      status=writeoutvartemplate(outfile,objfname->outtab[j].nvar,
                                 objfname->outtab[j].varname,
                                 objfname->outtab[j].vartext);
      if (status!=OK) return status;
      fclose(outfile);
      printf(" %s\n",fname);
      i++;
    }

  sprintf(fname,"gfan.template.ctrlf");
  if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
  status=writectrltemplate(outfile,ctrltabvarname,objfname->item);
  if (status!=OK) return status;
  fclose(outfile);
  printf(" %s\n",fname);

  if (objfname->notes) 
    { sprintf(fname,"gfan.template.README");
      if ((outfile=fopen(fname,"w"))==NULL) return OPENOUTERR;
      if (fprintf(outfile,"%s\n\n",objfname->notes)<0) return OUTERR;
      fclose(outfile);
      printf(" %s\n",fname);
    }

  return OK;
}


static int setnames (struct objfnamerec *objfname, char *ctrltabvarname[],
                     char *keyvarname[], char *itemnumvarname[], 
                     struct gspecvarnamerec gspecvarname[], 
                     char *dimflagvarname[], char **errtxt)

{ char *p;
  int len,i,j;

  if (!objfname->function)
    { *errtxt="missing function name - check objfun_setnames";
      return SPECIAL;
    }
  if (!objfname->function)
    { *errtxt="missing version name - check objfun_setnames";
      return SPECIAL;
    }

#if OPT_ENABLED == 1 

  for (i=0;i<N_PARMCTRLVAL;i++) 
    { if (!objfname->parmctrlvar[i])
        { *errtxt=
"missing parm control variable name\n - check objfun_setnames against objfun.h";
          return SPECIAL;
        }
    }

#endif

  len=0;
  for (j=0;j<N_ITEM;j++) 
    { if (!objfname->item[j].name) 
        { *errtxt=
            "missing item name\n - check objfun_setnames against objfun.h";
          return SPECIAL;
        }
      len+=3*strlen(objfname->item[j].name)+10;
    }
  for (j=0;j<N_OUTTAB;j++) 
    { if (!objfname->outtab[j].name) 
        { *errtxt=
        "missing output table name\n - check objfun_setnames against objfun.h";
          return SPECIAL; 
        }
      len+=strlen(objfname->outtab[j].name)+2;
      len+=strlen(objfname->outtab[j].name)+5;
    }
  for (j=0;j<N_DIM;j++) 
    { if (!objfname->dimension[j]) 
        { *errtxt=
            "missing dimension name\n - check objfun_setnames against objfun.h";
          return SPECIAL;
        }
      len+=7*strlen(objfname->dimension[j])+24;  
    }
  p=(char *)malloc(len);
  if (!p) return MEMERR; 

  i=0;
  ctrltabvarname[i++]="gfanlogf";

#if OPT_ENABLED == 1 

  ctrltabvarname[i++]="optpset";
  ctrltabvarname[i++]="optconfigf";
  ctrltabvarname[i++]="freeparmf";
  ctrltabvarname[i++]="parminitf";
  ctrltabvarname[i++]="funcgaf";
  ctrltabvarname[i++]="funcpowellf";
  ctrltabvarname[i++]="parmnewf";
  ctrltabvarname[i++]="reportf";

#endif

  for (j=0;j<N_ITEM;j++)
    { ctrltabvarname[i++]=p; 
      sprintf(p,"%sf",objfname->item[j].name);
      p+=strlen(p)+1;
      keyvarname[j]=p;
      sprintf(p,"%skey",objfname->item[j].name);
      p+=strlen(p)+1;
      itemnumvarname[j]=p;
      sprintf(p,"%snum",objfname->item[j].name);
      p+=strlen(p)+1;
    }

  if (N_CASEVAR>0) ctrltabvarname[i++]="casef";

  for (j=0;j<N_OUTTAB;j++)
    { ctrltabvarname[i++]=p; 
      sprintf(p,"%sf",objfname->outtab[j].name);
      p+=strlen(p)+1;
    }

  for (j=0;j<N_OUTTAB;j++)
    { ctrltabvarname[i++]=p; 
      sprintf(p,"%svarf",objfname->outtab[j].name);
      p+=strlen(p)+1;
    }

  for (i=0;i<N_DIM;i++)
    { gspecvarname[i].first=p;
      sprintf(p,"%s1",objfname->dimension[i]);
      p+=strlen(p)+1;
      gspecvarname[i].last=p;
      sprintf(p,"%sn",objfname->dimension[i]);
      p+=strlen(p)+1;
      gspecvarname[i].step=p;
      sprintf(p,"%sstep",objfname->dimension[i]);
      p+=strlen(p)+1;
      gspecvarname[i].n=p;
      sprintf(p,"n%s",objfname->dimension[i]);
      p+=strlen(p)+1;
      gspecvarname[i].min=p;
      sprintf(p,"%smin",objfname->dimension[i]);
      p+=strlen(p)+1;
      gspecvarname[i].max=p;
      sprintf(p,"%smax",objfname->dimension[i]);
      p+=strlen(p)+1;
      dimflagvarname[i]=p;
      sprintf(p,"%sflag",objfname->dimension[i]);
      p+=strlen(p)+1;
    }

  return OK;
}


static int writectrltemplate (FILE *outfile, char *varname[],
                              struct itemnamerec item[])

{ int optional[N_CTRLTABVAR];
  int nvar,i;

  if (!outfile) return OK;

nvar=0;
  optional[nvar++]=1;
#if OPT_ENABLED == 1 
  while (nvar<9) optional[nvar++]=1; 
#endif
  for (i=0;i<N_ITEM;i++) 
    { if (item[i].optional) optional[nvar++]=1;
      else optional[nvar++]=0;
    }
  if (N_CASEVAR>0) optional[nvar++]=1;
  for (i=0;i<2*N_OUTTAB;i++) optional[nvar++]=1;

  return writetemplate(outfile,nvar,optional,varname,NULL,NULL);
}


static int writeoutvartemplate (FILE *outfile, int nvar, char *varname[],
                                char *vartext[]) 
    
{ int optional[NFIELD_MAX];
  int i;

  if (!outfile) return OK;

  for (i=0;i<nvar;i++) optional[i]=1;
  return writetemplate(outfile,nvar,optional,varname,vartext,"1");
} 


static int writecasetemplate (FILE *outfile, char *casevarname[], 
                              char *keyvarname[])

{ int optional[N_CASEVAR+N_ITEM];
  char *varname[N_CASEVAR+N_ITEM];
  int nvar,i;

  if (!outfile) return OK;

  nvar=0;
  for (i=0;i<N_CASEVAR;i++) 
    { optional[nvar]=0;
      varname[nvar++]=casevarname[i];
    }
  for (i=0;i<N_ITEM;i++) 
    { optional[nvar]=1;
      varname[nvar++]=keyvarname[i];
    }
  
  return writetemplate(outfile,nvar,optional,varname,NULL,NULL);
}
 

static int writeitemtemplate (FILE *outfile, enum itemcategory itemcat,
                              char *casevarname[], char *keyvarname, 
                              struct itemnamerec *item,
                              struct gspecvarnamerec gspecvarname[], 
                              char *dimflagvarname[])

{ int optional[NFIELD_MAX];
  char *varname[NFIELD_MAX];
  char *vartext[NFIELD_MAX];
  int nvar,i;

  if (!outfile) return OK;

  nvar=0;
  for (i=0;i<N_CASEVAR;i++) 
    { optional[nvar]=1;
      vartext[nvar]=NULL;
      varname[nvar++]=casevarname[i];
    }
  optional[nvar]=1;
  vartext[nvar]=NULL;
  varname[nvar++]=keyvarname;
  switch (itemcat)
    { case PSET:
        for (i=0;i<item->nvar;i++) 
          { optional[nvar]=item->varoptional[i]; 
            if (item->vartext[i]) vartext[nvar]=item->vartext[i];
            else vartext[nvar]=NULL;
            varname[nvar++]=item->varname[i];
          }
        break;
      case MATRIX:
        optional[nvar]=0;
        varname[nvar++]="nrow";
        optional[nvar]=0;
        varname[nvar++]="ncol";
        break;
      case GDOMITEM:
        for (i=0;i<N_DIM;i++) if (item->dimenabled[i]) 
          { optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].first;
            optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].last;
            optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].step;
            optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].n;
            optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].min;
            optional[nvar]=1;
            varname[nvar++]=gspecvarname[i].max;
          }
        break;
      case NGDOMITEM:
        optional[nvar]=0;
        varname[nvar++]="npoint";
        for (i=0;i<N_DIM;i++) if (item->dimenabled[i] && item->dimoptional[i]) 
          { optional[nvar]=1;
            varname[nvar++]=dimflagvarname[i];
          }
        break;
    }

  if (nvar>NFIELD_MAX) return SPECIAL;
  if (itemcat==PSET)
    return writetemplate(outfile,nvar,optional,varname,vartext,NULL);
  else
    return writetemplate(outfile,nvar,optional,varname,NULL,NULL);
}


static int writeitemdatatemplate (FILE *outfile, enum itemcategory itemcat,
                                  char *casevarname[], char *keyvarname,
                                  struct itemnamerec *item, char *dimname[])

{ int optional[NFIELD_MAX];
  char *varname[NFIELD_MAX];
  char *vartext[NFIELD_MAX];
  int nvar,i;

  if (!outfile) return OK;

  nvar=0;
  for (i=0;i<N_CASEVAR;i++)
    { optional[nvar]=1;
      vartext[nvar]=NULL;
      varname[nvar++]=casevarname[i];
    }
  optional[nvar]=1;
  vartext[nvar]=NULL;
  varname[nvar++]=keyvarname;

  if (itemcat==NGDOMITEM)
    for (i=0;i<N_DIM;i++) if (item->dimenabled[i])
      { optional[nvar]=item->dimoptional[i];
        vartext[nvar]=NULL;
        varname[nvar++]=dimname[i];
      } 

  for (i=0;i<item->nvar;i++)
    { optional[nvar]=item->varoptional[i];
      if (item->vartext[i]) vartext[nvar]=item->vartext[i];
      else vartext[nvar]=NULL;
      varname[nvar++]=item->varname[i];
    }

  if (nvar>NFIELD_MAX) return SPECIAL;
  return writetemplate(outfile,nvar,optional,varname,vartext,NULL);
}


static int writeitemdatavariables (FILE *outfile, enum itemcategory itemcat,
                                  struct itemnamerec *item, char *dimname[],
                                  struct itemtabrec *t)

{ int nidvar,i;

  if (!outfile) return OK;

  if (t->idspec.itemkey) nidvar=1;
  else nidvar=t->idspec.ncvar;

  for (i=0;i<nidvar;i++)
    if (fprintf(outfile,"%s\n",t->idspec.varname[i])<0) return OUTERR;
  if (itemcat==NGDOMITEM)
    for (i=0;i<N_DIM;i++) 
      if (t->dimflag[i] && fprintf(outfile,"%s\n",dimname[i])<0) 
        return OUTERR;
  for (i=0;i<item->nvar;i++) 
    if (fprintf(outfile,"%s%s : %s\n",item->varname[i],
                   (item->varoptional[i]) ? " (optional)" : "",
                   (item->vartext[i]) ? item->vartext[i] 
                   : "--- undescribed variable ---")<0) return OUTERR;

  return OK; 
}


static int writetemplate (FILE *outfile, int nvar, int optional[], 
                          char *varname[], char *vartext[], char *fieldval)

{ char *outvarname[2];
  char *field1[4];
  struct datarec dout={NULL,NULL,field1,NULL};
  int maxlen[4];
  int i,len,status;

  if (!outfile) return OK;

  maxlen[0]=4;
  for (i=0;i<nvar;i++)
    { len=strlen(varname[i]);
      if (len>maxlen[0]) maxlen[0]=len;
    }   
  maxlen[1]=5;
  maxlen[2]=1;
  maxlen[3]=1;
        
  outvarname[0]="NAME";
  outvarname[1]="VALUE";
  status=writevar(outfile,0,2,outvarname,maxlen,3);
  if (status!=OK) return status;
      
  for (i=0;i<nvar;i++)
    { dout.field[0]=varname[i];
      if (fieldval) dout.field[1]=fieldval;
      else dout.field[1]=".";
      if (vartext && vartext[i]) 
        { dout.field[2]="#";
          dout.field[3]=vartext[i];
        }
      else
        { dout.field[2]="";
          dout.field[3]="";
        }
      if (optional[i]) 
        if (fprintf(outfile,"#")<0) return OUTERR;
      status=writedata(outfile,0,&dout,4,NULL,maxlen,NULL,DEFAULT,3,1);
      if (status!=OK) return OUTERR;
    }       

  return OK;
}


static int readctrlfile (char *fname, char *reqvarname[], 
                         struct itemnamerec item[],
                         struct ctrlrec **head, struct ctrlrec **curr,
                         int *lineno, char **missingvar)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  struct datarec d={NULL,buff1.fieldlist,buff1.field,NULL};
  int reqvar[N_CTRLTABVAR];  
  struct ctrlrec *newdata;
  int readstatus[N_ITEM+N_OUTTAB+4];
  struct fnamerec *fcurr;
  char *s;
  int status,ninvar,nvar,ttb,len,i,j,numerr;

  *lineno=0; 
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;

  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status; 
  if (ttb) return FMTERR;

  nvar=indexlist(N_CTRLTABVAR,reqvarname,ninvar,invarname.field,reqvar);

  if (nvar==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  if (nvar<N_CTRLTABVAR)
    { i=1;                                       /* skip log file (optional) */ 
#if OPT_ENABLED == 1 
      if (reqvar[i++]>=0)      /* if 'optpset' present check for 'freeparmf' */
        { if (reqvar[i+1]<0)  
            { *missingvar=reqvarname[i+1];
              return VARERR; 
            }
        }
      i+=7; 
#endif
      for (j=0;j<N_ITEM;j++)
        { if (reqvar[i]<0 && !item[j].optional)
             { *missingvar=reqvarname[i];
               return VARERR; 
             }
           i++;
        }
    }

  while ((status=getdata(ninvar,NULL,infile,0,NULL,invarname.field,&d,NULL,
                         lineno))==OK)

    { newdata=(struct ctrlrec *)malloc(sizeof(struct ctrlrec));
      if (!newdata) return MEMERR;

      len=0;
      for (i=0;i<N_CTRLTABVAR;i++) 
        if (reqvar[i]>=0) len+=strlen(d.field[reqvar[i]])+5; 
                        /* Allow space for 4 character file name suffix */

      if (*curr)

        { /* May need extra space for names copied from previous record */
 
#if OPT_ENABLED == 1 
          if ((*curr)->optconfigf.name) len+=strlen((*curr)->optconfigf.name)+1;
          if ((*curr)->freeparmf.name) len+=strlen((*curr)->freeparmf.name)+1;
          if ((*curr)->parminitf.name) len+=strlen((*curr)->parminitf.name)+1;
#endif
          if ((*curr)->casef.name) len+=strlen((*curr)->casef.name)+1;
          for (i=0;i<N_ITEM;i++) 
            if ((*curr)->itemf[i].name) len+=strlen((*curr)->itemf[i].name)+1;
          for (i=0;i<N_OUTTAB;i++) 
            if ((*curr)->outvarf[i].name) 
              len+=strlen((*curr)->outvarf[i].name)+1; 
        }

      s=(char *)malloc(len*sizeof(char)); 
      if (!s) return MEMERR;
      newdata->fieldlist=s;

      fcurr=NULL;
      
      if (reqvar[0]<0) newdata->logf.name=NULL;
      else
        { if (getoutfname(d.field[reqvar[0]],&newdata->logf,s)!=OK)  
            { fprintf(stderr,"*** Invalid log file name ***\n");
              return BADDATA; 
            }
          if (newdata->logf.flag==2)
            { fprintf(stderr,"*** Log file cannot be binary format ***\n");
              return BADDATA; 
            }
          s+=strlen(s)+1;
        }

      i=1;

#if OPT_ENABLED == 1 

      if (reqvar[i]<0)

        { newdata->ioptpset=-1;
          newdata->optconfigf.name=NULL;
          newdata->freeparmf.name=NULL;
          newdata->parminitf.name=NULL;
          newdata->funcgaf.name=NULL;
          newdata->funcpowellf.name=NULL;
          newdata->newparmf.name=NULL;
          newdata->optreportf.name=NULL;
          i+=8;
        }

      else

        { if (strcmp(d.field[reqvar[i]],".")==0)
            { if (*curr) newdata->ioptpset=(*curr)->ioptpset;
              else 
                { fprintf(stderr,
                          "*** Missing optimizable parameter set number ***\n");
                  return BADDATA;
                }
            } 
          else
            { newdata->ioptpset=
                floor(sgetnum(d.field[reqvar[i]],&numerr)+0.5)-1;
              if (numerr) newdata->ioptpset=-1;
              else if (newdata->ioptpset < 0 || newdata->ioptpset > N_PSET-1)
                { fprintf(stderr,
                    "*** Invalid optimizable parameter set number ***\n");
                  return BADDATA;
                }
              else if (reqvar[newdata->ioptpset+9]<0)
                { fprintf(stderr,
        "*** Missing file name for optimizable parameter set (item '%s') ***\n",
                          item[newdata->ioptpset].name);
                  return BADDATA;
                }
            }
          i++;

          if (reqvar[i]<0) newdata->optconfigf.name=NULL;
          else
            { if (*curr) fcurr=&(*curr)->optconfigf;
              if (getinfname(d.field[reqvar[i]],1,
                             fcurr,&newdata->optconfigf,s,&readstatus[0])!=OK) 
                { fprintf(stderr,
                    "*** Invalid name for optimizer configuration file ***\n");
                  return BADDATA;
                }
              if (newdata->ioptpset>=0) readstatus[0]=1;
              s+=strlen(s)+1;
            }
          i++;

          if (*curr) fcurr=&(*curr)->freeparmf;
          if (getinfname(d.field[reqvar[i++]],newdata->ioptpset<0,
                         fcurr,&newdata->freeparmf,s,&readstatus[1])!=OK) 
            { fprintf(stderr,
                      "*** Invalid/missing name for free parameter file ***\n");
              return BADDATA;
            }
          if (newdata->ioptpset>=0) readstatus[1]=1;
          s+=strlen(s)+1;

          if (reqvar[i]<0) newdata->parminitf.name=NULL;
          else
            { if (*curr) fcurr=&(*curr)->parminitf;
              if (getinfname(d.field[reqvar[i]],1,
                             fcurr,&newdata->parminitf,s,&readstatus[2])!=OK) 
                { fprintf(stderr,
                    "*** Invalid name for parameter initialization file ***\n");
                  return BADDATA;
                }

              if (newdata->ioptpset>=0) readstatus[2]=1;
              s+=strlen(s)+1;
            }
          i++;

          if (reqvar[i]<0) newdata->funcgaf.name=NULL;
          else
            { if (getoutfname(d.field[reqvar[i]],&newdata->funcgaf,s)!=OK) 
                { fprintf(stderr,
                  "*** Invalid name for G.A. function evaluation  file ***\n"); 
                  return BADDATA;
                }
              s+=strlen(s)+1;
            }
          i++;

          if (reqvar[i]<0) newdata->funcpowellf.name=NULL;
          else
            { if (getoutfname(d.field[reqvar[i]],&newdata->funcpowellf,s)!=OK) 
                { fprintf(stderr,
                  "*** Invalid name for Powell function evaluation file ***\n");
                  return BADDATA;
                }
              s+=strlen(s)+1;
            }
          i++;

          if (reqvar[i]<0) newdata->newparmf.name=NULL;
          else
            { if (getoutfname(d.field[reqvar[i]],&newdata->newparmf,s)!=OK) 
                { fprintf(stderr,
                    "*** Invalid name for new parameter file ***\n");
                  return BADDATA;
                }
              s+=strlen(s)+1;
            }
          i++;

          if (reqvar[i]<0) newdata->optreportf.name=NULL;
          else
            { if (getoutfname(d.field[reqvar[i]],&newdata->optreportf,s)!=OK) 
                { fprintf(stderr,
                    "*** Invalid name for optimizer report file ***\n");
                  return BADDATA;
                }
              if (newdata->optreportf.flag==2)
                { fprintf(stderr,
                    "*** Optimizer report file cannot be binary format ***\n");
                  return BADDATA;
                }
              s+=strlen(s)+1;
            }
          i++;
        }

#endif

      for (j=0;j<N_ITEM;j++) 
        { if (reqvar[i]<0) newdata->itemf[j].name=NULL;
          else
            { if (*curr) fcurr=&(*curr)->itemf[j];
              if (getinfname(d.field[reqvar[i]],
                             item[j].optional && j!=newdata->ioptpset,
                             fcurr,&newdata->itemf[j],s,&readstatus[4+j])!=OK) 
                { if (j==newdata->ioptpset)
                    fprintf(stderr,
"*** Invalid/missing file name for optimizable parameter set (item '%s') ***\n",
                            item[j].name);
                  else
                    fprintf(stderr,
                      "*** Invalid/missing file name for item '%s' ***\n",
                            item[j].name);
                  return BADDATA;
                }
              else
                { readstatus[4+j]=1;
                  s+=strlen(s)+1;
                }
            }
          i++;
        }
     
      if (N_CASEVAR<=0 || reqvar[i]<0) newdata->casef.name=NULL;
      else
        { if (*curr) fcurr=&(*curr)->casef;
          if (getinfname(d.field[reqvar[i]],1,fcurr,&newdata->casef,s,
                         &readstatus[3])!=OK)
            { fprintf(stderr,"*** Invalid case file name ***\n");
              return BADDATA;
            }
          readstatus[3]=1;
          s+=strlen(s)+1;
        }
      if (N_CASEVAR>0) i++;

      for (j=0;j<N_OUTTAB;j++)
        { if (reqvar[i]<0) newdata->outf[j].name=NULL;
          else 
            { if (getoutfname(d.field[reqvar[i]],&newdata->outf[j],s)!=OK) 
                { fprintf(stderr,"*** Invalid output table file name ***\n");
                  return BADDATA;
                }
              s+=strlen(s)+1;
            }
          i++;
        }

      for (j=0;j<N_OUTTAB;j++)
        { if (reqvar[i]<0) newdata->outvarf[j].name=NULL;
          else
            { if (*curr) fcurr=&(*curr)->outvarf[j];
              if (getinfname(d.field[reqvar[i]],0,fcurr,&newdata->outvarf[j],s,
                             &readstatus[4+(N_ITEM)+j])!=OK)
                { fprintf(stderr,
                    "*** Invalid name for output table variables file ***\n");
                  return BADDATA;
                }
              readstatus[4+(N_ITEM)+j]=1;
              s+=strlen(s)+1;
            }
          i++;
        }

      if (!(*head)) (*head)=newdata;
      else (*curr)->next=newdata;
      (*curr)=newdata;
    }
  
  if (status!=EOF) return status; 
  if (infile) fclose(infile);
  return OK;
}


static int getinfname (char *name, int allowblank, 
                       struct fnamerec *fprev, struct fnamerec *f, char *s,
                       int *readstatus)

/* File names are compared with those in the last control record to see 
   whether the data will already be resident in the database. If the name
   matches that in the last control record AND the file will have been read 
   for a previous experiment and not since replaced then it will be resident.
   Otherwise it is assumed not to be. 
   Whether the file will have been read is tracked by the 'readstatus' flag. 
   It is reset here to a default value of 0 when the file name changes in the
   control table.  It is then set to 1 by the calling function when the file 
   is to be read and remains so while the file name remains the same. */

{ if (*name=='-' || *name=='+')
    { if (*name=='-') 
        { f->flag=0;  /* suppress log output */
          name++;
        }
      else 
        { name++;
          if (*name=='+') 
            { f->flag=3;   /* expand log output (include full data table) */
              name++;
            }
          else f->flag=2;  /* expand log output (include data with limits ) */
        }
      if (!*name) return BADDATA;
    }
  else f->flag=1;  /* standard log output */

  if (*name=='!') 
    { f->createfile=1;  /* create copy of table in external file */
      name++;
      if (!*name) return BADDATA;
      if (*name=='=')
        { f->createfile=2;  /* use binary format */
          name++;
          if (!*name) return BADDATA;
        }
    }
  else f->createfile=0;  /* don't create external copy */

  f->name=s;

  if (!fprev) 
    { if (strcmp(name,".")==0) return BADDATA; 
      if (strcmp(name,NODATA_STR)==0) 
        { if (!allowblank) return BADDATA;
          f->name=NULL;
        }
      else strcpy(f->name,name);
      f->resident=0;
      *readstatus=0;
    }
  else
    { if (strcmp(name,".")==0)
        { if (!fprev->name) 
            { if (!allowblank) return BADDATA;
              f->name=NULL;
              f->resident=0;
              *readstatus=0;
            }
          else
            { strcpy(f->name,fprev->name);
              f->resident=(*readstatus);
            }
        }
      else
        { if (strcmp(name,NODATA_STR)==0) 
            { if (!allowblank) return BADDATA;
              f->name=NULL;
              f->resident=0;
              *readstatus=0;
            }
          else 
            { strcpy(f->name,name);
              if (fprev->name && strcmp(f->name,fprev->name)==0) 
                f->resident=(*readstatus);
              else 
                { f->resident=0;
                  *readstatus=0;
                }
            }
        }
    }
   
  return OK;
}


static int getoutfname (char *name, struct fnamerec *f, char *s)

{ int len;

  if (*name=='-')
    { f->flag=0;  /* suppress output table */
      name++;
      if (!*name) return BADDATA;
    }
  else if (*name=='=')
    { f->flag=2;  /* binary format */
      name++;  
      if (!*name) return BADDATA;
    }
  else f->flag=1;  /* standard ascii format (unless name has ttb suffix) */

  if (strcmp(name,NODATA_STR)==0) f->name=NULL; 
  else if (f->flag==2)
    f->name=ttbname(name,s);  
  else 
    { len=strlen(name);
      if (len>=4 && (strcmp(name+len-4,".ttb")==0 || 
                     strcmp(name+len-4,".TTB")==0)) f->flag=2;
      f->name=s;
      strcpy(f->name,name);
    }

  return OK;
}


static int readoutvarfile (char *fname, int nouttabvar, char *reqvarname[], 
                           unsigned char active[], int *lineno)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  int reqvar[NFIELD_MAX];
  struct datarec d={buff1.x,buff1.fieldlist,buff1.field,NULL};
  int ninvar,nreqvar,flag,i,status,ttb;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;

  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;

  if (indexlist(nouttabvar,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  /* Set output variable switches ON by default for all variables present
     and OFF for others */

  for (i=0;i<nouttabvar;i++)
    { if (reqvar[i]>=0) active[i]=1; 
      else active[i]=0;
    }

  /* If there is a data record then use explicit flags in place of defaults */

  status=getdata(ninvar,NULL,infile,ttb,NULL,invarname.field,&d,NULL,lineno);
  if (status==EOF) return OK;
  if (status!=OK) return status;

  if (infile) fclose(infile);

  for (i=0;i<nouttabvar;i++)
    { if (reqvar[i]>=0) 
        { if (!ttb) d.x[reqvar[i]]=sgetnum(d.field[reqvar[i]],&flag);
          if (d.x[reqvar[i]]==NODATA) 
            { fprintf(stderr,"*** Missing flag value (use 0 or 1) ***\n");
              return BADDATA;
            }
          flag=floor(d.x[reqvar[i]]+0.5);
          if (flag<0 || flag >1) 
            { fprintf(stderr,"*** Invalid flag (use 0 or 1) ***\n");
              return BADDATA;
            }
          active[i]=flag;
        }
    } 

  return OK; 
}


static int readcasefile (char *fname, char *casevarname[], char *keyvarname[], 
                         struct casetabrec *t, int *lineno, char **missingvar)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  char *reqvarname[N_CASETABVAR_NZ];
  int reqvar[N_CASETABVAR_NZ];
  int varpos[N_CASETABVAR_NZ];
  struct readctrlrec readctrl;
  struct datarec *dnew,*d,*head;
  struct datarec dtable;
  struct caseinputrec *tabentry;
  int status,ninvar,nvar,ttb,len,c,i,j,k;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;

  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;

  i=0;
  for (j=0;j<N_CASEVAR;j++) reqvarname[i++]=casevarname[j];
  for (j=0;j<N_ITEM;j++) reqvarname[i++]=keyvarname[j];

  if (indexlist(N_CASETABVAR,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  for (i=0; i<N_CASEVAR && reqvar[i]>=0 ;i++);
  if (i<N_CASEVAR)
    { *missingvar=reqvarname[i];
      return VARERR;
    }

  /* Set key flags to indicate which key variables are present */

  k=N_CASEVAR;
  for (i=0;i<N_ITEM;i++) t->keyflag[i]=(reqvar[k++]>=0);

  /* Locate id variables (i.e. all requested variables which are present) */ 

  nvar=0;
  for (i=0;i<N_CASETABVAR;i++) 
    if (reqvar[i]>=0) varpos[nvar++]=reqvar[i];

  /* Read records into temporary storage */

  head=NULL;
  readctrl.nstrvar=nvar;
  readctrl.strvar=varpos;
  readctrl.nkeyvar=nvar;
  readctrl.keyvar=varpos;

  while (t->ndata<NREC_MAX &&
         (status=getdata(ninvar,NULL,infile,ttb,&readctrl,invarname.field,NULL,
                         &dnew,lineno))==OK)

    { if (!head) 
        { for (i=0;i<nvar;i++)
            if (strcmp(dnew->field[varpos[i]],".")==0) 
              { fprintf(stderr,"*** Invalid '%s' value ***\n",reqvarname[i]);
                return BADDATA; 
              }
          head=dnew;
        }
      else 
        { for (i=0;i<nvar;i++)
            if (strcmp(dnew->field[varpos[i]],".")==0) 
              dnew->field[varpos[i]]=d->field[varpos[i]]; 
          c=datacomp(N_CASEVAR,dnew,varpos,d,varpos,NUMCOMP);
          if (c<0) return UNSORTED;
          if (c==0) return DUPLICATE;
          d->next=dnew;
        }
      d=dnew;
      t->ndata++;
    }

  if (t->ndata==NREC_MAX) return RECLIMIT;
  if (status!=EOF || t->ndata==0) return status; 
  if (infile) fclose(infile);

  /* Allocate case table space */
 
  (t->table)=
    (struct caseinputrec *)malloc((t->ndata)*sizeof(struct caseinputrec));
  if (!t->table) return MEMERR;

  /* Copy data into case table */

  dtable.x=NULL;
  d=head;
  for (i=0;i<t->ndata;i++)

    { tabentry=&t->table[i];
      tabentry->casenum=i+1;

      /* Allocate space for field strings */
 
      len=0;
      for (j=0;j<nvar;j++) 
        len+=strlen(d->field[varpos[j]])+1; 
      tabentry->fieldlist=(char *)malloc(len*sizeof(char));
      if (!tabentry->fieldlist) return MEMERR;
      dtable.fieldlist=tabentry->fieldlist;

      /* Copy data - blank out keys if key variables are absent */

      k=0; 
      for (j=0;j<N_CASEVAR;j++) 
        { dtable.field=&tabentry->caseid[j];
          copydata(&dtable,d,1,&varpos[k++]);
          dtable.fieldlist+=strlen(dtable.fieldlist)+1;
        }

      for (j=0;j<N_ITEM;j++)
        { if (t->keyflag[j])
            { dtable.field=&tabentry->itemkey[j];
              copydata(&dtable,d,1,&varpos[k++]);
              dtable.fieldlist+=strlen(dtable.fieldlist)+1;
            }
          else tabentry->itemkey[j]=NULL;
          tabentry->item[j]=NULL;
        }

      d=d->next;
    }  

  freelist(&head);
  return OK;
} 


static int readitemfile (char *fname, enum itemcategory itemcat,
                         char *casevarname[], char *keyvarname, 
                         struct itemnamerec *item, 
                         struct gspecvarnamerec gspecvarname[], 
                         char *dimflagvarname[],
                         struct itemtabrec *t, int *lineno, char **missingvar)  

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  char *reqvarname[NFIELD_MAX];
  int reqvar[NFIELD_MAX];
  int varpos[NFIELD_MAX];
  struct readctrlrec readctrl;
  struct datarec *dnew,*d,*head;
  struct itemrec *tabentry;
  struct gridspecrec *g;
  struct datarec dtable;
  int status,ninvar,nreqvar,nidvar,nvar,dimfound,len,ttb,i,j,k,c,dummyflag;
      
  t->itemcat=itemcat;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;
  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;

  nreqvar=0;
  for (j=0;j<N_CASEVAR;j++) reqvarname[nreqvar++]=casevarname[j];
  reqvarname[nreqvar++]=keyvarname;
  switch (itemcat)
    { case PSET:
        for (j=0;j<item->nvar;j++) reqvarname[nreqvar++]=item->varname[j]; 
        break;
      case MATRIX:
        reqvarname[nreqvar++]="nrow";
        reqvarname[nreqvar++]="ncol";
        break;
      case GDOMITEM:
        for (j=0;j<N_DIM;j++) if (item->dimenabled[j])
          { reqvarname[nreqvar++]=gspecvarname[j].first;
            reqvarname[nreqvar++]=gspecvarname[j].last;
            reqvarname[nreqvar++]=gspecvarname[j].step;
            reqvarname[nreqvar++]=gspecvarname[j].n;
            reqvarname[nreqvar++]=gspecvarname[j].min;
            reqvarname[nreqvar++]=gspecvarname[j].max;
          }
        break;
      case NGDOMITEM:
        reqvarname[nreqvar++]="npoint";
        for (j=0;j<N_DIM;j++) if (item->dimenabled[j])
          reqvarname[nreqvar++]=dimflagvarname[j];
        break;
    }

  if (nreqvar>NFIELD_MAX) return SPECIAL;
 
  if (indexlist(nreqvar,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  switch (itemcat)

    { case PSET:

        for (i=N_CASEVAR+1;i<nreqvar;i++)
          { if (reqvar[i]<0 && !item->varoptional[i-(N_CASEVAR+1)])
              { *missingvar=reqvarname[i];
                return VARERR;
              }
            t->varactive[i-(N_CASEVAR+1)]=(reqvar[i]>=0);
          }
        break;

      case MATRIX:

        for (i=N_CASEVAR+1;i<nreqvar;i++)
          { if (reqvar[i]<0) 
              { *missingvar=reqvarname[i];
                return VARERR;
              }
          }
        break;

      case GDOMITEM:

        /* A <dim_name>1 or <dim_name>min variable must be present for at least 
           one of the dimensions. This indicates that the dimension 
           <dim_name> is specified. 
           The presence of <dim_name>1 indicates grid registration.
           The presence of <dim_name>min indicates interval registration.
           (Grid registration takes precedence if both are present.)

           For each grid registered dimension either
             <dim_name>step and n<dim_name> or
             <dim_name>n and n<dim_name> or
             <dim_name>n and <dim_name>step
           are used to create the grid specification. 

           For each interval registered dimension either
             <dim_name>step and n<dim_name> or
             <dim_name>max and n<dim_name> or
             <dim_name>max and <dim_name>step
           are used to create the grid specification.

           The variable pairs are listed above in order of preference. 
           The 4th grid specification parameter is calculated
           from the other 3. It's presence (if any) in the input is ignored. */

        dimfound=0;
        i=N_CASEVAR+1; 

        for (j=0;j<N_DIM;j++) if (item->dimenabled[j])

          { if (reqvar[i]>=0) 
              { dimfound=1;
                t->dimflag[j]=1;
                if (reqvar[i+2]>=0 && reqvar[i+3]>=0) 
                  { t->gspecmode[j]=STEP_AND_N;
                    reqvar[i+1]=-1;
                  }
                else if (reqvar[i+1]>=0 && reqvar[i+3]>=0)
                  { t->gspecmode[j]=LAST_AND_N;
                    reqvar[i+2]=-1;
                  }
                else if (reqvar[i+1]>=0 && reqvar[i+2]>=0)
                  { t->gspecmode[j]=LAST_AND_STEP;
                    reqvar[i+3]=-1;
                  }
                else 
                  { t->gspecmode[j]=STEP_AND_N;
                    reqvar[i+1]=-1;
                  }
                reqvar[i+4]=-1;
                reqvar[i+5]=-1;
              }

            else if (reqvar[i+4]>=0)
              { dimfound=1;
                t->dimflag[j]=1;
                if (reqvar[i+2]>=0 && reqvar[i+3]>=0) 
                  { t->gspecmode[j]=IREG_STEP_AND_N;
                    reqvar[i+5]=-1;
                  }
                else if (reqvar[i+5]>=0 && reqvar[i+3]>=0)
                  { t->gspecmode[j]=IREG_MAX_AND_N;
                    reqvar[i+2]=-1;
                  } 
                else if (reqvar[i+5]>=0 && reqvar[i+2]>=0)
                  { t->gspecmode[j]=IREG_MAX_AND_STEP;
                    reqvar[i+3]=-1;
                  }
                else
                  { t->gspecmode[j]=IREG_STEP_AND_N;
                    reqvar[i+5]=-1;
                  }
                reqvar[i]=-1;
                reqvar[i+1]=-1;
              }

            else if (!item->dimoptional[j])
              { fprintf(stderr,
"*** No '<dimension_name>1' or '<dimension_name>min' for mandatory dimension ***\n"); 
                *missingvar=gspecvarname[j].first;   
                return VARERR;
              }

            i+=6; 
          }

        if (!dimfound)
          { fprintf(stderr,
        "*** No '<dimension_name>1' or '<dimension_name>min' variables ***\n");
            if (N_DIM==1) *missingvar=gspecvarname[0].first; 
            else *missingvar="<dimension_name>1"; 
            return VARERR;
          }
     
        break; 

      case NGDOMITEM:

        i=N_CASEVAR+1; 

        if (reqvar[i]<0)
          { *missingvar=reqvarname[i];
            return VARERR;
          }
        i++;

        dimfound=0;
        for (j=0;j<N_DIM;j++) if (item->dimenabled[j])
          { if (reqvar[i++]>=0 || !item->dimoptional[j]) 
              { dimfound=1;
                t->dimflag[j]=1;
              }
          }

        if (!dimfound)
          { if (N_DIM==1) *missingvar=dimflagvarname[0];
            else *missingvar="<dimension_name>flag";
            return VARERR;
          }

        break;
    }
    
  /* Determine which id variables are used (if any) 
     If item key variable is present then ignore case variables */
      
  if (reqvar[N_CASEVAR]>=0)
    { t->idspec.itemkey=1;
      t->idspec.varname[0]=keyvarname;
      nidvar=1;
      for (i=0;i<N_CASEVAR;i++) reqvar[i]=-1;
    }
  else
    { nidvar=0;
      for (i=0;i<N_CASEVAR;i++)
        { if (reqvar[i]>=0)
            { t->idspec.cvar[nidvar]=i;
              t->idspec.varname[nidvar]=casevarname[i];
              nidvar++;
            }
        } 
      t->idspec.ncvar=nidvar;
    }

  /* Determine positions of variables of interest 
     N.B. absent non-id variables are included (negative 'varpos')
     as some processing is required for these */

  nvar=0;
  for (i=0;i<N_CASEVAR+1;i++)
    if (reqvar[i]>=0) varpos[nvar++]=reqvar[i];
  for (i=N_CASEVAR+1;i<nreqvar;i++) varpos[nvar++]=reqvar[i]; 
  
  /* Read records into temporary storage
     Records must be in id order (blank ids leading) 
     Non-id values in previous record are taken as current defaults and 
     replace any '.' values */

  head=NULL;
  readctrl.nstrvar=nidvar;
  readctrl.strvar=varpos;
  readctrl.nkeyvar=nidvar; 
  readctrl.keyvar=varpos;

  while (t->ndata<NREC_MAX &&
         (status=getdata(ninvar,NULL,infile,ttb,&readctrl,invarname.field,NULL,
                         &dnew,lineno))==OK)
  
    { if (!ttb)
        for (i=nidvar;i<nvar;i++) 
          if (varpos[i]>=0) 
            dnew->x[varpos[i]]=sgetnum(dnew->field[varpos[i]],&dummyflag);

      if (!head) 
        { for (i=0;i<nidvar;i++)
            if (strcmp(dnew->field[varpos[i]],".")==0) 
              { fprintf(stderr,"*** Missing '%s' value ***\n",
                        t->idspec.varname[i]);
                return BADDATA; 
              }
          head=dnew;
        }
      else 
        { if (nidvar==0) 
            { fprintf(stderr,"*** Multiple records but no id variables ***\n");
              return BADDATA;
            }
          for (i=0;i<nidvar;i++) 
            if (strcmp(dnew->field[varpos[i]],".")==0)
              dnew->field[varpos[i]]=d->field[varpos[i]]; 
          for (i=nidvar;i<nvar;i++)
            { if (varpos[i]>=0 && dnew->x[varpos[i]]==NODATA) 
                { if (!ttb && strcmp(dnew->field[varpos[i]],".")==0) 
                    dnew->x[varpos[i]]=d->x[varpos[i]];
                }
            }
          c=datacomp(nidvar,dnew,varpos,d,varpos,NUMCOMP); 
          if (c<0) return UNSORTED;
          if (c==0) return DUPLICATE;
          d->next=dnew;
        }

      d=dnew;
      t->ndata++;
    }

  if (t->ndata==NREC_MAX) return RECLIMIT;
  if (status!=EOF || t->ndata==0) return status;
  if (infile) fclose(infile);

  /* Allocate item table space */

  (t->table)=(struct itemrec *)malloc((t->ndata)*sizeof(struct itemrec));
  if (!t->table) return MEMERR;

  /* Copy data into item table */ 

  dtable.x=NULL;
  d=head;
  *lineno=0;
  for (i=0;i<t->ndata;i++)

    { tabentry=&t->table[i];
      (*lineno)--;

      /* Initialize new table entry */

      tabentry->parm=NULL;

      for (j=0;j<N_DIM;j++) 
        { tabentry->dimflag[j]=t->dimflag[j]; 
          tabentry->dim[j]=NULL;
        }

      for (j=0;j<N_ITEMVAR_MAX;j++) 
        { tabentry->value[j]=NULL;
          tabentry->count[j]=NULL;
        }

      tabentry->objfunflag=0;

      /* Allocate space for field strings */

      if (nidvar==0) tabentry->fieldlist=NULL;
      else
        { len=0;
          for (j=0;j<nidvar;j++) len+=strlen(d->field[varpos[j]])+1; 
          tabentry->fieldlist=(char *)malloc(len*sizeof(char));
          if (!tabentry->fieldlist) return MEMERR;
          dtable.fieldlist=tabentry->fieldlist;
        }

      /* Copy item id data - blank out id values if id variables are absent */ 

      k=0;

      for (j=0;j<N_CASEVAR;j++)
        { if (reqvar[j]>=0)
            { dtable.field=&tabentry->caseid[j];
              copydata(&dtable,d,1,&varpos[k++]);
              dtable.fieldlist+=strlen(dtable.fieldlist)+1;
            }
          else tabentry->caseid[j]=NULL;
        }

      if (t->idspec.itemkey)
        { dtable.field=&tabentry->itemkey;
          copydata(&dtable,d,1,&varpos[k++]);
          dtable.fieldlist+=strlen(dtable.fieldlist)+1;
        }
      else tabentry->itemkey=NULL;
     
      /* Copy item spec. values */

      switch (itemcat)

        { case PSET:

            tabentry->parm=
              (OBJF_FLOAT_TYPE *)malloc(item->nvar*sizeof(OBJF_FLOAT_TYPE));
            if (!tabentry->parm) return MEMERR;
            for (j=0;j<item->nvar;j++) 
              { if (varpos[k]<0 || d->x[varpos[k]]==NODATA) 
                  { if (item->varoptional[j]) tabentry->parm[j]=OBJF_NODATA;
                    else
                      { fprintf(stderr,"*** Missing '%s' value ***\n",
                                item->varname[j]);
                        return BADDATA;
                      }
                  }
                else
                  { tabentry->parm[j]=(OBJF_FLOAT_TYPE)d->x[varpos[k]];
                    if (item->varmin[j]!=OBJF_NODATA && 
                        tabentry->parm[j]<item->varmin[j])
                      { fprintf(stderr,
                          "*** Value for '%s' is less than minimum value %g ***\n",
                                item->varname[j],item->varmin[j]); 
                            return BADDATA;
                      }
                    if (item->varmax[j]!=OBJF_NODATA && 
                        tabentry->parm[j]>item->varmax[j])
                      { fprintf(stderr,
                          "*** Value for '%s' is greater than maximum value %g ***\n",
                                item->varname[j],item->varmax[j]); 
                            return BADDATA;
                      }
                  }
                k++; 
              }
            break;

          case MATRIX:

            if (varpos[k]<0 || d->x[varpos[k]]==NODATA) 
              { fprintf(stderr,"*** Missing 'nrow' value ***\n");
                return BADDATA; 
              }
            tabentry->nrow=floor(d->x[varpos[k++]]+0.5);
            if (varpos[k]<0 || d->x[varpos[k]]==NODATA) 
              { fprintf(stderr,"*** Missing 'ncol' value ***\n");
                return BADDATA; 
              }
            tabentry->ncol=floor(d->x[varpos[k++]]+0.5);
            if (tabentry->nrow<1 || tabentry->ncol<1) 
              { fprintf(stderr,"*** Invalid matrix dimension ***\n");
                return BADDATA; 
              }
            break;

          case GDOMITEM:

            for (j=0;j<N_DIM;j++) 

              { g=&tabentry->gspec[j]; 

                /* Assign active variables according to mode
                   and calculate the remaining grid parameters */

                if (t->dimflag[j])
 
                  { if (t->gspecmode[j]<IREG_STEP_AND_N)

                      { if (varpos[k]<0 || d->x[varpos[k]]==NODATA)
                          { if (item->dimoptional[j]) tabentry->dimflag[j]=0;
                            else
                              { fprintf(stderr,"*** Missing '%s' value ***\n",
                                        gspecvarname[j].first);
                                return BADDATA;
                              }
                          }
                        else g->first=(OBJF_FLOAT_TYPE)d->x[varpos[k]];
                        g->iregflag=0;
                      }

                    else

                      { if (varpos[k+4]<0 || d->x[varpos[k+4]]==NODATA)
                          { if (item->dimoptional[j]) tabentry->dimflag[j]=0;
                            else
                              { fprintf(stderr,"*** Missing '%s' value ***\n",
                                        gspecvarname[j].min);
                                return BADDATA;
                              }
                          }
                        else g->min=(OBJF_FLOAT_TYPE)d->x[varpos[k+4]];
                        g->iregflag=1;
                      }
                  }
             
                if (tabentry->dimflag[j])

                  { switch (t->gspecmode[j])

                      { case STEP_AND_N:

                          if (varpos[k+2]<0 || d->x[varpos[k+2]]==NODATA)
                            g->step=0;
                          else 
                            g->step=(OBJF_FLOAT_TYPE)d->x[varpos[k+2]];

                          if (varpos[k+3]<0 || d->x[varpos[k+3]]==NODATA)
                            g->n=1;
                          else 
                            { g->n=floor(d->x[varpos[k+3]]+0.5);
                              if (g->n<1) 
                                { fprintf(stderr,"*** Invalid '%s' value ***\n",
                                          gspecvarname[j].n);
                                  return BADDATA; 
                                }
                              if (g->n>1 && g->step==0)
                                { fprintf(stderr,
                                  "*** Invalid/missing '%s' for '%s' > 1 ***\n",
                                    gspecvarname[j].step,gspecvarname[j].n);
                                  return BADDATA; 
                                }
                            }

                          g->last=g->first+(g->n-1)*g->step;
                          g->min=g->first;
                          g->max=g->last;

                          break;

                        case LAST_AND_N:

                          if (varpos[k+1]<0 || d->x[varpos[k+1]]==NODATA)
                            g->last=g->first;
                          else 
                            g->last=(OBJF_FLOAT_TYPE)d->x[varpos[k+1]];

                          if (varpos[k+3]<0 || d->x[varpos[k+3]]==NODATA)
                            g->n=1;
                          else
                            { g->n=floor(d->x[varpos[k+3]]+0.5);
                              if (g->n<1) 
                                { fprintf(stderr,"*** Invalid '%s' value ***\n",
                                          gspecvarname[j].n);
                                  return BADDATA; 
                                }
                              if (g->n>1 && g->last==g->first)
                                { fprintf(stderr,
                                  "*** Invalid/missing '%s' for '%s' > 1 ***\n",
                                    gspecvarname[j].last,gspecvarname[j].n);
                                  return BADDATA; 
                                }
                            }

                          if (g->n==1) g->step=0;
                          else g->step=(g->last-g->first)/(g->n-1);
                          g->min=g->first;
                          g->max=g->last;

                          break;

                        case LAST_AND_STEP:

                          if (varpos[k+1]<0 || d->x[varpos[k+1]]==NODATA)
                            g->last=g->first;
                          else 
                            g->last=(OBJF_FLOAT_TYPE)d->x[varpos[k+1]];

                          if (varpos[k+2]<0 || d->x[varpos[k+2]]==NODATA)
                            g->step=0;
                          else 
                            g->step=(OBJF_FLOAT_TYPE)d->x[varpos[k+2]];

                          if (g->step==0)
                            { if (g->last!=g->first)
                                { fprintf(stderr,
                                    "*** Invalid/missing '%s' value ***\n",
                                          gspecvarname[j].step);
                                  return BADDATA; 
                                }
                              g->n=1; 
                            }
                          else
                            { g->n=1+floor((g->last-g->first)/g->step);
                              if (g->n<1)
                                { fprintf(stderr,
                                    "*** '%s' has incorrect sign? ***\n",
                                          gspecvarname[j].step);
                                  return BADDATA;
                                } 
                              g->last=g->first+(g->n-1)*g->step;
                            }

                          g->min=g->first;
                          g->max=g->last;

                          break;

                        case IREG_STEP_AND_N:

                          if (varpos[k+2]<0 || d->x[varpos[k+2]]==NODATA)
                            g->step=0;
                          else 
                            g->step=(OBJF_FLOAT_TYPE)d->x[varpos[k+2]];

                          if (varpos[k+3]<0 || d->x[varpos[k+3]]==NODATA)
                            g->n=1;
                          else 
                            g->n=floor(d->x[varpos[k+3]]+0.5);

                          if (g->n<1)
                            { fprintf(stderr,"*** Invalid '%s' value ***\n",
                                      gspecvarname[j].n); 
                              return BADDATA;
                            }
                          if (g->n>1 && g->step==0)
                            { *lineno=-(i+1);
                              fprintf(stderr,
                                "*** Invalid/missing '%s' for '%s' > 1 ***\n",
                                      gspecvarname[j].step,gspecvarname[j].n);
                              return BADDATA;
                            }

                          if (g->step>=0)
                            { g->max=g->min+(g->n)*g->step;
                              g->first=g->min+g->step/2;
                              g->last=g->max-g->step/2;
                            }
                          else
                            { g->max=g->min-(g->n)*g->step;
                              g->first=g->max+g->step/2;
                              g->last=g->min-g->step/2;
                            }

                          break;

                        case IREG_MAX_AND_N:

                          if (varpos[k+3]<0 || d->x[varpos[k+3]]==NODATA)
                            g->n=1;
                          else 
                            g->n=floor(d->x[varpos[k+3]]+0.5);

                          if (varpos[k+5]<0 || d->x[varpos[k+5]]==NODATA)
                            g->max=g->min;
                          else 
                            { g->max=(OBJF_FLOAT_TYPE)d->x[varpos[k+5]];
                              if (g->max<g->min)
                                { *lineno=-(i+1);
                                  fprintf(stderr,
                                    "*** '%s' less than '%s' ***\n",
                                    gspecvarname[j].max,gspecvarname[j].min); 
                                  return BADDATA; 
                                }
                            }

                          if (g->n<1) 
                            { *lineno=-(i+1);
                              fprintf(stderr,"*** Invalid '%s' value ***\n",
                                      gspecvarname[j].n); 
                              return BADDATA; 
                            }
                          if (g->n>1 && g->min==g->max)
                            { *lineno=-(i+1);
                              fprintf(stderr,
                                "*** Invalid/missing '%s' for '%s' > 1 ***\n",
                                      gspecvarname[j].max,gspecvarname[j].n);
                              return BADDATA;
                            }

                          if (g->n==1) g->step=0;
                          else g->step=(g->max-g->min)/g->n;
                          g->first=g->min+g->step/2;
                          g->last=g->max-g->step/2;

                          break;

                        case IREG_MAX_AND_STEP:

                          if (varpos[k+2]<0 || d->x[varpos[k+2]]==NODATA)
                            g->step=0;
                          else 
                            g->step=(OBJF_FLOAT_TYPE)d->x[varpos[k+2]];

                          if (varpos[k+5]<0 || d->x[varpos[k+5]]==NODATA)
                            g->max=g->min;
                          else 
                            { g->max=(OBJF_FLOAT_TYPE)d->x[varpos[k+5]];
                              if (g->max<g->min)
                                { *lineno=-(i+1);
                                  fprintf(stderr,
                                    "*** '%s' less than '%s' ***\n",
                                    gspecvarname[j].max,gspecvarname[j].min); 
                                  return BADDATA; 
                                }
                            }

                          if (g->step==0)
                            { if (g->max!=g->min)
                                { *lineno=-(i+1);
                                  fprintf(stderr,
                                    "*** Invalid/missing '%s' value ***\n",
                                          gspecvarname[j].step); 
                                  return BADDATA; 
                                }
                              g->n=1; 
                            }
                          else
                            { g->n=ceil((g->max-g->min)/fabs(g->step));
                              if (g->n<1)
                                { *lineno=-(i+1);
                                  fprintf(stderr,
                         "*** zero range incompatible with non-zero '%s' ***\n",
                                          gspecvarname[j].step);
                                  return BADDATA;
                                }
                            }

                          if (g->step>=0)
                            { g->max=g->min+(g->n)*g->step;
                              g->first=g->min+g->step/2;
                              g->last=g->max-g->step/2;
                            } 
                          else
                            { g->max=g->min-(g->n)*g->step;
                              g->first=g->max+g->step/2;
                              g->last=g->min-g->step/2;
                            }

                          break;
                      }

                    g->rsearch=fabs(g->step/2); 
                  }

                else 

                  { /* dimension not specified */ 

                    g->first=OBJF_NODATA;
                    g->last=OBJF_NODATA;
                    g->step=OBJF_NODATA;
                    g->n=1;
                    g->min=OBJF_NODATA;
                    g->max=OBJF_NODATA;
                    g->rsearch=0;
                  }

                if (item->dimenabled[j]) k+=6;  /* next dimension */
              }

            break;

          case NGDOMITEM:

            if (varpos[k]<0 || d->x[varpos[k]]==NODATA) 
              { *lineno=-(i+1);
                fprintf(stderr,"*** Missing 'npoint' value ***\n");
                return BADDATA; 
              }
            tabentry->npoint=floor(d->x[varpos[k++]]+0.5); 
            if (tabentry->npoint<0) 
              { *lineno=-(i+1);
                fprintf(stderr,"*** Invalid 'npoint' value ***\n");
                return BADDATA; 
              }

            for (j=0;j<N_DIM;j++) if (t->dimflag[j])

              { if (varpos[k]<0) tabentry->dimflag[j]=1; 
                          /* flag variable is absent but dimflag is set anyway 
                             since dim. is  mandatory */ 
                else
                  { if (d->x[varpos[k]]==NODATA)
                      { fprintf(stderr,
                        "*** Missing dimension flag value (use 0 or 1) ***\n");
                        return BADDATA;
                      }
                    tabentry->dimflag[j]=floor(d->x[varpos[k]]+0.5);
                    if (tabentry->dimflag[j]<0 || tabentry->dimflag[j]>1)
                      { fprintf(stderr,
                          "*** Invalid dimension flag (use 0 or 1) ***\n");
                        return BADDATA;
                      }
                    if (!tabentry->dimflag[j] && !item->dimoptional[j])
                      { fprintf(stderr,
                   "*** Flag=0 for mandatory dimension (use 1 or omit) ***\n");
                        return BADDATA;
                      }
                  }

                if (item->dimenabled[j]) k++; 
              }

            break;
        }

      d=d->next;
    }

  freelist(&head);
  return OK;
}


static int readitemdatafile (char *fname, enum itemcategory itemcat,
                             struct itemnamerec *item, char *dimname[], 
                             struct itemtabrec *t, int *lineno, 
                             char **missingvar)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  char *reqvarname[NFIELD_MAX];
  int reqvar[NFIELD_MAX];
  struct readctrlrec readctrl;
  struct datarec d={buff1.x,buff1.fieldlist,buff1.field,NULL};
  struct datarec dcurr={NULL,buff2.fieldlist,buff2.field,NULL};
  struct itemrec *tabentry;
  unsigned long nval,count;
  int status,ninvar,nidvar,nreqvar,nvar,nleadvar,ttb,i,j,k,m,first,dummyflag;
  double x;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;

  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;

  if (ttb) nidvar=0;
  else if (t->idspec.itemkey) nidvar=1;
  else nidvar=t->idspec.ncvar;
 
  for (i=0;i<nidvar;i++) reqvarname[i]=t->idspec.varname[i];
  nreqvar=nidvar;
  if (itemcat==NGDOMITEM) 
    for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
      reqvarname[nreqvar++]=dimname[j];
  nleadvar=nreqvar;  
  for (i=0;i<item->nvar;i++) reqvarname[nreqvar++]=item->varname[i];

  nvar=indexlist(nreqvar,reqvarname,ninvar,invarname.field,reqvar);

  if (nvar==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  if (nvar<nreqvar)
    { for (i=0; (i<nreqvar && reqvar[i]>=0) || 
                (i>=nleadvar && item->varoptional[i-nleadvar]); i++);
      if (i<nreqvar)
        { *missingvar=reqvarname[i];
          return VARERR;
        }
    }

  /* Set flags to show active variables */

  for (i=0;i<item->nvar;i++)  t->varactive[i]=(reqvar[i+nleadvar]>=0);

  first=1;     
  for (i=0;i<t->ndata;i++)

    { tabentry=&t->table[i];

      /* Allocate space for item data instance matching current 
         item table entry */

      status=grabitemspace(itemcat,t->dimflag,t->varactive,tabentry,item->nvar,
                           0,&nval);
      if (status!=OK) return status;

      /* Read first data record (if at start of file) */

      readctrl.nstrvar=nidvar;
      readctrl.strvar=reqvar;
      readctrl.nkeyvar=nidvar;
      readctrl.keyvar=reqvar;

      if (first)
        { status=getdata(ninvar,NULL,infile,ttb,&readctrl,invarname.field,&d,
                         NULL,lineno);
          if (status!=OK) return status;
          first=0;
        }

      if (nidvar>0)

        { /* Copy current id from item table */

          if (t->idspec.itemkey) dcurr.field[0]=tabentry->itemkey;
          else
            for (j=0;j<t->idspec.ncvar;j++)
              dcurr.field[j]=tabentry->caseid[t->idspec.cvar[j]];

          if (nval>0)
      
            { /* Skip any records with non-matching id values */  
          
              while (datacomp(nidvar,&d,reqvar,&dcurr,0,NUMCOMP)!=0)
                { status=getdata(ninvar,NULL,infile,ttb,&readctrl,
                                 invarname.field,&d,NULL,lineno);
                  if (status!=OK) return status;
                }
            }

          else

            { /* No values expected so check there are no matching records */

              if (datacomp(nidvar,&d,reqvar,&dcurr,0,NUMCOMP)==0)
                { fprintf(stderr,
     "*** Data record(s) found for an instance expected to have size 0 ***\n");
                  return BADDATA;
                }
            }
        }
      
      /* Read remaining item data for current item table entry */
  
      count=0; 
      for (j=0;j<nval;j++)

        { count++;
          m=nidvar;

          if (itemcat==NGDOMITEM)
            { for (k=0;k<N_DIM;k++) if (t->dimflag[k]) 
                { if (tabentry->dimflag[k])
                    { if (ttb) x=d.x[reqvar[m]];
                      else x=sgetnum(d.field[reqvar[m]],&dummyflag);
                      if (x==NODATA)
                        { if (!item->dimoptional)
                            { fprintf(stderr,"*** Invalid '%s' value ***\n",
                                      dimname[k]);
                              return BADDATA;
                            }
                          tabentry->dim[k][j]=OBJF_NODATA;
                        }
                      else tabentry->dim[k][j]=(OBJF_FLOAT_TYPE)x;
                    }
                  else tabentry->dim[k][j]=OBJF_NODATA;
                  m++;
                }
            }

          for (k=0;k<item->nvar;k++) if (t->varactive[k])
            { if (ttb) x=d.x[reqvar[nleadvar+k]];
              else x=sgetnum(d.field[reqvar[nleadvar+k]],&dummyflag);
              if (x==NODATA) tabentry->value[k][j]=OBJF_NODATA; 
              else tabentry->value[k][j]=(OBJF_FLOAT_TYPE)x;
            }

          status=getdata(ninvar,NULL,infile,ttb,&readctrl,invarname.field,&d,
                         NULL,lineno);

          if (nidvar>0) 

            { if (count<nval)
                { if (status!=OK) return status; 
                  if (datacomp(nidvar,&d,reqvar,&dcurr,0,NUMCOMP)!=0)
                    { fprintf(stderr,
        "*** Instance too small - expecting %ld data records, found %ld ***\n",
                              nval,count);
                      return BADDATA;
                    }
                }
              else 
                { if (status!=EOF && 
                      datacomp(nidvar,&d,reqvar,&dcurr,0,NUMCOMP)==0)
                    { fprintf(stderr,
        "*** Instance too big - expecting %ld data records, found extra ***\n",
                              nval);
                      return BADDATA;
                    }
                }
            }

          else if (count<nval && status!=OK) return status; 
        }
    }

  if (nidvar==0 && status!=EOF)
    { fprintf(stderr,"*** Found more data records than expected ***\n");
      return BADDATA;
    }

  if (infile) fclose(infile);
  return OK;
}


static int grabitemspace (enum itemcategory itemcat, int dimflag[],
                          unsigned char varactive[], struct itemrec *tabentry,
                          int nvar, int includecount, unsigned long *nval)

{ int j;

  switch (itemcat)
    { case MATRIX:
        (*nval)=tabentry->nrow*tabentry->ncol;
        break;
      case GDOMITEM:
        (*nval)=1;
        for (j=0;j<N_DIM;j++)
          if (dimflag[j]) (*nval)*=tabentry->gspec[j].n;
            break;
          case NGDOMITEM:
        (*nval)=tabentry->npoint;
        break;
    }

  if ((*nval)>0)

    { for (j=0;j<nvar;j++) if (varactive[j])
        { tabentry->value[j]=
            (OBJF_FLOAT_TYPE *)malloc((*nval)*sizeof(OBJF_FLOAT_TYPE));
          if (!tabentry->value[j]) return MEMERR;
          if (includecount)
            { tabentry->count[j]=(int *)malloc((*nval)*sizeof(int));
              if (!tabentry->count[j]) return MEMERR;
            }
        }

      if (itemcat==NGDOMITEM)
        { for (j=0;j<N_DIM;j++) if (dimflag[j])
            { tabentry->dim[j]=
                (OBJF_FLOAT_TYPE *)malloc((*nval)*sizeof(OBJF_FLOAT_TYPE));
              if (!tabentry->dim[j]) return MEMERR;
            }
        }
    }

  return OK;
}


#if NETCDF_ENABLED == 1


static int readitemdata_nc (char *fname, int firstfile, 
                            enum itemcategory itemcat,
                            struct itemnamerec *item, char *dimname[], 
                            int dimbyindex[], char *varname[], 
                            struct itemtabrec *t, 
                            double fillval, double dimscale[], 
                            double dimoffset[], double varscale[],
                            double varoffset[], FILE *logfile, char **errtxt)

{ FILE *infile;  
  struct nciorec io; 
  int dimid[N_DIM_NZ]; 
  int dimidv[N_DIM_NZ]; 
  int coordvarid[N_DIM_NZ];
  double *coord[N_DIM_NZ];
  size_t *cindex[N_DIM_NZ];
  size_t dimlen[N_DIM_NZ];
  size_t *selflag[N_DIM_NZ];
  size_t *sel;
  size_t segstart[N_DIM_NZ];
  size_t seglen[N_DIM_NZ];
  size_t len,k,num;
  nc_type vartype;
  struct gridspecrec *g;
  OBJF_FLOAT_TYPE gridref,diff;
  unsigned long nval;
  int i,j,m,c,status,nvar,ndimreq,ndimv,kstep,endofgrid,endofdimv;
  char linebuff[LINE_MAX+1];

  *errtxt=NULL;
  if (itemcat!=GDOMITEM)
    { *errtxt="NetCDF input only available for gridded domain items";
      return SPECIAL;
    }

  /* Open NetCDF file */

  status=nc_open(fname,NC_NOWRITE,&io.ncid);
  if (status!=NC_NOERR) 
    { if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;
      else 
        { fclose(infile);
          *errtxt="cannot open input file as NetCDF data set";
          return SPECIAL;
        }
    }

  if (logfile)
    { fprintf(logfile,"\n");
      status=writelog(logfile,0,"Extracting from NetCDF data set");
      if (status!=OK) return OUTERR;
    }

  /* Check for item variables 
      - Activation of any optional variables depends on first data set
      - All active variables must be present in subsequent data sets */

  io.nvar=item->nvar;
  io.varactive=t->varactive;
  if (firstfile) nvar=0;

  for (i=0;i<io.nvar;i++)

    { if (firstfile)
        { status=nc_inq_varid(io.ncid,varname[i],&io.varid[i]);
          if (item->varoptional[i]) io.varactive[i]=(status==NC_NOERR);
          else
            { io.varactive[i]=1;
              if (status!=NC_NOERR)
                { *errtxt=varname[i];
                  return VARERR;
                }
            }
          if (io.varactive[i]) nvar++;
        }
      else if (io.varactive[i])  
        { status=nc_inq_varid(io.ncid,varname[i],&io.varid[i]);
          if (status!=NC_NOERR) 
            { *errtxt=varname[i];
              return VARERR;
            }
        }

      if (io.varactive[i])

        { status=nc_inq_vartype(io.ncid,io.varid[i],&vartype);
          if (status!=NC_NOERR)
            { *errtxt="NetCDF error on variable type enquiry for item variable";
              return SPECIAL;
            }   
          if (vartype==NC_CHAR)
            { *errtxt="gridded-domain item variable is non-numeric";
              return SPECIAL;
            }
        }
    }

  if (firstfile && nvar==0)
    { *errtxt="no recognized item variables in NetCDF data set";
      return SPECIAL;
    } 

  /* Update NetCDF file counter */

  if (firstfile) io.filecount=1;
  else io.filecount++;

  /* Allocate and initialize space for all item instances if reading 
     first data set */

  if (firstfile) 

    { for (i=0;i<t->ndata;i++)

        { io.tabentry=&t->table[i];

          status=grabitemspace(itemcat,t->dimflag,io.varactive,io.tabentry,
                               item->nvar,1,&nval);
          if (status!=OK) return status;
          for (j=0;j<item->nvar;j++) if (io.varactive[j])
            { for (k=0;k<nval;k++)  
                { io.tabentry->value[j][k]=OBJF_NODATA;
                  io.tabentry->count[j][k]=0; 
                }
            }

        }

      for (j=0;j<item->nvar;j++) if (io.varactive[j])
        { io.varreport[j].totcount=0;
          io.varreport[j].gridcount=0;
          io.varreport[j].countmax=0;
        }
    }

  /* Get NetCDF dimension ids */

  ndimreq=N_DIM;
  for (i=0;i<N_DIM;i++) 

    { io.dimflag[i]=t->dimflag[i]; 

      if (!io.dimflag[i])

        { ndimreq--;
          dimid[i]=-1;
        }

      else

        { if (dimbyindex && dimbyindex[i])

            { /* Use dimension name supplied */

              status=nc_inq_dimid(io.ncid,dimname[i],&dimid[i]);
              if (status!=NC_NOERR)
                { *errtxt=
                    "NetCDF error on dimension id enquiry for dimension name";
                  return SPECIAL;
                }
            }

          else

            { /* Treat dimension name supplied as a variable name and determine 
                 dimension from the corresponding variable */           

              status=nc_inq_varid(io.ncid,dimname[i],&coordvarid[i]);
              if (status!=NC_NOERR) 
                { *errtxt=dimname[i];
                  return VARERR;
                }
              status=nc_inq_vartype(io.ncid,coordvarid[i],&vartype);
              if (status!=NC_NOERR)
                { *errtxt=
          "NetCDF error on variable type enquiry for co-ordinate variable";
                  return SPECIAL;
                }
              if (vartype==NC_CHAR) 
                { *errtxt=
          "co-ordinate variable for gridded-domain item is non-numeric";
                  return SPECIAL;
                }
              status=nc_inq_varndims(io.ncid,coordvarid[i],&ndimv);
              if (status!=NC_NOERR)
                { *errtxt=
          "NetCDF error on dimension number enquiry for co-ordinate variable";
                  return SPECIAL;
                }
              if (ndimv!=1) 
                { *errtxt=
          "coordinate variable for gridded-domain item is not a vector";
                  return SPECIAL;
                } 
              status=nc_inq_vardimid(io.ncid,coordvarid[i],&dimid[i]);
              if (status!=NC_NOERR)
                { *errtxt=
          "NetCDF error on dimension id enquiry for co-ordinate variable";
                  return SPECIAL;
                }
            }
        }
    }      

  /* Check item variable dimensions: each must have dimensions matching the 
     grid specification in the item table */

  nvar=0;
  for (i=0;i<io.nvar;i++) if (io.varactive[i])

    { status=nc_inq_varndims(io.ncid,io.varid[i],&ndimv);
      if (ndimv!=ndimreq)
        { sprintf(linebuff,"item variable %s has %d dimension(s), %d expected",
                  varname[i],ndimv,ndimreq); 
          *errtxt=linebuff;
          return SPECIAL;
        }
      status=nc_inq_vardimid(io.ncid,io.varid[i],dimidv);
      if (status!=NC_NOERR)
        { *errtxt="NetCDF error on dimension id enquiry for item variable";
          return SPECIAL;
        }

      nvar++;

      /* NetCDF variable dimensions may be ordered differently from 
         item variable dimensions in <dimid> that are in objective function  
         specific order so create map <io.dimpos> giving dimension positions 
         in NetCDF variable dimension list <dimidv> */ 

      for (j=0;j<N_DIM;j++) if (io.dimflag[j])
      
        { if (nvar==1)

            { /* Find position of current dimension */ 

              m=0;
              while (dimidv[m]!=dimid[j] && m<ndimv) m++;
              if (m==ndimv)
                { *errtxt="item variable is missing an expected dimension";
                  return SPECIAL;
                }
              io.dimpos[j]=m;
            }

          else

            { /* Check the current dimension is in the same position as it was
                 for the first variable */ 
              
              if (dimidv[io.dimpos[j]]!=dimid[j])
                { *errtxt="item variables have different dimension(s)";
                  return SPECIAL;
                }
            }
        }
    }
 
  /* Set up selection flag vectors and coordinate vectors for each 
     dimension specified in the item table */
 
  for (i=0;i<N_DIM;i++)

    { coord[i]=NULL;
      cindex[i]=NULL; 

      if (io.dimflag[i])

        { j=io.dimpos[i];

          /* Get dimension length 
             (dimensions in NetCDF variable specific order) */
 
          status=nc_inq_dimlen(io.ncid,dimid[i],&dimlen[j]);
          if (status!=NC_NOERR) 
            { *errtxt="NetCDF error on dimension length enquiry";
              return SPECIAL;
            }
          if (dimlen[j]<1) 
            { *errtxt="NetCDF data missing; dimension length < 1";
              return SPECIAL;
            }

          /* Set up selection flag vectors with end stops
             (dimensions in NetCDF variable specific order) */

          selflag[j]=(size_t *)malloc((dimlen[j]+1)*sizeof(size_t));
          selflag[j][dimlen[j]]=0;

          /* Create or retrieve raw coordinate vectors 
             (dimensions in objective function specific order) */

          coord[i]=(double *)malloc(dimlen[j]*sizeof(double));
          if (!coord[i]) return MEMERR;

          if (dimbyindex && dimbyindex[i])
            for (k=0;k<dimlen[j];k++) coord[i][k]=k+1; 
          else
            { status=nc_get_var_double(io.ncid,coordvarid[i],coord[i]);
              if (status!=NC_NOERR)
                { *errtxt="NetCDF error on retrieving co-ordinate vector";
                  return SPECIAL;
                }
            }

          /* Calibrate coordinates before use if required */

          if (dimscale && dimoffset)
            { for (k=0;k<dimlen[j];k++)  
                coord[i][k]=dimscale[i]*coord[i][k]+dimoffset[i];
           }
          else if (dimscale)
            { for (k=0;k<dimlen[j];k++)  
                coord[i][k]=dimscale[i]*coord[i][k];
            }
          else if (dimoffset)
            { for (k=0;k<dimlen[j];k++)  
                coord[i][k]=coord[i][k]+dimoffset[i];
            }

          /* Set up sorted coordinate index */

          cindex[i]=(size_t *)malloc(dimlen[j]*sizeof(size_t));
          if (!cindex[i]) return MEMERR;
          for (k=0;k<dimlen[j];k++) cindex[i][k]=k; 
          quicksort(cindex[i],coord[i],0,dimlen[j]-1);
        }
    }

  /* Process item table: populate item instances with data from NetCDF file
     matching item specification */

  io.fillval=fillval;
  io.varscale=varscale;
  io.varoffset=varoffset;

  for (i=0;i<t->ndata;i++)

    { io.tabentry=&t->table[i];

      /* Determine selection flags for each coordinate vector:
         Non-zero values in vector <selflag[i]> give the destination grid point 
         numbers along the axis corresponding to the <ith> dimension of the 
         NetCDF variable (zeros indicate data not selected) */

      for (j=0;j<N_DIM;j++) if (io.dimflag[j])

        { g=&io.tabentry->gspec[j];
          sel=selflag[io.dimpos[j]];
          len=dimlen[io.dimpos[j]];

          /* Set selection flags for current dimension */

          if (!io.tabentry->dimflag[j]) for (k=0;k<len;k++) sel[k]=1;  
          else

            { /* Match by coordinate values within distance <rsearch> 
                 (<rsearch> assumed to be a maximum of half grid step).
                 Coordinate displacement from current grid reference 
                 is positive in direction of <step> */ 

              if (g->step>=0)
                { k=0;
                  kstep=1;
                }
              else
                { k=len-1;
                  kstep=-1;
                }

              gridref=g->first;
              num=1;
              endofgrid=0;
              endofdimv=0;

              while(!endofdimv && !endofgrid) 

                { /* Compare current grid point and NetCDF coordinate */
 
                  if (g->step>=0)
                    diff=coord[j][cindex[j][k]]-gridref;
                  else
                    diff=gridref-coord[j][cindex[j][k]];
                  if (g->rsearch==0)
                    { if (diff<0) c=-1;
                      else 
                        { if (diff>0) c=1;
                          else c=0;
                        }
                    }
                  else 
                    { if (diff<=-g->rsearch) c=-1;
                      else 
                        { if (diff>=g->rsearch) c=1;
                          else c=0;
                        }
                    }

                  if (c>0)
 
                    { /* Go to next grid point */

                      if (g->step==0) endofgrid=1;
                      else
                        { gridref+=g->step;
                          num++;
                          if (g->step>0) 
                            endofgrid=(gridref>g->last+0.5*g->step);
                          else 
                            endofgrid=(gridref<g->last+0.5*g->step);
                        }
                    }

                  else

                    { /* Update selection flag and get next coordinate */

                      if (c<0) sel[cindex[j][k]]=0;
                      else sel[cindex[j][k]]=num;
                      if (kstep>0) endofdimv=(k==len-1);
                      else endofdimv=(k==0);
                      k+=kstep;
                    } 
                }

              /* Set remaining selection flags off */

              while (!endofdimv) 
                { sel[cindex[j][k]]=0;
                  if (kstep>0) endofdimv=(k==len-1);
                  else endofdimv=(k==0);
                  k+=kstep;
                }
            }
        }

      /* Set block lengths for calculating data destinations in item 
         data space: block length for dimension j is the product of 
         axis lengths of all lower level (faster changing) dimensions 
         in the grid. */

      len=1;
      for (j=N_DIM-1;j>=0;j--) if (io.tabentry->dimflag[j]) 
        { io.blocklen[j]=len;
          len*=io.tabentry->gspec[j].n;
        }

      /* Extract data to item data table */

      status=extractdata_nc(&io,ndimv,dimlen,selflag,0,segstart,seglen);
      if (status!=OK) 
        { if (status==SPECIAL) *errtxt="NetCDF error on item data extraction";
          else *errtxt=NULL;
          return status;
        }
    }

  /* Free temporary storage specific to input file */

  for (i=0;i<N_DIM;i++) 
    { free(coord[i]);
      free(cindex[i]);
    }
  for (i=0;i<ndimv;i++) free(selflag[i]);

  status=nc_close(io.ncid);

  /* Report data extracted for each variable */

  if (logfile)
    { fprintf(logfile,"\n");
      status=writencvarreport(logfile,io.filecount,io.nvar,io.varactive,varname,
                              io.varreport);
      if (status!=OK) return status;
      if (fflush(logfile)==EOF) return OUTERR; 
    }

  return OK;
}


static void quicksort (size_t index[], double value[], int left,
                        int right)

{ int i,last;

  if (left>=right) return;
  swap(index,left,(left+right)/2);
  last=left;
  for (i=left+1;i<=right;i++)
    if (value[index[i]]<value[index[left]]) swap(index,++last,i);
  swap(index,left,last);
  quicksort(index,value,left,last-1);
  quicksort(index,value,last+1,right);
}


static void swap (size_t index[], int i, int j)

{ int temp;

  temp=index[i];
  index[i]=index[j];
  index[j]=temp;
}


static int writencvarreport (FILE *outfile, int ncfilecount, int nvar, 
                             unsigned char varactive[], char *varname[],                                     struct ncvarreportrec report[])

{ char *field1[5];
  double x1[5];
  struct datarec dout={x1,NULL,field1,NULL};
  int maxlen[5];
  int i,len,status;

  if (!outfile) return OK;

  fprintf(outfile," |   DATA SETS ACCESSED: %d\n |\n",ncfilecount);

  maxlen[0]=2;
  maxlen[1]=8;
  for (i=0;i<nvar;i++)
    { len=strlen(varname[i]);
      if (len>maxlen[1]) maxlen[1]=len;
    }
  maxlen[2]=11;
  maxlen[3]=15;
  maxlen[4]=13;

  dout.field[0]=" |";
  dout.field[1]="VARIABLE";
  dout.field[2]="DATA_VALUES";
  dout.field[3]="GRID_PTS_FILLED";
  dout.field[4]="GRID_PT_N_MAX";
  status=writedata(outfile,0,&dout,5,NULL,maxlen,NULL,DEFAULT,3,1);
  if (status!=OK) return OUTERR;

  dout.field[2]=NULL;
  dout.field[3]=NULL;
  dout.field[4]=NULL;
  for (i=0;i<nvar;i++) if (varactive[i])
    { dout.field[1]=varname[i];
      dout.x[2]=(double)report[i].totcount;
      dout.x[3]=(double)report[i].gridcount;
      dout.x[4]=(double)report[i].countmax;
      status=writedata(outfile,0,&dout,5,NULL,maxlen,NULL,DEFAULT,3,1);
      if (status!=OK) return OUTERR;
    }
 
  return OK;
}


static int readitemdata_nct (char *fname, enum itemcategory itemcat,
                            struct itemnamerec *item, char *dimname[],
                            struct itemtabrec *t, FILE *logfile,
                            char **ncerrfile, int *lineno, char **errtxt)

{ FILE *infile,ncfile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  struct datarec d={buff1.x,buff1.fieldlist,buff1.field,NULL};
  struct datarec dprev={buff2.x,buff2.fieldlist,buff2.field,NULL};
  char *reqvarname[N_NCTABVAR];
  char *varname[N_NCTABVAR];
  int reqvar[N_NCTABVAR];
  int varpos[N_NCTABVAR];
  double dimscale[N_DIM_NZ],*dimscalep;
  double dimoffset[N_DIM_NZ],*dimoffsetp;
  double varscale[N_ITEMVAR_MAX],*varscalep;
  double varoffset[N_ITEMVAR_MAX],*varoffsetp;
  double fillval;
  char *coordname[N_DIM_NZ],*p[N_DIM_NZ],*q[N_ITEMVAR_MAX],*s;
  int dimbyindex[N_DIM_NZ];
  char *itemvarname[N_ITEMVAR_MAX];
  int first,status,nreqvar,ninvar,nvar,ttb,i,j,numerr;

  *ncerrfile=NULL;
  *lineno=0;
  *errtxt=NULL;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;

  /* Check for variables of interest */

  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;
  if (ttb) return FMTERR;

  nreqvar=0;
  reqvarname[nreqvar++]="ncfile";
  reqvarname[nreqvar++]="fillval";
  for (i=0;i<N_DIM;i++) if (t->dimflag[i])
    { reqvarname[nreqvar++]=dimname[i];
      p[i]=malloc((2*strlen(dimname[i])+15)*sizeof(char));
      s=p[i];
      sprintf(s,"scale.%s",dimname[i]); 
      reqvarname[nreqvar++]=s;
      s+=strlen(s)+1;
      sprintf(s,"offset.%s",dimname[i]); 
      reqvarname[nreqvar++]=s;
    }
  for (i=0;i<item->nvar;i++) 
    { reqvarname[nreqvar++]=item->varname[i];
      q[i]=malloc((2*strlen(item->varname[i])+15)*sizeof(char));
      s=q[i];
      sprintf(s,"scale.%s",item->varname[i]); 
      reqvarname[nreqvar++]=s;
      s+=strlen(s)+1;
      sprintf(s,"offset.%s",item->varname[i]); 
      reqvarname[nreqvar++]=s;
    }

  if (indexlist(nreqvar,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  if (reqvar[0]<0)
    { *errtxt=reqvarname[0];
       return VARERR;
    }
  
  /* Set up default calibration coefficients to use if the relevant variables 
     are absent */

  dimscalep=NULL;
  dimoffsetp=NULL;
  j=2; 
  for (i=0;i<N_DIM;i++) if (t->dimflag[i])
    { dimscale[i]=1;
      dimoffset[i]=0;
      j++;
      if (reqvar[j++]>=0) dimscalep=dimscale; 
      if (reqvar[j++]>=0) dimoffsetp=dimoffset; 
    }

  varscalep=NULL;
  varoffsetp=NULL;
  for (i=0;i<item->nvar;i++) 
    { varscale[i]=1;
      varoffset[i]=0;
      j++;
      if (reqvar[j++]>=0) varscalep=varscale;
      if (reqvar[j++]>=0) varoffsetp=varoffset;
    }

  /* Get positions of variables of interest present and write names 
     to log file */ 

  if (logfile)

    { nvar=0;
      for (i=0;i<nreqvar;i++) if (reqvar[i]>=0) 
          { varpos[nvar]=reqvar[i];
            varname[nvar]=reqvarname[i];
            nvar++;
          }

      if (fprintf(logfile,
          "\n--- PROCESSING NETCDF FILE TABLE FOR ITEM %s (file: %s) ---\n",
                  item->name,fname)<0) return OUTERR;  
      if (fflush(logfile)==EOF) return OUTERR;
    }

  /* Process NetCDF file table records */

  first=1;

  while ((status=getdata(ninvar,NULL,infile,0,NULL,invarname.field,&d,NULL,
                         lineno))==OK)

    { /* Reference data from previous record as necessary */
 
      for (i=0;i<nvar;i++) 
        if (strcmp(d.field[varpos[i]],".")==0)
          { if (first) 
              { fprintf(stderr,"*** Invalid '%s' value ***\n",reqvarname[i]);
                return BADDATA;
              } 
            d.field[varpos[i]]=dprev.field[i]; 
          }

      /* Write current record to log file */

      if (logfile)
        { fprintf(logfile,"\n");
          status=writenctrec(logfile,nvar,varname,&d,varpos);
          if (status!=OK) return status;
        }

      /* Set fill value */

      j=1;
      if (reqvar[j]>=0) 
        { d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&numerr);
          if (numerr) fillval=OBJF_NODATA;
          else fillval=d.x[reqvar[j]]; 
        }
      else fillval=OBJF_NODATA;

      /* Set alternative names and/or calibration coefficients for 
         NetCDF co-ordinate variables, for each dimension. 
         Alternative names of the form num.<nc_dimension> cause 
         dimension <nc_dimension> to be accessed directly using dimension
         indices */

      j++;
      for (i=0;i<N_DIM;i++) if (t->dimflag[i])
        { dimbyindex[i]=0; 
          if (reqvar[j]>=0) 
            { coordname[i]=d.field[reqvar[j]]; 
              if (strcmp(coordname[i],NODATA_STR)==0) coordname[i]=dimname[i];
              else if (strncmp(coordname[i],"num.",4)==0)
                { coordname[i]+=4;
                  dimbyindex[i]=1; 
                } 
            }
          else coordname[i]=dimname[i];
          j++;
          if (reqvar[j]>=0) 
            { d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&numerr);
              if (numerr) dimscale[i]=1;
              else dimscale[i]=d.x[reqvar[j]];
            }
          j++;
          if (reqvar[j]>=0) 
            { d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&numerr);
              if (numerr) dimoffset[i]=0;
              else dimoffset[i]=d.x[reqvar[j]];
            }
          j++;
        }

      /* Set alternative names and/or calibration coefficients for 
         item variables */

      for (i=0;i<item->nvar;i++) 
        { if (reqvar[j]>=0) 
            { itemvarname[i]=d.field[reqvar[j]];
              if (strcmp(itemvarname[i],NODATA_STR)==0)
                itemvarname[i]=item->varname[i];
            }
          else itemvarname[i]=item->varname[i];
          j++;
          if (reqvar[j]>=0)
            { d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&numerr);
              if (numerr) varscale[i]=1;
              else varscale[i]=d.x[reqvar[j]];
            }
          j++;
          if (reqvar[j]>=0)
            { d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&numerr);
              if (numerr) varoffset[i]=0;
              else varoffset[i]=d.x[reqvar[j]];
            }
          j++;
        }

      /* Get NetCDF data */

      status=readitemdata_nc(d.field[reqvar[0]],first,itemcat,item,coordname,
                             dimbyindex,itemvarname,t,fillval,dimscalep,
                             dimoffsetp,varscalep,varoffsetp,logfile,errtxt);
      if (status!=OK) 
        { *ncerrfile=d.field[reqvar[0]];
          *lineno=0; 
          return status;
        }

      copydata(&dprev,&d,nvar,varpos);
      first=0;
    }

  if (status!=EOF) return status;

  if (logfile) 
    { fprintf(logfile,"\n");
      if (fflush(logfile)==EOF) return OUTERR;
    }

  if (infile) fclose(infile);

  /* Free variable name workspace */

  for (i=0;i<N_DIM;i++) if (t->dimflag[i])
    free(p[i]);

  for (i=0;i<item->nvar;i++) 
    free(q[i]);

  return OK;
}


static int writenctrec (FILE *outfile, int nvar, char *varname[], 
                        struct datarec *d, int varpos[])

{ char *field1[2];
  struct datarec dout={NULL,NULL,field1,NULL};
  int maxlen[2];
  int i,len,status;

  if (!outfile) return OK;
 
  maxlen[0]=4;
  for (i=0;i<nvar;i++)
    { len=strlen(varname[i]);
      if (len>maxlen[0]) maxlen[0]=len;
    }
  maxlen[1]=1;

  for (i=0;i<nvar;i++)
    { dout.field[0]=varname[i];
      dout.field[1]=d->field[varpos[i]];
      status=writedata(outfile,0,&dout,2,NULL,maxlen,NULL,DEFAULT,3,1);
      if (status!=OK) return OUTERR;
    }

  return OK;
}
  

static int extractdata_nc (struct nciorec *io, int ndim, size_t dimlen[], 
                           size_t *selflag[], int level,
                           size_t segstart[], size_t seglen[]) 

/* Extract data from variable(s) <io->varid> in data set <io->ncid> as
   specified by vectors of selection flags (selflag), one for each dimension
   for which subranging is required.

   A contiguous range of selected data along a single dimension is called a
   segment.
   <ndim> segments (one for each dimension) define a cuboid of data to be
   extracted.
   Each cuboid is represented by the path from the root to a bottom level node
   of an inverted tree structure, where each hierarchical level corresponds to 
   a different dimension.
   The top level is associated with the slowest changing dimension.

   Starting at the top level (level 0), 'extractdata_nc' is called 
   recursively to process the sub-tree at each node.
   At the bottom level 'mapcuboid_nc' is called to output the required data.
  
   On input segstart & seglen hold the current segment start & length for
   higher level dimensions (if any). */

{ size_t *sel,i;
  int inseg,status;

  segstart[level]=0;
  sel=selflag[level];
  inseg=0;

  if (!sel)

    { seglen[level]=dimlen[level];
      if (level+1<ndim)
        status=extractdata_nc(io,ndim,dimlen,selflag,level+1,segstart,seglen);
      else status=mapcuboid_nc(io,ndim,selflag,segstart,seglen);
      if (status!=OK) return status;
    }

  else

    { seglen[level]=0;

      for (i=0;i<=dimlen[level];i++)

        { if (inseg)
            { if (sel[i]) seglen[level]++;
              else
                { inseg=0;
                  if (level+1<ndim)
                    status=extractdata_nc(io,ndim,dimlen,selflag,level+1,
                                          segstart,seglen);
                  else status=mapcuboid_nc(io,ndim,selflag,segstart,seglen);
                  if (status!=OK) return status;
                }
            }

          else if (sel[i])
            { inseg=1;
              segstart[level]=i;
              seglen[level]=1;
            }
        }
    }

  return OK;
}


static int mapcuboid_nc (struct nciorec *io, int ndim, size_t *selflag[], 
                         size_t segstart[], size_t seglen[]) 

/* Reads the NetCDF data for the cuboid specified by <segstart> and <seglen>
   and maps to item data space according to values in the selection flag 
   vectors <selflag>.
   The values in vector <selflag[i]> give the destination grid point numbers 
   along the axis corresponding to the <ith> dimension of the NetCDF variable.
*/

{ size_t cuboidsize;
  ptrdiff_t imap[N_DIM_NZ];
  size_t segfin[N_DIM_NZ];
  size_t isel[N_DIM_NZ];
  size_t offset[N_DIM_NZ];
  size_t len,num,kin,kout;
  int i,j,m,status;
  double *dp;
  OBJF_FLOAT_TYPE *value;
  int *count;
  struct ncvarreportrec *report;

  cuboidsize=1;
  for (i=0;i<ndim;i++) cuboidsize*=seglen[i];
  
  dp=(double *)malloc(cuboidsize*sizeof(double)); 
  if (!dp) return MEMERR;

  /* Create input map for cuboid to get dimensions in objective function 
     specific order */

  len=1;
  for (j=N_DIM-1;j>=0;j--) if (io->dimflag[j])
    { imap[io->dimpos[j]]=len;
      len*=seglen[io->dimpos[j]];
    }

  for (i=0;i<io->nvar;i++) if (io->varactive[i]) 

    { /* Read variable data for cuboid */ 

      status=nc_get_varm_double(io->ncid,io->varid[i],segstart,seglen,
                                NULL,imap,dp);
      if (status!=NC_NOERR) return SPECIAL;

      /* Initialize indices in selection flag vectors to start of cuboid */

      for (m=0;m<ndim;m++) 
        { isel[m]=segstart[m];
          segfin[m]=segstart[m]+seglen[m]-1;
        }

      /* Initialize offsets for item data space */
       
      for (j=0;j<N_DIM;j++) if (io->dimflag[j])
        { m=io->dimpos[j]; 
          num=selflag[m][isel[m]];
          offset[j]=(num-1)*io->blocklen[j];
        }

      value=io->tabentry->value[i];
      count=io->tabentry->count[i];
      report=&io->varreport[i];

      for (kin=0;kin<cuboidsize;kin++) 

        { if (dp[kin]!=io->fillval) 

            { /* Apply variable calibrations if any */

              if (io->varscale && io->varscale[i]!=1)
                dp[kin]=dp[kin]*io->varscale[i];
              if (io->varoffset && io->varoffset[i]!=0)
                dp[kin]=dp[kin]+io->varoffset[i];

              /* Calculate destination index in item data space using
                 current offsets */ 

              kout=0;
              for (j=0;j<N_DIM;j++) if (io->dimflag[j]) 
                kout+=offset[j];

              /* Copy data value (multiple values are averaged) */

              if (count[kout]==0) 
                { value[kout]=(OBJF_FLOAT_TYPE)dp[kin];
                  report->gridcount++;
                }
              else value[kout]=
                (count[kout]*value[kout]+(OBJF_FLOAT_TYPE)dp[kin])/
                (count[kout]+1);
              count[kout]++;
              report->totcount++;
              if (count[kout]>report->countmax) report->countmax=count[kout]; 
            }

          /* Update selection flag vector indices and recalculate
             data space offsets for each changed dimension */

          if (kin<cuboidsize-1) 

            { j=N_DIM-1; 
              while (!io->dimflag[j]) j--;

              m=io->dimpos[j]; 

              isel[m]++;
              num=selflag[m][isel[m]];
              offset[j]=(num-1)*io->blocklen[j];

              while (isel[m]>segfin[m] && j>0)

                { /* Reset index for current dimension */
 
                  isel[m]=segstart[m];
                  num=selflag[m][isel[m]];
                  offset[j]=(num-1)*io->blocklen[j];

                  /* Carry over index increment to higher level dimension  */

                  j--;
                  while (!io->dimflag[j]) j--;

                  if (j>=0)       

                    { m=io->dimpos[j]; 

                      isel[m]++;
                      num=selflag[m][isel[m]];
                      offset[j]=(num-1)*io->blocklen[j];
                    }
                }
            }
        }
     }

  free(dp);

  return OK;
}


#endif  /* End of NetCDF-related function defininitons */


static int makesinglecase (struct casetabrec *t)

{ int i;

  t->ndata=1;
  t->table=(struct caseinputrec *)malloc(sizeof(struct caseinputrec));
  if (!t->table) return MEMERR;
  t->table[0].casenum=1;
  t->table[0].fieldlist=NULL;
  for (i=0;i<N_CASEVAR;i++) t->table[0].caseid[i]=NODATA_STR;
  for (i=0;i<N_ITEM;i++) 
    { if (t->keyflag[i]) t->table[0].itemkey[i]=NODATA_STR;
      t->table[0].item[i]=NULL;
    }
  return OK; 
}


static int crossref (struct casetabrec *casetab, int itemtype, 
                     struct itemtabrec *itemtab, int *errcasenum)

{ int match,def,i,j,status;
 
  /* Cross-referencing is done by item key if this is present in the case 
     table. If not, then it is done by case variables if case variables are 
     used as identifiers in the item table (NB if item key is present in the 
     item table then any case variables therein are not used). Otherwise there
     is no explicit cross-referencing but if there is only 1 item in the item 
     table then this item is used for all cases. */ 

  if (casetab->keyflag[itemtype] )

    { /* Cross-reference by itemkey */  

      if (!itemtab->idspec.itemkey) 

        { /* No key in item table.
             If only 1 item in item table then treat this as default and 
             use for all cases. Otherwise cross-referencing fails. */  

          if (itemtab->ndata==1)
            for (i=0;i<casetab->ndata;i++)
              { casetab->table[i].itemnum[itemtype]=1;
                casetab->table[i].item[itemtype]=&itemtab->table[0];
              }
          else 
            { *errcasenum=1;
              return SPECIAL;
            }
        }

      else

        { if ( strcmp(itemtab->table[0].itemkey,NODATA_STR)==0) def=0;
          else def=-1;
          for (i=0;i<casetab->ndata;i++)
            { match=0;
              j=0;
              while (!match && j<itemtab->ndata)
                { match=(strcmp(itemtab->table[j].itemkey,
                                casetab->table[i].itemkey[itemtype])==0); 
                  if (!match) j++;
                }
              if (match) 
                { casetab->table[i].itemnum[itemtype]=j+1;
                  casetab->table[i].item[itemtype]=&itemtab->table[j];
                }
              else if (def>=0) 
                { casetab->table[i].itemnum[itemtype]=def+1;
                  casetab->table[i].item[itemtype]=&itemtab->table[def];
                }
              else 
                { *errcasenum=i+1;
                  return SPECIAL; 
                }
            }
        }  
    }

  else if (itemtab->idspec.ncvar!=0) 

    { /* Cross-reference by case variable(s) */ 

      status=xrefgroup(casetab,0,itemtype,itemtab,0,0,errcasenum);
      if (status!=OK) return status;
    }

  else if (itemtab->ndata==1)

    for (i=0;i<casetab->ndata;i++)
      { casetab->table[i].itemnum[itemtype]=1;
        casetab->table[i].item[itemtype]=&itemtab->table[0];
      }

  else 
    { *errcasenum=1;
      return SPECIAL;
    }

  return OK;
} 


static int xrefgroup (struct casetabrec *casetab, int currcase, 
                        int itemtype, struct itemtabrec *itemtab, 
                        int curritem, int ngrpvar, int *errcasenum)

{ char casegrp[LINE_MAX+1];
  char itemgrp[LINE_MAX+1];
  char *currcasegrp[N_CASEVAR_NZ]; 
  char *curritemgrp[N_CASEVAR_NZ];
  struct datarec dcase={NULL,NULL,NULL,NULL}; 
  struct datarec ditem={NULL,NULL,NULL,NULL}; 
  char *p,*q;
  int *cvar;
  int ncvar,subgrpvar,ingroup_case=1,ingroup_item=1,c,match,i,j,k,def,jfirst,
      status;

  /* This function looks for a matching or default subgroup within
     the current case table and item table groups starting at indices 
     'currcase' and 'curritem'. The level in the case variable hierarchy is 
     defined by 'ngrpvar' which gives the number of case variables used to 
     identify the current group. */ 

  ncvar=itemtab->idspec.ncvar;
  cvar=itemtab->idspec.cvar;
  i=currcase;
  j=curritem;
  jfirst=curritem;
 
  /* Record ids for current case group and current item group
     (These may be different because the latter could be a default) */ 

  p=casegrp;
  q=itemgrp;
  for(k=0;k<ngrpvar;k++) 
    { strcpy(p,casetab->table[i].caseid[cvar[k]]);
      currcasegrp[k]=p;
      p+=strlen(p)+1;
      strcpy(q,itemtab->table[j].caseid[cvar[k]]);
      curritemgrp[k]=q;
      q+=strlen(q)+1;
    }

  /* Check for a default subgroup in the item table */

  subgrpvar=cvar[ngrpvar]; 
  if (strcmp(itemtab->table[j].caseid[subgrpvar],NODATA_STR)==0) def=j;
  else def=-1;

  do
    { /* For each case table entry in the current group:- */

      /* First check for matching subgroup within current item table group */
 
      do 
        { ditem.field=&itemtab->table[j].caseid[subgrpvar];
          dcase.field=&casetab->table[i].caseid[subgrpvar];
          c=datacomp(1,&ditem,NULL,&dcase,NULL,NUMCOMP); 
          match=0;
          if (c<0) 
            { if (++j==itemtab->ndata) ingroup_item=0;
              else for (k=0;k<ngrpvar;k++)
                if (strcmp(itemtab->table[j].caseid[cvar[k]],
                           curritemgrp[k])!=0) 
                  ingroup_item=0; 
            }
          else if (c>0) ingroup_item=0;
          else match=1;
        }
      while (!match && ingroup_item); 

      if (match)

        { /* Matching subgroup found so process next level down 
             or if all levels done then update reference in case table */
 
          if (ngrpvar+1<ncvar) 
            { status=
                xrefgroup(casetab,i,itemtype,itemtab,j,ngrpvar+1,errcasenum);
              if (status!=OK) return status;
            }
          else 
            { casetab->table[i].itemnum[itemtype]=j+1;
              casetab->table[i].item[itemtype]=&itemtab->table[j];
            }
        }
      
      else

        { /* No matching subgroup
             If there is no default subgroup then cross-referencing fails */

          if (def<0) 
            { *errcasenum=i+1; 
              return SPECIAL;
            }

          /* Match with default subgroup */

          if (ngrpvar+1<ncvar)
            { status=
                xrefgroup(casetab,i,itemtype,itemtab,def,ngrpvar+1,errcasenum);
              if (status!=OK) return status;
            }
          else
            { casetab->table[i].itemnum[itemtype]=def+1;
              casetab->table[i].item[itemtype]=&itemtab->table[def];
            }
        }

      /* Next case */

      if (++i==casetab->ndata) ingroup_case=0;
      else 
        { ingroup_case=1;
          for (k=0;k<ngrpvar;k++) 
            if (strcmp(casetab->table[i].caseid[k],currcasegrp[k])!=0) 
              ingroup_case=0; 
        }

      /*  Case table is not necessarily sorted by the item id variables  
          so item index is reset */

      if (ingroup_case) 
        { j=jfirst;
          ingroup_item=1;
        }
    }

  while (ingroup_case);

  return OK;     
}


static int writectrlrec (FILE *outfile, struct ctrlrec *d, 
                         struct objfnamerec *objf)

{ char *label, *name;
  enum itemcategory itemcat;
  int fw,len,i,j;

  if (!outfile) return OK;

  fw=46;         /* strlen("Function eval. file - Powell (funcpowellf) .. ") */
  for (i=0;i<N_PSET;i++)
    { len=strlen(objf->item[i].name)+30;
      if (len>fw) fw=len;
    }
  j=N_PSET;
  for (i=j;i<j+N_MATITEM;i++)
    { len=strlen(objf->item[i].name)+23;
      if (len>fw) fw=len;
    }
  j=N_PSET+N_MATITEM;
  for (i=j;i<j+N_GDOMITEM;i++)
    { len=strlen(objf->item[i].name)+31;
      if (len>fw) fw=len;
    }
  j=N_PSET+N_MATITEM+N_GDOMITEM;
  for (i=j;i<j+N_NGDOMITEM;i++)
    { len=strlen(objf->item[i].name)+35;
      if (len>fw) fw=len;
    }
  for (i=0;i<N_OUTTAB;i++)
    { len=strlen(objf->outtab[i].name)+24;
      if (d->outf[i].name && d->outvarf[i].name) len+=10;
      if (len>fw) fw=len;
    }

#if OPT_ENABLED == 1 

  if (d->ioptpset>=0)

    { len=fprintf(outfile,"Optimizable parameter set (optpset) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
      if (fprintf(outfile," : %s\n",objf->item[d->ioptpset].name)<0)
        return OUTERR;

      if (d->optconfigf.name)
        { if (d->optconfigf.resident) label="(RESIDENT)";
          else
            { if (d->optconfigf.flag) label="(LOGGED)";
              else label="(UNLOGGED)";
            }
          len=fprintf(outfile,"Optimizer configuration file (optconfigf) ");
          if (len<0) return OUTERR;
          for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
          if (fprintf(outfile," : %s %s\n", d->optconfigf.name,label)<0)
            return OUTERR;
        }

      if (d->freeparmf.resident) label="(RESIDENT)";
      else 
        { if (d->freeparmf.flag) label="(LOGGED)";
          else label="(UNLOGGED)";
        }
      len=fprintf(outfile,"Free parameter file (freeparmf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
      if (fprintf(outfile," : %s %s\n", d->freeparmf.name,label)<0)
        return OUTERR;

      if (d->parminitf.name)
        { if (d->parminitf.resident) label="(RESIDENT)";
          else 
            { if (d->parminitf.flag) label="(LOGGED)";
              else label="(UNLOGGED)";
            }
          len=fprintf(outfile,"Parameter initialization file (parminitf) ");
          if (len<0) return OUTERR;
          for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
          if (fprintf(outfile," : %s %s\n",d->parminitf.name,label)<0)
            return OUTERR;
        }

      if (!d->funcgaf.name || !d->funcgaf.flag) name=NODATA_STR;
      else name=d->funcgaf.name;
      len=fprintf(outfile,"Function eval. file - G.A. (funcgaf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
      if (fprintf(outfile," : %s\n",name)<0)
        return OUTERR;

      if (!d->funcpowellf.name || !d->funcpowellf.flag) name=NODATA_STR;
      else name=d->funcpowellf.name;
      len=fprintf(outfile,"Function eval. file - Powell (funcpowellf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
      if (fprintf(outfile," : %s\n",name)<0)
        return OUTERR;

      if (!d->newparmf.name || !d->newparmf.flag) name=NODATA_STR; 
      else name=d->newparmf.name;
      len=fprintf(outfile,"New parameter file (parmnewf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
      if (fprintf(outfile," : %s\n",name)<0) 
        return OUTERR;

      if (!d->optreportf.name || !d->optreportf.flag) name=NODATA_STR;
      else name=d->optreportf.name;
      len=fprintf(outfile,"Optimizer report file (reportf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
      if (fprintf(outfile," : %s\n",name)<0)
        return OUTERR;

      if (fprintf(outfile,"\n")<0) return OUTERR;
    }

#endif

  for (i=0;i<N_ITEM;i++) if (d->itemf[i].name)

    { if (i<N_PSET) itemcat=PSET;
      else if (i<N_PSET+N_MATITEM) itemcat=MATRIX;
      else if (i<N_PSET+N_MATITEM+N_GDOMITEM) itemcat=GDOMITEM;
      else itemcat=NGDOMITEM;

      if (d->itemf[i].resident) label="(RESIDENT)";
      else
        { if (d->itemf[i].flag==1) 
            { if (d->itemf[i].createfile) label="(LOGGED,DATOUT)";
              else label="(LOGGED)";
            }
          else
            { if (d->itemf[i].flag>1) 
                { if (itemcat==PSET) label="(LOGGED)";
                  else 
                    { if (d->itemf[i].flag==2) 
                        { if (d->itemf[i].createfile) label="(LOGGED+,DATOUT)";
                          else label="(LOGGED+)";
                        }
                      else
                        { if (d->itemf[i].createfile) label="(LOGGED++,DATOUT)";
                          else label="(LOGGED++)";
                        }
                    }
                }
              else 
                { if (d->itemf[i].createfile) label="(UNLOGGED,DATOUT)"; 
                  else label="(UNLOGGED)";
                }
            }
        }

      switch (itemcat)
        { case PSET: 
            len=fprintf(outfile,"Parameter set item '%s' file ",
                        objf->item[i].name);
            if (len<0) return OUTERR;
            for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
            if (fprintf(outfile," : %s %s\n",d->itemf[i].name,label)<0)
              return OUTERR;
            break;
          case MATRIX: 
            len=fprintf(outfile,"Matrix item '%s' file ",objf->item[i].name);
            if (len<0) return OUTERR;
            for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
            if (fprintf(outfile," : %s %s\n",d->itemf[i].name,label)<0)
              return OUTERR;
            break;
          case GDOMITEM: 
            len=fprintf(outfile,"Gridded domain item '%s' file ",
                        objf->item[i].name);
            if (len<0) return OUTERR;
            for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
            if (fprintf(outfile," : %s %s\n",d->itemf[i].name,label)<0)
              return OUTERR;
            break;
          case NGDOMITEM: 
            len=fprintf(outfile,"Non-gridded domain item '%s' file ",
                        objf->item[i].name);
            if (len<0) return OUTERR;
            for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
            if (fprintf(outfile," : %s %s\n",d->itemf[i].name,label)<0)
              return OUTERR;
            break;
        }
    }

  if (N_CASEVAR>0 && d->casef.name)
    { if (d->casef.resident) label="(RESIDENT)";
      else
        { if (d->casef.flag) label="(LOGGED)";
          else label="(UNLOGGED)";
        }
      len=fprintf(outfile,"Case file ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
      if (fprintf(outfile," : %s %s\n",d->casef.name,label)<0)  
        return OUTERR;
    }

  if (fprintf(outfile,"\n")<0) return OUTERR;
      
  for (i=0;i<N_OUTTAB;i++)
    { if (!d->outf[i].name || !d->outf[i].flag) name=NODATA_STR; 
      else name=d->outf[i].name;
      len=fprintf(outfile,"Output table '%s' file ",objf->outtab[i].name); 
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR; 
      if (fprintf(outfile," : %s\n",name)<0) return OUTERR; 
    }

  if (fprintf(outfile,"\n")<0) return OUTERR;

  for (i=0;i<N_OUTTAB;i++)
    { if (d->outf[i].name && d->outf[i].flag && d->outvarf[i].name)
        { if (d->outvarf[i].resident) label="(RESIDENT)";
          else label="";
          len=fprintf(outfile,"Output table '%s' variables file ",
                      objf->outtab[i].name);
          if (len<0) return OUTERR;
          for (j=0;j<fw-len-1;j++) if (fprintf(outfile,".")<0) return OUTERR;
          if (fprintf(outfile," : %s %s\n",d->outvarf[i].name,label)<0)
            return OUTERR;
        }
    }

  if (d->logf.name)
    { if (fprintf(outfile,"\n")<0) return OUTERR;
      len=fprintf(outfile,"GFAn log file (gfanlogf) ");
      if (len<0) return OUTERR;
      for (j=0;j<fw-len-1;j++) 
        if (fprintf(outfile,".")<0) return OUTERR; 
      if (fprintf(outfile," : %s\n",d->logf.name)<0) return OUTERR;
    }

  return OK;
}


static int writelogcasetab (FILE *outfile, struct casetabrec *t, 
                            char *casevarname[], char *keyvarname[], 
                            char *itemnumvarname[])

{ char *varname[N_CASETABVAR+N_ITEM+1]; 
  int fw[N_CASETABVAR+N_ITEM+1];
  struct caseinputrec *tabentry;
  int status,nvar,npage,pageno,i,j,k,ipage;

  if (!outfile) return OK;

  /* Set variable names 
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  nvar=0;
  varname[nvar]="casenum";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  for (j=0;j<N_CASEVAR;j++) varname[nvar++]=casevarname[j];
  for (j=0;j<N_ITEM;j++)
    if (t->keyflag[j] && t->itemactive[j]) varname[nvar++]=keyvarname[j];
  if (itemnumvarname)
    for (j=0;j<N_ITEM;j++) if (t->itemactive[j]) 
      { varname[nvar]=itemnumvarname[j];
        for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
        nvar++;
      }

  npage=1+(int)((t->ndata-1)/NPAGEREC_MAX);
  pageno=1; 
  i=0;
  while (i<t->ndata)
          
    { /* Set up table data on current logical page */

      for (ipage=0; ipage<NPAGEREC_MAX && i<t->ndata ;ipage++)

        { tabentry=&t->table[i];

          k=0;
          dpage[ipage].x[k++]=(double)tabentry->casenum;
          for (j=0;j<N_CASEVAR;j++) 
            dpage[ipage].field[k++]=tabentry->caseid[j];
          for (j=0;j<N_ITEM;j++) 
            if (t->keyflag[j] && t->itemactive[j]) 
              dpage[ipage].field[k++]=tabentry->itemkey[j];
          if (itemnumvarname)
            for (j=0;j<N_ITEM;j++) 
              if (t->itemactive[j])
                dpage[ipage].x[k++]=(double)tabentry->itemnum[j];

          i++;
        }

      /* Write logical page */

      setcolwidths(nvar,varname,ipage,dpage,NULL,CWIDTH_MAX,fw);

      if (i>NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage>1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,1,varname,fw,NULL,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status!=OK) return status;
    }

  return OK;
}


static int writelogitemtab (FILE *outfile, struct itemtabrec *t, 
                            char *numvarname, struct itemnamerec *item, 
                            struct gspecvarnamerec gspecvarname[], 
                            char *dimflagvarname[])

{ char *varname[NFIELD_MAX];
  int sigdig[NFIELD_MAX];
  int fw[NFIELD_MAX];
  struct itemrec *tabentry;
  struct gridspecrec *g;
  int status,nvar,npage,pageno,nidvar,i,j,k,ipage,printflag;
  double size;

  if (!outfile) return OK;

  /* Write variable descriptions */
 
  if (t->itemcat==PSET)           
    { printflag=0;
      for (i=0;i<item->nvar;i++)
        if (t->varactive[i] && item->vartext[i])
          { fprintf(outfile,"%s: %s\n",item->varname[i],item->vartext[i]);
            printflag=1;
          }
      if (printflag) fprintf(outfile,"\n");
    }

  /* Set variable names */

  nvar=0;
  varname[nvar]=numvarname;
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  if (t->idspec.itemkey) varname[nvar++]=t->idspec.varname[0]; 
  else 
    for (i=0;i<t->idspec.ncvar;i++) varname[nvar++]=t->idspec.varname[i];
  nidvar=nvar;

  switch (t->itemcat)
    { case PSET:
        for (j=0;j<item->nvar;j++) 
          if (t->varactive[j]) varname[nvar++]=item->varname[j];
        break;
      case MATRIX:
        varname[nvar++]="nrow";
        varname[nvar++]="ncol";
        varname[nvar++]="size";
        break;
      case GDOMITEM:
        for (j=0;j<N_DIM;j++) if (t->dimflag[j])
          { varname[nvar++]=gspecvarname[j].first;
            varname[nvar++]=gspecvarname[j].last;
            varname[nvar++]=gspecvarname[j].step;
            varname[nvar++]=gspecvarname[j].n;
            varname[nvar++]=gspecvarname[j].min;
            varname[nvar++]=gspecvarname[j].max;
          }
        varname[nvar++]="size";
        break;
      case NGDOMITEM:
        varname[nvar++]="npoint";
        for (j=0;j<N_DIM;j++) if (t->dimflag[j])
          varname[nvar++]=dimflagvarname[j];
        break;
    }

  /* Remove string fields in page buffer for numerical data to force
     use of values in number field */

  for (i=0;i<NPAGEREC_MAX;i++) 
    for (j=nidvar;j<nvar;j++) dpage[i].field[j]=NULL;

  npage=1+(int)((t->ndata-1)/NPAGEREC_MAX);
  pageno=1; 
  i=0;
  while (i<t->ndata)

    { /* Set up table data on current logical page
         (logical page may span multiple physical pages) */

      for (ipage=0; ipage<NPAGEREC_MAX && i<t->ndata ;ipage++)

        { tabentry=&t->table[i];

          k=0;
          dpage[ipage].x[k++]=(double)i+1;
          if (t->idspec.itemkey)
            dpage[ipage].field[k++]=tabentry->itemkey;
          else
            for (j=0;j<t->idspec.ncvar;j++)
              dpage[ipage].field[k++]=tabentry->caseid[t->idspec.cvar[j]];
          
          switch (t->itemcat)
            { case PSET:
                for (j=0;j<item->nvar;j++) if (t->varactive[j])
                  { if (tabentry->parm[j]==OBJF_NODATA)
                      dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)tabentry->parm[j];
                  }
                break;
              case MATRIX:
                dpage[ipage].x[k++]=(double)tabentry->nrow;
                dpage[ipage].x[k++]=(double)tabentry->ncol;
                dpage[ipage].x[k++]=(double)(tabentry->nrow*tabentry->ncol);
                break;
              case GDOMITEM:
                size=1;
                for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
                  { g=&tabentry->gspec[j]; 
                    if (g->first==OBJF_NODATA) dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)g->first;
                    if (g->last==OBJF_NODATA) dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)g->last;
                    if (g->step==OBJF_NODATA) dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)g->step;
                    dpage[ipage].x[k++]=(double)g->n;
                    if (g->min==OBJF_NODATA) dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)g->min;
                    if (g->max==OBJF_NODATA) dpage[ipage].x[k++]=NODATA;
                    else dpage[ipage].x[k++]=(double)g->max;
                    size*=g->n;
                  }
                dpage[ipage].x[k++]=size;
                break;
              case NGDOMITEM:
                dpage[ipage].x[k++]=(double)tabentry->npoint;
                for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
                  dpage[ipage].x[k++]=(double)tabentry->dimflag[j];
                break;
            }

          i++;
        }

      /* Write page */

      for (j=0;j<nidvar;j++) sigdig[j]=DEFAULT;
      switch (t->itemcat)
        { case PSET:
            for (j=nidvar;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
            break;
          case MATRIX:
          case NGDOMITEM:
            for (j=nidvar;j<nvar;j++) sigdig[j]=DEFAULT;
            break;
          case GDOMITEM:
            j=nidvar;
            for (k=0;k<N_DIM;k++) if (t->dimflag[k])
              { sigdig[j++]=OBJF_SIGDIG_DEF;
                sigdig[j++]=OBJF_SIGDIG_DEF;
                sigdig[j++]=OBJF_SIGDIG_DEF;
                sigdig[j++]=DEFAULT;
                sigdig[j++]=OBJF_SIGDIG_DEF;
                sigdig[j++]=OBJF_SIGDIG_DEF;
              }
            break;
        }  

      setcolwidths(nvar,varname,ipage,dpage,sigdig,CWIDTH_MAX,fw);

      if (i>NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage>1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,1,varname,fw,sigdig,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status!=OK) return status;
    }

  return OK;
}


static int writelogitemdatatab (FILE *outfile, int noutmax, 
                                struct itemtabrec *t, 
                                char *numvarname, struct itemnamerec *item, 
                                char *dimname[])

{ char *varname[NFIELD_MAX];
  int sigdig[NFIELD_MAX];
  int fw[NFIELD_MAX];
  struct itemrec *tabentry;
  int count[N_DIM_NZ];
  OBJF_FLOAT_TYPE dim[N_DIM_NZ];
  unsigned long nval,ival,nrec;
  int status,nvar,nidvar,npage,pageno,newitem,dimupdate,
      i,j,k,row,col,ipage,printflag,trunc;

  if (!outfile) return OK;

  /* Write variable descriptions */

  printflag=0;
  for (i=0;i<item->nvar;i++)
    if (t->varactive[i] && item->vartext[i]) 
       { fprintf(outfile,"%s: %s\n",item->varname[i],item->vartext[i]);
         printflag=1;
       }
  if (printflag) fprintf(outfile,"\n");

  if (noutmax==0) 
    { fprintf(outfile,"*** Values not logged ***\n\n");
      return OK;
    }

  /* Set variable names
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  nvar=0;
  varname[nvar]=numvarname;
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  varname[nvar]="num";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  if (t->idspec.itemkey) varname[nvar++]=t->idspec.varname[0];
  else
    for (i=0;i<t->idspec.ncvar;i++) varname[nvar++]=t->idspec.varname[i];
  nidvar=nvar;

  switch (t->itemcat)
    { case MATRIX:
        varname[nvar++]="row";
        varname[nvar++]="col";
        break;
      case GDOMITEM:
      case NGDOMITEM:
        for (j=0;j<N_DIM;j++) 
          if (t->dimflag[j]) varname[nvar++]=dimname[j];
        break;
    }

  for (i=0;i<item->nvar;i++)
    if (t->varactive[i]) varname[nvar++]=item->varname[i];

  for (i=0;i<NPAGEREC_MAX;i++)
    for (j=nidvar;j<nvar;j++) dpage[i].field[j]=NULL;

  /* Find total number of records and pages */

  nrec=0;
  trunc=0;
  for (i=0;i<t->ndata;i++) 
    { tabentry=&t->table[i];
      switch (t->itemcat)
        { case MATRIX:
            nval=tabentry->nrow*tabentry->ncol;
            break;
          case GDOMITEM:
            nval=1;
            for (j=0;j<N_DIM;j++) if (t->dimflag[j])
              nval*=tabentry->gspec[j].n;
            break;
          case NGDOMITEM:
            nval=tabentry->npoint;
            break;
        }
      if (noutmax>0 && nval>noutmax)  
        { nrec+=noutmax;
          trunc=1;
        }
      else
        nrec+=nval;
    }
  npage=1+(int)((nrec-1)/NPAGEREC_MAX);

  if (trunc) 
    { if (t->ndata>1)
        fprintf(outfile,
          "*** Logging first %d records of each instance ... ***\n\n",noutmax);
      else
        fprintf(outfile,"*** Logging first %d records ... ***\n\n",noutmax);
    }

  newitem=1;
  pageno=1; 
  i=0;
  while (i<t->ndata)

    { /* Set up table data on current logical page
         (logical page may span multiple physical pages) */

      ipage=0;
      while (ipage<NPAGEREC_MAX && i<t->ndata) 

        { if (newitem)

            { tabentry=&t->table[i];
              newitem=0;
              ival=0;
              switch (t->itemcat)
                { case MATRIX:
                    nval=tabentry->nrow*tabentry->ncol;
                    row=1;
                    col=1;
                    break;
                  case GDOMITEM:
                    nval=1;
                    for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
                      { nval*=tabentry->gspec[j].n; 
                        count[j]=0;
                        if (tabentry->dimflag[j]) 
                          dim[j]=tabentry->gspec[j].first;
                        else dim[j]=OBJF_NODATA;
                      }
                    break;
                  case NGDOMITEM:
                    nval=tabentry->npoint;
                    break;
                }
              if (noutmax>0 && nval>noutmax) nval=noutmax;  
            }

          if (nval>0) 

            { k=0;
              dpage[ipage].x[k++]=(double)i+1;
              dpage[ipage].x[k++]=(double)ival+1;
              if (t->idspec.itemkey)
                dpage[ipage].field[k++]=tabentry->itemkey;
              else
                for (j=0;j<t->idspec.ncvar;j++)
                  dpage[ipage].field[k++]=tabentry->caseid[t->idspec.cvar[j]];

              switch (t->itemcat)

                { case MATRIX:

                    dpage[ipage].x[k++]=(double)row;
                    dpage[ipage].x[k++]=(double)col++;
                    for (j=0;j<item->nvar;j++) if (t->varactive[j])
                      { if (tabentry->value[j][ival]==OBJF_NODATA) 
                          dpage[ipage].x[k++]=NODATA;
                        else 
                          dpage[ipage].x[k++]=(double)tabentry->value[j][ival];
                      }

                    if (col>tabentry->ncol)
                      { row++;
                        col=1;
                      }
                    break;

                  case GDOMITEM:

                    for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
                      { if (dim[j]==OBJF_NODATA)
                          dpage[ipage].x[k++]=NODATA; 
                        else     
                          dpage[ipage].x[k++]=(double)dim[j];
                      }
                    for (j=0;j<item->nvar;j++) if (t->varactive[j])
                      { if (tabentry->value[j][ival]==OBJF_NODATA)
                          dpage[ipage].x[k++]=NODATA; 
                        else     
                          dpage[ipage].x[k++]=(double)tabentry->value[j][ival];
                      }

                    dimupdate=1;
                    for (j=N_DIM-1; j>=0 && dimupdate ;j--) 
                      if (tabentry->dimflag[j])
                        { dim[j]+=tabentry->gspec[j].step;
                          if (++count[j]==tabentry->gspec[j].n)
                            { count[j]=0;
                              dim[j]=tabentry->gspec[j].first;
                            }
                          else dimupdate=0;
                        }
                    break;

                  case NGDOMITEM:

                    for (j=0;j<N_DIM;j++) if (t->dimflag[j]) 
                      { if (tabentry->dim[j][ival]==OBJF_NODATA)
                          dpage[ipage].x[k++]=NODATA;  
                        else  
                          dpage[ipage].x[k++]=(double)tabentry->dim[j][ival];
                      }
                    for (j=0;j<item->nvar;j++) if (t->varactive[j])
                      { if (tabentry->value[j][ival]==OBJF_NODATA)
                          dpage[ipage].x[k++]=NODATA;  
                        else  
                          dpage[ipage].x[k++]=(double)tabentry->value[j][ival];
                      }

                    break;
                }

              ipage++;
              ival++;
            }

          if (ival==nval) 
            { newitem=1;
              i++;
            }
        }

      /* Write page */

      for (j=0;j<nidvar;j++) sigdig[j]=DEFAULT;
      switch (t->itemcat)
        { case MATRIX: 
            for (j=nidvar;j<nidvar+2;j++) sigdig[j]=DEFAULT;
            for (j=nidvar+2;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
            break;
          case GDOMITEM:
          case NGDOMITEM:
            for (j=nidvar;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
            break;
        }
      
      setcolwidths(nvar,varname,ipage,dpage,sigdig,CWIDTH_MAX,fw);

      if (pageno>1) fprintf(outfile,"\n");
      if (npage>1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,2,varname,fw,sigdig,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status!=OK) return status;
    }

  return OK;
}


static int writeitemdatatab (FILE *outfile, int ttb, struct itemtabrec *t,
                             char *numvarname, struct itemnamerec *item, 
                             char *dimname[]) 

{ struct datarec d={buff1.x,NULL,buff1.field,NULL};
  char *varname[NFIELD_MAX];
  int sigdig[NFIELD_MAX];
  struct itemrec *tabentry;
  int count[N_DIM_NZ];
  OBJF_FLOAT_TYPE dim[N_DIM_NZ];
  unsigned long nval,ival;
  int status,nvar,nidvar,newitem,dimupdate,i,j,k,row,col;

  if (!outfile) return OK;

  /* Set variable names */

  nvar=0;
  varname[nvar++]=numvarname;
  if (t->idspec.itemkey) varname[nvar++]=t->idspec.varname[0];
  else
    for (i=0;i<t->idspec.ncvar;i++) varname[nvar++]=t->idspec.varname[i];
  nidvar=nvar;  

  switch (t->itemcat)
    { case MATRIX:
        varname[nvar++]="row";
        varname[nvar++]="col";
        break;
      case GDOMITEM:
      case NGDOMITEM:
        for (j=0;j<N_DIM;j++)
          if (t->dimflag[j]) varname[nvar++]=dimname[j];
        break;
    }

  for (i=0;i<item->nvar;i++)
    if (t->varactive[i]) varname[nvar++]=item->varname[i];

  /* Remove number fields in buffer for id variable data to force use 
     of values in string field */

  for (j=1;j<nidvar;j++) d.x[j]=NODATA;

  /* Remove string fields in buffer for numerical data to force
     use of values in number field */

  d.field[0]=NULL;
  for (j=nidvar;j<nvar;j++) d.field[j]=NULL;

  /* Set format */

  if (!ttb)
    { for (j=0;j<nidvar;j++) sigdig[j]=DEFAULT;
      switch (t->itemcat)
        { case MATRIX:
            for (j=nidvar;j<nidvar+2;j++) sigdig[j]=DEFAULT;
            for (j=nidvar+2;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
            break;
          case GDOMITEM:
          case NGDOMITEM:
            for (j=nidvar;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
            break;
        }
    }

  /* Write header */

  status=writevar(outfile,ttb,nvar,varname,NULL,1);
  if (status!=OK) return status;

  /* Write data */

  newitem=1;
  i=0;

  while (i<t->ndata)

    { if (newitem)

        { tabentry=&t->table[i];
          newitem=0;
          ival=0;
          switch (t->itemcat)
            { case MATRIX:
                nval=tabentry->nrow*tabentry->ncol;
                row=1;
                col=1;
                break;
              case GDOMITEM:
                nval=1;
                for (j=0;j<N_DIM;j++) if (t->dimflag[j])
                  { nval*=tabentry->gspec[j].n;
                    count[j]=0;
                    if (tabentry->dimflag[j])
                      dim[j]=tabentry->gspec[j].first;
                    else dim[j]=OBJF_NODATA;
                  }
                break;
              case NGDOMITEM:
                nval=tabentry->npoint;
                break;
            }
        }

      if (nval>0)

        { k=0;
          d.x[k++]=(double)(i+1); 
          if (t->idspec.itemkey)
            d.field[k++]=tabentry->itemkey;
          else
            for (j=0;j<t->idspec.ncvar;j++)
              d.field[k++]=tabentry->caseid[t->idspec.cvar[j]];

          switch (t->itemcat)

            { case MATRIX:

                d.x[k++]=(double)row; 
                d.x[k++]=(double)col++; 
                for (j=0;j<item->nvar;j++) if (t->varactive[j])
                  { if (tabentry->value[j][ival]==OBJF_NODATA)
                      d.x[k++]=NODATA;
                    else
                      d.x[k++]=(double)tabentry->value[j][ival];
                  }

                if (col>tabentry->ncol)
                  { row++;
                    col=1;
                  }
                break;

              case GDOMITEM:

                for (j=0;j<N_DIM;j++) if (t->dimflag[j])
                  { if (dim[j]==OBJF_NODATA)
                      d.x[k++]=NODATA;
                    else
                      d.x[k++]=(double)dim[j];
                  }
                for (j=0;j<item->nvar;j++) if (t->varactive[j])
                  { if (tabentry->value[j][ival]==OBJF_NODATA)
                      d.x[k++]=NODATA;
                    else
                      d.x[k++]=(double)tabentry->value[j][ival];
                  }

                dimupdate=1;
                for (j=N_DIM-1; j>=0 && dimupdate ;j--)
                  if (tabentry->dimflag[j])
                    { dim[j]+=tabentry->gspec[j].step;
                      if (++count[j]==tabentry->gspec[j].n)
                        { count[j]=0;
                          dim[j]=tabentry->gspec[j].first;
                        }
                      else dimupdate=0;
                    }
                break;

              case NGDOMITEM:

                for (j=0;j<N_DIM;j++) if (t->dimflag[j])
                  { if (tabentry->dim[j][ival]==OBJF_NODATA)
                      d.x[k++]=NODATA;
                    else
                      d.x[k++]=(double)tabentry->dim[j][ival];
                  }
                for (j=0;j<item->nvar;j++) if (t->varactive[j])
                  { if (tabentry->value[j][ival]==OBJF_NODATA)
                      d.x[k++]=NODATA;
                    else
                      d.x[k++]=(double)tabentry->value[j][ival];
                  }

                break;
            }

          if (ttb)
            status=writedata(outfile,ttb,&d,nvar,NULL,NULL,NULL,DEFAULT,1,1);
          else
            status=writedata(outfile,ttb,&d,nvar,NULL,NULL,sigdig,DEFAULT,1,1);
          if (status!=OK) return status;

          ival++;
        }

      if (ival==nval)
        { newitem=1;
          i++;
        }
    }

  return OK;
}


int writeouttabvardescrip (int iouttab, FILE *outfile, 
                           struct outtabnamerec *outtab, char *prefix, 
                           int fullvarlist)

{ int i; 

  if (!outfile) return OK;

  for (i=0;i<outtab->nvar;i++) if (fullvarlist || outvaractive[iouttab][i])
    { if (outtab->vartext[i])
        { if (fprintf(outfile,"%s %s: %s\n",prefix,outtab->varname[i],
                      outtab->vartext[i])<0) 
          return OUTERR;
        }
      else
        { if (fprintf(outfile,"%s %s: --- undescribed variable ---\n",prefix,
                      outtab->varname[i])<0) 
          return OUTERR;
        }
    }

  fprintf(outfile,"\n");
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writeouttabvar (int iouttab, FILE *outfile, char *casevarname[], 
                    struct outtabnamerec *outtab) 

{ char *varname[NFIELD_MAX];
  int ttb,nvar,i,status;

  if (!outfile) return OK;
  if (N_CASEVAR>0 && !casevarname) return OK;

  ttb=outtabttb[iouttab];

  nvar=0;
  varname[nvar++]="casenum";
  if (casevarname) for (i=0;i<N_CASEVAR;i++) varname[nvar++]=casevarname[i];
  for (i=0;i<outtab->nvar;i++) 
    if (outvaractive[iouttab][i]) varname[nvar++]=outtab->varname[i];
  status=writevar(outfile,ttb,nvar,varname,NULL,1);
  if (status!=OK) return status;
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writeouttabdata (int iouttab, FILE *outfile, int flush, int nrec, 
                     struct caseinputrec *tabentry, int noutvar, ...) 

{ struct datarec d={buff1.x,NULL,buff1.field,NULL};
  OBJF_FLOAT_TYPE *value[NFIELD_MAX]; 
  int sigdig[NFIELD_MAX];
  va_list a;
  int ttb,nvar,nactive,i,j,k,status;

  if (!outfile) return OK;
  if (N_CASEVAR>0 && !tabentry->caseid) return OK;

  ttb=outtabttb[iouttab];

  va_start(a,noutvar);
  for (j=0;j<noutvar;j++) value[j]=va_arg(a, OBJF_FLOAT_TYPE *);
  va_end(a); 

  nactive=0;
  for (j=0;j<noutvar;j++)
    if (outvaractive[iouttab][j]) nactive++; 
 
  nvar=N_CASEVAR+nactive+1;
  d.field[0]=NULL;
  d.x[0]=(double)tabentry->casenum;
  if (!ttb) sigdig[0]=OBJF_SIGDIG_DEF;
  k=1;
  for (j=1;j<=N_CASEVAR;j++) 
    { d.field[k]=tabentry->caseid[j];
      d.x[k]=NODATA;
      k++;
    }
  for (j=0;j<nactive;j++) 
    { d.field[k]=NULL;
      if (!ttb) sigdig[k]=OBJF_SIGDIG_DEF;
      k++;
    }
 
  for (i=0;i<nrec;i++)
    { k=N_CASEVAR+1; 
      for (j=0;j<noutvar;j++) if (outvaractive[iouttab][j]) 
        { d.x[k]=(double)value[j][i]; 
          if (d.x[k]==OBJF_NODATA) d.x[k]=NODATA;
          k++;
        }
      if (ttb)
        status=writedata(outfile,ttb,&d,nvar,NULL,NULL,NULL,DEFAULT,1,1); 
      else
        status=writedata(outfile,ttb,&d,nvar,NULL,NULL,sigdig,DEFAULT,1,1); 
      if (status!=OK) return status;
    }
  if (flush)
    if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writeouttabdata_array (int iouttab, FILE *outfile, int flush, 
                           int nrec, struct caseinputrec *tabentry, 
                           int noutvar, OBJF_FLOAT_TYPE *value)

{ struct datarec d={buff1.x,NULL,buff1.field,NULL};
  int sigdig[NFIELD_MAX];
  int ttb,nvar,nactive,i,j,k,status;

  if (!outfile) return OK;
  if (N_CASEVAR>0 && !tabentry->caseid) return OK;

  ttb=outtabttb[iouttab];

  nactive=0;
  for (j=0;j<noutvar;j++)
    if (outvaractive[iouttab][j]) nactive++; 

  nvar=N_CASEVAR+nactive+1;
  d.field[0]=NULL;
  d.x[0]=(double)tabentry->casenum;
  if (!ttb) sigdig[0]=OBJF_SIGDIG_DEF;
  k=1;
  for (j=0;j<N_CASEVAR;j++) 
    { d.field[k]=tabentry->caseid[j];
      d.x[k]=NODATA;
      k++;
    }
  for (j=0;j<nactive;j++) 
    { d.field[k]=NULL;
      if (!ttb) sigdig[k]=OBJF_SIGDIG_DEF;
      k++;
    }

  for (i=0;i<nrec;i++)
    { k=N_CASEVAR+1; 
      for (j=0;j<noutvar;j++) if (outvaractive[iouttab][j]) 
        { d.x[k]=(double)value[i*noutvar+j];
          if (d.x[k]==OBJF_NODATA) d.x[k]=NODATA;
          k++;
        }
      if (ttb)
        status=writedata(outfile,ttb,&d,nvar,NULL,NULL,NULL,DEFAULT,1,1);
      else
        status=writedata(outfile,ttb,&d,nvar,NULL,NULL,sigdig,DEFAULT,1,1);
      if (status!=OK) return status;
    }
  if (flush) 
    if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writelog (FILE *outfile, int echo, char *text)

{ time_t systime;
  char *s;

  if (!outfile) return OK;

  time(&systime);
  s=ctime(&systime);
  *(s+strlen(s)-1)='\0';
  if (fprintf(outfile,"%s - %s\n",s,text)<0) return OUTERR;
  if (fflush(outfile)==EOF) return OUTERR;
  if (echo) printf("%s - %s\n",s,text);
  
  return OK;
}


int writeheading (FILE *outfile, char *text)

{ int len,leading,trailing;
 
  if (!outfile) return OK;

  if (!text) 
    { leading=PAGEWIDTH;
      while (leading--) fprintf(outfile,"="); 
    }
  else
    { len=strlen(text)+2;
      leading=(PAGEWIDTH-len)/2;
      if (leading<0) leading=0;
      trailing=(PAGEWIDTH-len-leading);
      if (trailing<0) trailing=0;
      fprintf(outfile,"\n");
      while (leading--) fprintf(outfile,"="); 
      if (fprintf(outfile," %s ",text)<0) return OUTERR;
      while (trailing--) fprintf(outfile,"="); 
    }
  fprintf(outfile,"\n\n");

  return OK;
}


void exiterror_logged (FILE *outfile, int status, char *progname, char *s, 
                       int gcount)

{ if (outfile) 
    { writelog(outfile,0,"GFAn TERMINATED ON DETECTING AN ERROR"); 
      fprintf(outfile,"\n");
    }
  exiterror(status,progname,s,gcount);
}


static void clearobjfnames (struct objfnamerec *name)

{ int i,j;

  name->function=NULL;
  name->version=NULL;
  name->description=NULL;
  name->notes=NULL;
  for (i=0;i<N_DIM;i++) name->dimension[i]=NULL; 
  for (i=0;i<N_CASEVAR;i++) name->casevar[i]=NULL; 
  for (i=0;i<N_ITEM;i++) 
    { name->item[i].name=NULL;
      name->item[i].optional=0;
      for (j=0;j<N_DIM;j++) 
        { name->item[i].dimenabled[j]=1; 
          name->item[i].dimoptional[j]=0;
        }
      name->item[i].nvar=0;
      for (j=0;j<N_ITEMVAR_MAX;j++) 
        { name->item[i].varname[j]=NULL;
          name->item[i].vartext[j]=NULL;
          name->item[i].varoptional[j]=0;
          name->item[i].varmin[j]=OBJF_NODATA;
          name->item[i].varmax[j]=OBJF_NODATA;
        }
    }
  for (i=0;i<N_OUTTAB;i++) 
    { name->outtab[i].name=NULL;
      name->outtab[i].nvar=0;
    }

#if OPT_ENABLED == 1 
  for (i=0;i<N_PARMCTRLVAL;i++) name->parmctrlvar[i]=NULL; 
#endif

}


static void deletectrlrec (struct ctrlrec *ctrl)

{ if (!ctrl) return;

  free(ctrl->fieldlist);
  free(ctrl);
}


static void clearcasetab (struct casetabrec *t)

{ int i;

  if (!t) return;

  for (i=0;i<N_ITEM;i++) t->keyflag[i]=0;
  for (i=0;i<t->ndata;i++) free(t->table[i].fieldlist);
  t->ndata=0;
  free(t->table);
  t->table=NULL;
}


static void clearitemtab (struct itemtabrec *t)

{ int i,j;

  if (!t) return;

  t->idspec.itemkey=0;
  t->idspec.ncvar=0;
  for (i=0;i<N_DIM;i++) t->dimflag[i]=0;
  for (i=0;i<t->ndata;i++) 
    { free(t->table[i].fieldlist);
      free(t->table[i].parm);
      for (j=0;j<N_DIM;j++) free(t->table[i].dim[j]);
      for (j=0;j<N_ITEMVAR_MAX;j++) free(t->table[i].value[j]);
    }
  t->ndata=0;
  free(t->table);
  t->table=NULL;
}


#if OPT_ENABLED == 1 

/* OPTIMIZATION RELATED FUNCTIONS */


static int writeoptconfigtemplate (FILE *outfile, char *varname[]) 

{ int optional[N_OPTCONFVAR];
  int i;

  if (!outfile) return OK;

  for (i=0;i<N_OPTCONFVAR;i++) optional[i]=1;
  return writetemplate(outfile,N_OPTCONFVAR,optional,varname,NULL,NULL);
}


static int writefreeparmtemplate (FILE *outfile, char *ctrlvarname[])

{ char *varname[N_PARMCTRLVAL+7];
  int optional[N_PARMCTRLVAL+7];
  int nvar,i;

  if (!outfile) return OK;

  nvar=0;
  optional[nvar]=0;
  varname[nvar++]="parmname";
  for (i=0;i<N_PARMCTRLVAL;i++) 
    { optional[nvar]=0;
      varname[nvar++]=ctrlvarname[i];
    }
  optional[nvar]=1;
  varname[nvar++]="logmin";
  optional[nvar]=1;
  varname[nvar++]="logmax";
  optional[nvar]=1;
  varname[nvar++]="min";
  optional[nvar]=1;
  varname[nvar++]="max";
  optional[nvar]=1;
  varname[nvar++]="nbits_ga";
  optional[nvar]=1;
  varname[nvar++]="powellbounds";
  if (nvar>NFIELD_MAX) return SPECIAL;

  return writetemplate(outfile,nvar,optional,varname,NULL,NULL);
}


static int writeparminittemplate (FILE *outfile, struct itemnamerec *parm)

{ char *varname[NFIELD_MAX];
  int optional[NFIELD_MAX];
  int nvar,i;

  if (!outfile) return OK;

  nvar=0;
  optional[nvar]=1;
  varname[nvar++]="psetkey";
  for (i=0;i<parm->nvar;i++)
    { optional[nvar]=1;
      varname[nvar++]=parm->varname[i];
    }
  if (nvar>NFIELD_MAX) return SPECIAL;

  return writetemplate(outfile,nvar,optional,varname,NULL,NULL);
}


static int readoptconfigfile (char *fname, char *reqvarname[], 
                              struct optdatarec *optdata, int *lineno, 
                              char **errtxt)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  int reqvar[N_OPTCONFVAR];
  struct datarec d={buff1.x,buff1.fieldlist,buff1.field,NULL};
  int status,ttb,ninvar,i,j,found,dummyflag;
  char linebuff[LINE_MAX+1];

  *errtxt=NULL;
  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;
  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;
  
  if (indexlist(N_OPTCONFVAR,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  status=getdata(ninvar,NULL,infile,ttb,NULL,invarname.field,&d,NULL,lineno);
  if (infile) fclose(infile);
  if (status==EOF) return OK;
  if (status!=OK) return status;
 
  if (!ttb) 
    { for (i=0;i<N_OPTCONFVAR;i++) 
        if (reqvar[i]>=0) 
          d.x[reqvar[i]]=sgetnum(d.field[reqvar[i]],&dummyflag);
    }

  i=0;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->gactrl.maxgen=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->gactrl.maxgen<0) optdata->gactrl.maxgen=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->gactrl.mingen=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->gactrl.mingen<0) optdata->gactrl.mingen=0;
    }
  if (optdata->gactrl.mingen>optdata->gactrl.maxgen) 
    optdata->gactrl.mingen=optdata->gactrl.maxgen;
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->gactrl.npopsiz=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->gactrl.npopsiz<0) optdata->gactrl.npopsiz=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->gactrl.nout=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->gactrl.nout<0) optdata->gactrl.nout=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    optdata->gactrl.nchild=(int)(d.x[reqvar[i]]+0.5);
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    optdata->gactrl.psetini=
      (int)(d.x[reqvar[i]]+(d.x[reqvar[i]]<0?-0.5:0.5)) ? 1 : 0;
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    optdata->gactrl.iunifrm=
      (int)(d.x[reqvar[i]]+(d.x[reqvar[i]]<0?-0.5:0.5)) ? 1 : 0;
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    optdata->gactrl.pcross=d.x[reqvar[i]];
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    optdata->gactrl.seed=(int)(d.x[reqvar[i]]+0.5);

  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->powellctrl.maxiter=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->powellctrl.maxiter<0) optdata->powellctrl.maxiter=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->powellctrl.maxiterbracket=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->powellctrl.maxiterbracket<0) 
        optdata->powellctrl.maxiterbracket=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->powellctrl.maxiterbracket0=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->powellctrl.maxiterbracket0<0) 
        optdata->powellctrl.maxiterbracket0=0;
    }
  if (optdata->powellctrl.maxiterbracket0>optdata->powellctrl.maxiterbracket) 
    optdata->powellctrl.maxiterbracket0=optdata->powellctrl.maxiterbracket;
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA) 
    { optdata->powellctrl.maxiterbrent=(int)(d.x[reqvar[i]]+0.5);
      if (optdata->powellctrl.maxiterbrent<0) 
        optdata->powellctrl.maxiterbrent=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA)
    { optdata->powellctrl.ffrctol=d.x[reqvar[i]];
      if (optdata->powellctrl.ffrctol<0) optdata->powellctrl.ffrctol=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA)
    { optdata->powellctrl.xabstol=d.x[reqvar[i]];
      if (optdata->powellctrl.xabstol<0) optdata->powellctrl.xabstol=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA)
    { optdata->powellctrl.ffrctolbrent=d.x[reqvar[i]];
      if (optdata->powellctrl.ffrctolbrent<0)
        optdata->powellctrl.ffrctolbrent=0;
    }
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA)
    optdata->powellctrl.xfrctolbrent=d.x[reqvar[i]];
  i++;
  if (reqvar[i]>=0 && d.x[reqvar[i]]!=NODATA)
    optdata->powellctrl.xabstolbrent=d.x[reqvar[i]];

  if (optdata->gactrl.maxgen==0 && optdata->powellctrl.maxiter==0)
    { *errtxt="no G.A. generations or Powell iterations"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.npopsiz<2)
    { *errtxt="less than 2 in G.A. population (bad 'ga_npop')"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.nout>optdata->gactrl.npopsiz)
    { *errtxt="number of output parameter sets exceeds G.A. population"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.nchild<1 || optdata->gactrl.nchild>2)
    { *errtxt="invalid no. of children ('ga_nchild' must be 1 or 2)"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.nchild==2 && fmod(optdata->gactrl.npopsiz,2)>0.25)
    { *errtxt="incompatible 'ga_nchild' and 'ga_npop'"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.pcross<0 || optdata->gactrl.pcross>1)
    { *errtxt="crossover probability out of range (bad 'ga_pcross')"; 
      return SPECIAL;
    } 
  if (optdata->gactrl.seed<0)
    { *errtxt="seed for G.A. cannot be negative (bad 'ga_seed')"; 
      return SPECIAL;
    }
  if (optdata->gactrl.seed>UINT_MAX)
    { *errtxt="seed for G.A. is too big (bad 'ga_seed')"; 
      return SPECIAL;
    }
  if (optdata->powellctrl.xfrctolbrent<sqrt(DBL_EPSILON))
    { sprintf(linebuff,
              "fractional tolerance must be > %g (bad 'powell_xftolbr' = %g)",
              sqrt(DBL_EPSILON),optdata->powellctrl.xfrctolbrent);
      *errtxt=linebuff;
      return SPECIAL;
    }
  if (optdata->powellctrl.xabstolbrent<=0)
    { *errtxt="absolute tolerance must be > 0 (bad 'powell_xatolbr')";
      return SPECIAL;
    }
    
  return OK;
}


static int readfreeparmfile (char *fname, char *ctrlvarname[], 
                             struct itemnamerec *parm,  
                             struct parmctrlrec *parmctrl, int *lineno, 
                             char **missingvar)

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  char *reqvarname[N_PARMCTRLVAL+7];
  int reqvar[N_PARMCTRLVAL+7];
  struct datarec d={buff1.x,buff1.fieldlist,buff1.field,NULL};
  int status,ttb,ninvar,nreqvar,nvar,i,j,k,found,dummyflag;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;
  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;
  if (ttb) return FMTERR;

  nreqvar=0;
  reqvarname[nreqvar++]="parmname";
  for (i=0;i<N_PARMCTRLVAL;i++) reqvarname[nreqvar++]=ctrlvarname[i];
  reqvarname[nreqvar++]="logmin";
  reqvarname[nreqvar++]="logmax";
  reqvarname[nreqvar++]="min";
  reqvarname[nreqvar++]="max";
  reqvarname[nreqvar++]="nbits_ga";
  reqvarname[nreqvar++]="powellbounds";
  if (nreqvar>NFIELD_MAX) return SPECIAL;

  nvar=indexlist(nreqvar,reqvarname,ninvar,invarname.field,reqvar);

  if (nvar==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
      return BADDATA;
    }

  if (nvar<nreqvar-6)
    { for (i=0;reqvar[i]>=0;i++);
      *missingvar=reqvarname[i];
      return VARERR;
    }

  i=0;
  while ((status=getdata(ninvar,NULL,infile,0,NULL,invarname.field,&d,NULL,
                         lineno))==OK)

    { found=0;
      for (j=0; j<parm->nvar && !found ;j++)
        if (strcmp(d.field[reqvar[0]],parm->varname[j])==0) found=1;
      if (!found) 
        { fprintf(stderr,
                  "*** Parameter '%s' not in optimizable parameter set ***\n",
                  d.field[reqvar[0]]);
          return BADDATA;
        }

      if (i==N_FREEPARM_MAX) return RECLIMIT;
      parmctrl->parmno[i]=j;

      for (j=1;j<nreqvar;j++)
          d.x[reqvar[j]]=sgetnum(d.field[reqvar[j]],&dummyflag);

      k=1;

      for (j=0;j<N_PARMCTRLVAL;j++) 
        { if (d.x[reqvar[k]]==NODATA)
            { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                parmctrl->value[j][i]=parmctrl->value[j][i-1];
              else 
                { fprintf(stderr,"*** Invalid '%s' value ***\n",reqvarname[k]);
                  return BADDATA;
                }
            }
          else parmctrl->value[j][i]=(OBJF_FLOAT_TYPE)d.x[reqvar[k]];
          k++; 
        }

      parmctrl->logtrans[i]=0;

      parmctrl->logmin[i]=OBJF_NODATA;
      if (reqvar[k]>=0)
        { if (d.x[reqvar[k]]==NODATA) 
            { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                { parmctrl->logmin[i]=parmctrl->logmin[i-1];
                  if (parmctrl->logmin[i]!=OBJF_NODATA) parmctrl->logtrans[i]=1;
                }
            }
          else
            { parmctrl->logmin[i]=(OBJF_FLOAT_TYPE)d.x[reqvar[k]];
              parmctrl->logtrans[i]=1;
            }
        }
      k++;

      parmctrl->logmax[i]=OBJF_NODATA;
      if (reqvar[k]>=0) 
        { if (d.x[reqvar[k]]==NODATA)    
            { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                { parmctrl->logmax[i]=parmctrl->logmax[i-1];
                  if (parmctrl->logmax[i]!=OBJF_NODATA) parmctrl->logtrans[i]=1;
                }
            }
          else
            { parmctrl->logmax[i]=(OBJF_FLOAT_TYPE)d.x[reqvar[k]];
              parmctrl->logtrans[i]=1;
            }
        }
      k++;

      if (parmctrl->logtrans[i])
        { if (parmctrl->logmin[i]==OBJF_NODATA) parmctrl->logmin[i]=0;
          if (parmctrl->logmax[i]==OBJF_NODATA) parmctrl->logmax[i]=0;
          parmctrl->min[i]=pow(10,(double)parmctrl->logmin[i]);
          parmctrl->max[i]=pow(10,(double)parmctrl->logmax[i]);
          k+=2;
        }

      else

        { parmctrl->min[i]=0;
          if (reqvar[k]>=0)
            { if (d.x[reqvar[k]]==NODATA)
                { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                    parmctrl->min[i]=parmctrl->min[i-1];
                }
              else parmctrl->min[i]=(OBJF_FLOAT_TYPE)d.x[reqvar[k]];
            }
          k++;

          parmctrl->max[i]=1;
          if (reqvar[k]>=0) 
            { if (d.x[reqvar[k]]==NODATA)
                { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                    parmctrl->max[i]=parmctrl->max[i-1];
                }
              else parmctrl->max[i]=(OBJF_FLOAT_TYPE)d.x[reqvar[k]];
            }
          k++;
        }

      parmctrl->nbits[i]=8;
      if (reqvar[k]>=0) 
        { if (d.x[reqvar[k]]==NODATA)
            { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                parmctrl->nbits[i]=parmctrl->nbits[i-1];
            }
          else 
            { parmctrl->nbits[i]=floor(d.x[reqvar[k]]+0.5);
              if (parmctrl->nbits[i]<2)
                { fprintf(stderr,"*** Invalid 'nbits_ga' value ***\n");
                  return BADDATA;
                }
              if (parmctrl->nbits[i]>genome_nbitslimit)
                { fprintf(stderr,
                          "*** Invalid 'nbits_ga' value; too many bits ***\n");
                  return BADDATA;
                }
            }
        }
      k++;

      if (OBJF_FREEPARMTRANSFORM_REQ) parmctrl->finitepowell[i]=0;
      else 
        { parmctrl->finitepowell[i]=1;
          if (reqvar[k]>=0)
            { if ((strcmp(d.field[reqvar[k]],"OFF")==0) ||
                  (strcmp(d.field[reqvar[k]],"off")==0)) 
                parmctrl->finitepowell[i]=0;
              else if (!((strcmp(d.field[reqvar[k]],"ON")==0) ||
                         (strcmp(d.field[reqvar[k]],"on")==0)))
                { if (d.x[reqvar[k]]==NODATA)
                    { if (i!=0 && strcmp(d.field[reqvar[k]],".")==0)
                        parmctrl->finitepowell[i]=parmctrl->finitepowell[i-1];
                    }
                  else parmctrl->finitepowell[i]=floor(d.x[reqvar[k]]+0.5);
                }
            }
        }

      if (parmctrl->min[i]>=parmctrl->max[i]) 
        { fprintf(stderr,"*** Invalid parameter range ***\n");
          return BADDATA;
        }

      if (parmctrl->logtrans[i])
        { parmctrl->step[i]=(parmctrl->logmax[i]-parmctrl->logmin[i])/
                              pow(2.0,(double)parmctrl->nbits[i]);
        }
      else
        { parmctrl->step[i]=(parmctrl->max[i]-parmctrl->min[i])/
                              pow(2.0,(double)parmctrl->nbits[i]);
        }
      if (parmctrl->step[i]<=0)
        { fprintf(stderr,"*** Bad range or too many bits ***\n");
          return BADDATA;
        }

      i++;
    }

  parmctrl->nparm=i; 

  if (status!=EOF || parmctrl->nparm==0) return status;
  if (infile) fclose(infile);
  return OK;
}


static int readparminitfile (char *fname, struct itemnamerec *parm,
                             struct psettabrec *t, int *lineno) 

{ FILE *infile;
  struct datarec invarname={NULL,buff1.fieldlist,buff1.field,NULL};
  char *reqvarname[NFIELD_MAX];
  int reqvar[NFIELD_MAX];
  int varpos[NFIELD_MAX];
  struct datarec *dnew,*d,*head; 
  struct psetrec *tabentry;
  int status,ninvar,nreqvar,nvar,firstpvar,ttb,i,j,dummyflag;

  *lineno=0;
  if ((infile=fopen(fname,"r"))==NULL) return OPENINERR;
  status=readvar(infile,invarname.fieldlist,invarname.field,&ninvar,&ttb,
                 lineno);
  if (status==OK_INLINE)
    { fclose(infile);
      infile=NULL;
    }
  else if (status!=OK) return status;

  nreqvar=0;
  reqvarname[nreqvar++]="psetkey";
  for (i=0;i<parm->nvar;i++) reqvarname[nreqvar++]=parm->varname[i];
  if (nreqvar>NFIELD_MAX) return SPECIAL;

  if (indexlist(nreqvar,reqvarname,ninvar,invarname.field,reqvar)==0)
    { fprintf(stderr,"*** No recognized variables ***\n");
       return BADDATA;
    }

  /* Determine positions of variables of interest and
     set parameter numbers (which are the same as reqvar index as 
     there is one leading variable) */

  if (reqvar[0]>=0) firstpvar=1;
  else firstpvar=0;

  nvar=0;
  for (i=0;i<nreqvar;i++)
    if (reqvar[i]>=0) 
      { varpos[nvar]=reqvar[i];
        if (i>0) t->parmno[nvar-firstpvar]=i;
        nvar++;
      }

  t->nparm=nvar-firstpvar;
  
  /* Read records into temporary storage 
     Values in previous record are taken as current defaults and
     replace any missing values */

  head=NULL;
  while (t->ndata<NREC_MAX &&
         (status=getdata(ninvar,NULL,infile,ttb,NULL,invarname.field,NULL,
                         &dnew,lineno))==OK)

    { if (!ttb)
        for (i=0;i<nvar;i++)
          dnew->x[varpos[i]]=sgetnum(dnew->field[varpos[i]],&dummyflag);

      if (!head)
        { for (i=0;i<nvar;i++)
            if (dnew->x[varpos[i]]==NODATA) 
              { fprintf(stderr,"*** Invalid '%s' value ***\n",reqvarname[i]);
                return BADDATA;
              }
          head=dnew;
        }
      else
        { for (i=0;i<nvar;i++)
            { if (dnew->x[varpos[i]]==NODATA)
                { if (!ttb && strcmp(dnew->field[varpos[i]],".")==0)
                    dnew->x[varpos[i]]=d->x[varpos[i]];
                  else 
                    { fprintf(stderr,"*** Invalid '%s' value ***\n",
                              reqvarname[i]);
                      return BADDATA;
                    }
                }
            }
          d->next=dnew;
        }

      d=dnew;
      t->ndata++;
    }

  if (t->ndata==NREC_MAX) return RECLIMIT;
  if (status!=EOF || t->ndata==0) return status;
  if (infile) fclose(infile);

  /* Allocate parameter set table space */

  (t->table)=(struct psetrec *)malloc((t->ndata)*sizeof(struct psetrec));
  if (!t->table) return MEMERR;

  /* Copy data into parameter set table */

  d=head;
  for (i=0;i<t->ndata;i++)

    { tabentry=&t->table[i];

      if (reqvar[0]<0) tabentry->psetno=i+1;
      else tabentry->psetno=floor(d->x[varpos[0]]+0.5);
      tabentry->parm=
        (OBJF_FLOAT_TYPE *)malloc(t->nparm*sizeof(OBJF_FLOAT_TYPE));
      if (!tabentry->parm) return MEMERR;
      for (j=0;j<t->nparm;j++)
        tabentry->parm[j]=(OBJF_FLOAT_TYPE)d->x[varpos[firstpvar+j]]; 

      d=d->next;
    }

  freelist(&head);
  return OK;
}


static int writelogoptconfig (FILE *outfile, struct optdatarec *optdata)

{ if (!outfile) return OK;

  if (fprintf(outfile,"Max. no. of G.A. generations (ga_maxgen) .... : %d\n",
              optdata->gactrl.maxgen)<0) return OUTERR;
  if (fprintf(outfile,"Min. no. of G.A. generations (ga_mingen) .... : %d\n",
              optdata->gactrl.mingen)<0) return OUTERR;
  if (fprintf(outfile,"Population size (ga_npop) ................... : %d\n",
              optdata->gactrl.npopsiz)<0) return OUTERR;
  if (fprintf(outfile,"No. of output parameter vectors (ga_nout) ... : %d\n",
              optdata->gactrl.nout)<0) return OUTERR;
  if (fprintf(outfile,"Children per parent pair (ga_nchild) ........ : %d\n",
              optdata->gactrl.nchild)<0) return OUTERR;
  if (fprintf(outfile,"Explicit initialization flag (ga_initflag) .. : %d\n",
              optdata->gactrl.psetini)<0) return OUTERR;
  if (fprintf(outfile,"Uniform crossover flag (ga_uniflag) ......... : %d\n",
              optdata->gactrl.iunifrm)<0) return OUTERR;
  if (fprintf(outfile,"Crossover probability (ga_pcross) ........... : %g\n",
              optdata->gactrl.pcross)<0) return OUTERR;
  if (fprintf(outfile,"Random number seed (ga_seed) ................ : %d\n",
              optdata->gactrl.seed)<0) return OUTERR;
  fprintf(outfile,"\n");
  if (fprintf(outfile,"Max. Powell iterations (powell_max) ......... : %d\n",
              optdata->powellctrl.maxiter)<0) return OUTERR;
  if (fprintf(outfile,"Max. bracketing iterations (powell_maxbrac) . : %d\n",
              optdata->powellctrl.maxiterbracket)<0) return OUTERR;
  if (fprintf(outfile,"Max. for finding a slope (powell_maxbrac0) .. : %d\n",
              optdata->powellctrl.maxiterbracket0)<0) return OUTERR;
  if (fprintf(outfile,"Max. Brent iterations (powell_maxbr) ........ : %d\n",
              optdata->powellctrl.maxiterbrent)<0) return OUTERR;
  if (fprintf(outfile,"Fractional tol. on function (powell_fftol) .. : %g\n",
              optdata->powellctrl.ffrctol)<0) return OUTERR;
  if (fprintf(outfile,"Abs. tol. on parameter knobs (powell_xatol) . : %g\n",
              optdata->powellctrl.xabstol)<0) return OUTERR;
  if (fprintf(outfile,"Frac. tol. on f(x) in Brent (powell_fftolbr)  : %g\n",
              optdata->powellctrl.ffrctolbrent)<0) return OUTERR;
  if (fprintf(outfile,"Frac. tol. on x in Brent (powell_xftolbr) ... : %g\n",
              optdata->powellctrl.xfrctolbrent)<0) return OUTERR;
  if (fprintf(outfile,"Abs. tol on x in Brent (powell_xatolbr) ..... : %g\n",
              optdata->powellctrl.xabstolbrent)<0) return OUTERR;

  return OK;
}


int writelogfreeparmtab (FILE *outfile, struct parmctrlrec *parmctrl, 
                         OBJF_FLOAT_TYPE parm[], char *ctrlvarname[], 
                         char *parmname[])

{ char *varname[N_PARMCTRLVAL+9];
  int sigdig[NFIELD_MAX];
  int fw[N_PARMCTRLVAL+9];
  int logtransflag,status,nvar,npage,pageno,i,j,k,ipage;

  if (!outfile) return OK;

  /* Set variable names
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  logtransflag=0;
  for (i=0; i<parmctrl->nparm && !logtransflag; i++) 
    if (parmctrl->logtrans[i]) logtransflag=1;

  nvar=0;
  varname[nvar]="num";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  varname[nvar++]="parmname";
  if (logtransflag)
    { varname[nvar]="logmin";
      for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
      nvar++;
      varname[nvar]="logmax";
      for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
      nvar++;
    }
  varname[nvar]="min";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  varname[nvar]="max";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  varname[nvar]="nbits_ga";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  if (OBJF_FREEPARMTRANSFORM_REQ==0) varname[nvar++]="powellbounds";
  for (j=0;j<N_PARMCTRLVAL;j++)
    { varname[nvar]=ctrlvarname[j];
      for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
      nvar++;
    }
  if (parm) 
    { varname[nvar]="value";
      for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
      nvar++;
    }

  npage=1+(int)((parmctrl->nparm-1)/NPAGEREC_MAX);
  pageno=1;
  i=0;
  while (i<parmctrl->nparm)

    { /* Set up table data on current logical page */

      for (ipage=0; ipage<NPAGEREC_MAX && i<parmctrl->nparm ;ipage++)

        { k=0;
          dpage[ipage].x[k++]=(double)(i+1);
          dpage[ipage].field[k++]=parmname[parmctrl->parmno[i]-1];
          if (logtransflag)
            { if (parmctrl->logtrans[i])  
                { dpage[ipage].x[k++]=(double)parmctrl->logmin[i];
                  dpage[ipage].x[k++]=(double)parmctrl->logmax[i];
                }
              else
                { dpage[ipage].x[k++]=NODATA;
                  dpage[ipage].x[k++]=NODATA;
                }
            }
          dpage[ipage].x[k++]=(double)parmctrl->min[i];
          dpage[ipage].x[k++]=(double)parmctrl->max[i];
          dpage[ipage].x[k++]=(double)parmctrl->nbits[i];
          if (OBJF_FREEPARMTRANSFORM_REQ==0)
            { if (parmctrl->finitepowell[i]) dpage[ipage].field[k++]="ON";
              else dpage[ipage].field[k++]="OFF";
            }
          for (j=0;j<N_PARMCTRLVAL;j++)
            dpage[ipage].x[k++]=(double)parmctrl->value[j][i]; 
          if (parm) dpage[ipage].x[k++]=parm[i];

          i++;
        }

      /* Write logical page */

      sigdig[0]=DEFAULT;
      k=2;     
      if (logtransflag)
        { sigdig[k++]=OBJF_SIGDIG_DEF;
          sigdig[k++]=OBJF_SIGDIG_DEF;
        }
      sigdig[k++]=OBJF_SIGDIG_DEF;     
      sigdig[k++]=OBJF_SIGDIG_DEF;     
      sigdig[k++]=DEFAULT;    
      k++; 
      for (j=0;j<N_PARMCTRLVAL;j++) sigdig[k++]=OBJF_SIGDIG_DEF;
      sigdig[k++]=OBJF_SIGDIG_DEF;     

      setcolwidths(nvar,varname,ipage,dpage,sigdig,CWIDTH_MAX,fw);

      if (i>NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage>1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,1,varname,fw,sigdig,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status!=OK) return status;
    }

  return OK;
}


int writelogparminittab (FILE *outfile, struct psettabrec *t, char *parmname[])

{ char *varname[N_FREEPARM_MAX+2];
  int sigdig[NFIELD_MAX];
  int fw[N_FREEPARM_MAX+2];
  struct psetrec *tabentry;
  int status,nvar,npage,pageno,i,j,k,ipage;

  if (!outfile) return OK;

  /* Set variable names
     Remove string fields in page buffer for numerical data to force
     use of values in number field */

  nvar=0;
  varname[nvar]="num";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  varname[nvar]="psetkey";
  for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
  nvar++;
  for (j=0;j<t->nparm;j++)
    { varname[nvar]=parmname[t->parmno[j]-1];
      for (i=0;i<NPAGEREC_MAX;i++) dpage[i].field[nvar]=NULL;
      nvar++;
    }

  npage=1+(int)((t->ndata-1)/NPAGEREC_MAX);
  pageno=1;
  i=0;
  while (i<t->ndata)

    { /* Set up table data on current logical page */

      for (ipage=0; ipage<NPAGEREC_MAX && i<t->ndata ;ipage++)

        { tabentry=&t->table[i];

          k=0;
          dpage[ipage].x[k++]=(double)(i+1);
          dpage[ipage].x[k++]=(double)tabentry->psetno;
          for (j=0;j<t->nparm;j++)
            dpage[ipage].x[k++]=tabentry->parm[j];

          i++;
        }

      /* Write logical page */

      sigdig[0]=DEFAULT;
      for (j=1;j<nvar;j++) sigdig[j]=OBJF_SIGDIG_DEF;
      setcolwidths(nvar,varname,ipage,dpage,sigdig,CWIDTH_MAX,fw);

      if (i>NPAGEREC_MAX) fprintf(outfile,"\n");
      if (npage>1) fprintf(outfile,"[Page %d of %d]\n",pageno++,npage);
      status=writelogicalpage(outfile,nvar,1,varname,fw,sigdig,DEFAULT,
                              COLSEP_LOG,PAGEWIDTH,ipage,dpage);
      if (status!=OK) return status;
    }

  return OK;
}


static int writenewparmvar (FILE *outfile, struct parmctrlrec *freeparm, 
                            char *parmname[])

{ char *varname[NFIELD_MAX];
  int ttb,nparm,i,status;
 
  if (!outfile) return OK;
 
  ttb=newparmttb;

  nparm=freeparm->nparm;
  varname[0]="psetkey";
  for (i=0;i<nparm;i++) varname[1+i]=parmname[freeparm->parmno[i]-1];
  varname[nparm+1]="funcmin";
  varname[nparm+2]="n.iter";
  varname[nparm+3]="n.eval";
#ifdef CLOCKS_PER_SEC 
  varname[nparm+4]="cputime";
#else
  varname[nparm+4]="clock";
#endif
  varname[nparm+5]="time";
  varname[nparm+6]="status";
  status=writevar(outfile,ttb,nparm+7,varname,NULL,1);
  if (status!=OK) return status;
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writenewparmdata_noopt (FILE *outfile, struct parmctrlrec *freeparm,
                            struct psettabrec *psettab, double *func)

{ struct datarec d={buff1.x,NULL,NULL,NULL};
  int sigdig[NFIELD_MAX];
  int ttb,nparm,i,j,status;

  if (!outfile) return OK;

  ttb=newparmttb;
  nparm=freeparm->nparm;

  if (!ttb)
    { sigdig[0]=DEFAULT;
      for (i=0;i<nparm;i++) sigdig[i+1]=OBJF_SIGDIG_DEF;
      sigdig[nparm+1]=FLT_DIG;
      sigdig[nparm+2]=DEFAULT;
      sigdig[nparm+3]=DEFAULT;
      sigdig[nparm+4]=DEFAULT;
      sigdig[nparm+5]=DEFAULT;
      sigdig[nparm+6]=DEFAULT;
    }
  
  for (i=0;i<psettab->ndata;i++)
    { d.x[0]=psettab->table[i].psetno;
      for (j=0;j<nparm;j++) d.x[1+j]=psettab->table[i].parm[j];
      d.x[nparm+1]=func[psettab->table[i].psetno-1];
      d.x[nparm+2]=0;
      d.x[nparm+3]=0;
      d.x[nparm+4]=0;
      d.x[nparm+5]=0;
      d.x[nparm+6]=0;
      if (ttb) 
        status=writedata(outfile,ttb,&d,nparm+7,NULL,NULL,NULL,DEFAULT,1,1);
      else
        status=writedata(outfile,ttb,&d,nparm+7,NULL,NULL,sigdig,DEFAULT,1,1);
      if (status!=OK) return status;
    }
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writenewparmdatarec (FILE *outfile, struct parmctrlrec *freeparm, 
                         struct optparmrec *report, clock_t cputime, 
                         double elapsedtime) 

{ struct datarec d={buff1.x,NULL,NULL,NULL};
  int sigdig[NFIELD_MAX];
  int ttb,nparm,i,status; 

  if (!outfile) return OK;

  ttb=newparmttb;
  nparm=freeparm->nparm;

  if (!ttb)
    { sigdig[0]=DEFAULT;
      for (i=0;i<nparm;i++) sigdig[i+1]=OBJF_SIGDIG_DEF;
      sigdig[nparm+1]=FLT_DIG;
      sigdig[nparm+2]=DEFAULT;
      sigdig[nparm+3]=DEFAULT;
      sigdig[nparm+4]=DEFAULT;
      sigdig[nparm+5]=DEFAULT;
      sigdig[nparm+6]=DEFAULT;
    }

  d.x[0]=report->psetno;
  for (i=0;i<nparm;i++) d.x[1+i]=report->parm[i];
  d.x[nparm+1]=report->funcmin;
  d.x[nparm+2]=report->itercount;
  d.x[nparm+3]=report->funccount;
#ifdef CLOCKS_PER_SEC
  d.x[nparm+4]=(double)cputime/CLOCKS_PER_SEC;
#else
  d.x[nparm+4]=cputime;
#endif
  d.x[nparm+5]=elapsedtime;
  d.x[nparm+6]=report->status;
  if (ttb)
    status=writedata(outfile,ttb,&d,nparm+7,NULL,NULL,NULL,DEFAULT,1,1);
  else
    status=writedata(outfile,ttb,&d,nparm+7,NULL,NULL,sigdig,DEFAULT,1,1);
  if (status!=OK) return status;
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


static int writeobjfevalvar_ga (FILE *outfile, struct parmctrlrec *freeparm,
                                char *parmname[])

{ char *varname[NFIELD_MAX];
  int ttb,nparm,i,status;

  if (!outfile) return OK;

  ttb=funcgattb;

  nparm=freeparm->nparm;
  varname[0]="generation";
  varname[1]="individual";
  for (i=0;i<nparm;i++) varname[2+i]=parmname[freeparm->parmno[i]-1];
  varname[nparm+2]="func";
#ifdef CLOCKS_PER_SEC
  varname[nparm+3]="cputime";
#else
  varname[nparm+3]="clock";
#endif

  status=writevar(outfile,ttb,nparm+4,varname,NULL,1);
  if (status!=OK) return status;
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writeobjfevaldata_ga (FILE *outfile, int flush, int gennum, int key, 
                          int nparm, OBJF_FLOAT_TYPE *parm, 
                          double func, double t)

{ struct datarec d={buff1.x,NULL,NULL,NULL};
  int sigdig[NFIELD_MAX];
  int ttb,i,status;
  char *s;

  if (!outfile) return OK;

  ttb=funcgattb;

  if (!ttb)
    { sigdig[0]=DEFAULT;
      sigdig[1]=DEFAULT;
      for (i=0;i<nparm;i++) sigdig[2+i]=OBJF_SIGDIG_DEF;
      sigdig[nparm+2]=OBJF_SIGDIG_DEF;
      sigdig[nparm+3]=DEFAULT;
    }
 
  d.x[0]=(double)gennum;
  d.x[1]=(double)key;
  for (i=0;i<nparm;i++) d.x[2+i]=(double)parm[i];
  d.x[nparm+2]=func;
#ifdef CLOCKS_PER_SEC
  d.x[nparm+3]=t/CLOCKS_PER_SEC;
#else
  d.x[nparm+3]=t;
#endif
 
  if (ttb) 
    status=writedata(outfile,ttb,&d,nparm+4,NULL,NULL,NULL,DEFAULT,1,1);
  else
    status=writedata(outfile,ttb,&d,nparm+4,NULL,NULL,sigdig,DEFAULT,1,1);
  if (status!=OK) return status;
  if (flush)
    if (fflush(outfile)==EOF) return OUTERR;
  
  return OK;
}


static int writeobjfevalvar_powell (FILE *outfile, struct parmctrlrec *freeparm,
                                    char *parmname[])

{ char *varname[NFIELD_MAX];
  int ttb,nparm,i,status;

  if (!outfile) return OK;

  ttb=funcpowellttb;

  nparm=freeparm->nparm;
  varname[0]="psetkey";
  varname[1]="iternum";
  varname[2]="dirnum";
  varname[3]="evalnum";
  for (i=0;i<nparm;i++) varname[4+i]=parmname[freeparm->parmno[i]-1];
  varname[nparm+4]="func";
#ifdef CLOCKS_PER_SEC 
  varname[nparm+5]="cputime";
#else
  varname[nparm+5]="clock";
#endif

  status=writevar(outfile,ttb,nparm+6,varname,NULL,1);
  if (status!=OK) return status;
  if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


int writeobjfevaldata_powell (FILE *outfile, int flush, int key, 
                              int iternum, int dirnum, int evalnum, 
                              int nparm, OBJF_FLOAT_TYPE *parm, 
                              double func, double t)

{ struct datarec d={buff1.x,NULL,NULL,NULL};
  int sigdig[NFIELD_MAX];
  int ttb,i,status;
  char *s;

  if (!outfile) return OK;

  ttb=funcpowellttb;

  if (!ttb)
    { sigdig[0]=DEFAULT;
      sigdig[1]=DEFAULT;
      sigdig[2]=DEFAULT;
      sigdig[3]=DEFAULT;
      for (i=0;i<nparm;i++) sigdig[4+i]=OBJF_SIGDIG_DEF;
      sigdig[nparm+4]=FLT_DIG;
      sigdig[nparm+5]=DEFAULT;
    }

  d.x[0]=key;
  d.x[1]=iternum;
  d.x[2]=dirnum;
  d.x[3]=evalnum;
  for (i=0;i<nparm;i++) d.x[4+i]=(double)parm[i];
  d.x[nparm+4]=func;
#ifdef CLOCKS_PER_SEC
  d.x[nparm+5]=t/CLOCKS_PER_SEC;
#else
  d.x[nparm+5]=t;
#endif

  if (ttb)
    status=writedata(outfile,ttb,&d,nparm+6,NULL,NULL,NULL,DEFAULT,1,1);
  else
    status=writedata(outfile,ttb,&d,nparm+6,NULL,NULL,sigdig,DEFAULT,1,1);
  if (status!=OK) return status;
  if (flush) 
    if (fflush(outfile)==EOF) return OUTERR;

  return OK;
}


static void clearoptconfig (struct optdatarec *optdata)

{ if (!optdata) return;

  /* Set up default optimizer configuration */

  optdata->gactrl.maxgen=1000; 
  optdata->gactrl.mingen=200; 
  optdata->gactrl.nchild=1;
  optdata->gactrl.iunifrm=1;
  optdata->gactrl.pcross=0.5;
  optdata->gactrl.seed=1000;
  optdata->gactrl.npopsiz=5;
  optdata->gactrl.nout=5;
  optdata->gactrl.psetini=0;
  optdata->powellctrl.maxiter=200; 
  optdata->powellctrl.maxiterbracket=100;
  optdata->powellctrl.maxiterbracket0=10;
  optdata->powellctrl.maxiterbrent=100; 
  optdata->powellctrl.ffrctol=1e-4;
  optdata->powellctrl.xabstol=1e-10;
  optdata->powellctrl.ffrctolbrent=5e-5;
  optdata->powellctrl.xfrctolbrent=2e-4;
  optdata->powellctrl.xabstolbrent=1e-10;
}


static void clearparmctrlrec (struct parmctrlrec *p)

{ if (!p) return;

  p->nparm=0;
}


static void clearpsettab (struct psettabrec *t)

{ if (!t) return;

  t->ndata=0;
  free(t->table);
  t->table=NULL;
}


static OBJF_FLOAT_TYPE *saveparmitem (struct itemtabrec *item, 
                                      struct itemnamerec *name)

{ OBJF_FLOAT_TYPE *p; 
  int i,j,ioffset;

  p=(OBJF_FLOAT_TYPE *)malloc(item->ndata*name->nvar*sizeof(OBJF_FLOAT_TYPE));

  for (i=0;i<item->ndata;i++)
    { ioffset=i*name->nvar;
      for (j=0;j<name->nvar;j++)
        p[ioffset+j]=item->table[i].parm[j];
    }

  return p;
}


static void restoreparmitem (OBJF_FLOAT_TYPE *p, struct itemtabrec *item, 
                             struct itemnamerec *name) 

{ int i,j,ioffset;

  for (i=0;i<item->ndata;i++)
    { ioffset=i*name->nvar;
      for (j=0;j<name->nvar;j++)
        item->table[i].parm[j]=p[ioffset+j];
    }

  free(p);
}

#endif
