/***********************************************************************
SOFTWARE RELEASE: MarMOT 1.1.1 DATE: 23 January 2015 LICENCE: CeCILL 2.1

Copyright 2015 John C. P. Hemmings 

Author: John Hemmings
e-mail: marmot@wessexenv.mail1.co.uk 

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

***********************************************************************/
/***********************************************************************

 genome.c - John Hemmings - 14 Jan 2015 

 Genome-handling utilities for implementing genetic algorithms
 where the genome of an individual is represented by a bit field 
 held in one or more unsigned integers. 

***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "genome.h"
#include "garandom.h"


/* Limits associated with genome-handling functions */ 

int genome_nbitslimit = 8 * sizeof(Genome_Element); 
                                   /* Max. no. of bits for representing a 
                                      single parameter value */

Genome_Element genome_uparmmax = ~ (Genome_Element) 0; 
                                   /* Max. integer for representing a parameter
                                      value */

static struct codemaprec codemap;  /* Info. for coding/decoding parameter 
                                      vector */


int make_codemap (int nparm, int *nbits)

/* Set up mapping information for coding and decoding a parameter vector as 
   a bit string (or 'genome'). Parameter vector is of size 'nparm' with
   bits per parameter given by 'nbits[0..nparm-1]'. Return total number 
   of bits to represent vector. */

{ int n,ppos,jstart,jfin;
  int nbitsout;
  int i,j;

  codemap.bitselect =
    (Parmvec_Int) malloc(nparm * sizeof(Genome_Element));
  if (!codemap.bitselect) return 0;

  codemap.bitselect2 =
    (Parmvec_Int) malloc(nparm * sizeof(Genome_Element));
  if (!codemap.bitselect2) return 0;

  codemap.bitshift = (int *) malloc(nparm * sizeof(int));
  if (!codemap.bitshift) return 0;

  codemap.nparm = nparm;
  codemap.nbitsge = 8 * sizeof(Genome_Element);
  codemap.nbitsg = 0;
  for (i = 0; i < nparm; i++) codemap.nbitsg += nbits[i];
  codemap.nge = ceil((double)codemap.nbitsg / codemap.nbitsge);

  n = 0;    /* initialize genome array index */
  ppos = 0; /* initialize bit position (from left) of parameter in genome */

  for (i = 0; i < nparm; i++)

    { /* Calc. bit positions within current element of genome array and
         mark these positions in 'bitselect' */

      jstart = ppos - n * codemap.nbitsge;
      jfin = jstart + nbits[i] - 1;
      if (jfin > codemap.nbitsge - 1) jfin = codemap.nbitsge - 1;
         

      codemap.bitselect[i] = 0;
      for (j = jstart; j <= jfin; j++)
        codemap.bitselect[i] += pow(2, codemap.nbitsge - 1 - j);

      /* Calc. shift for integer representation of parameter to match bit
         positions in genome */

      codemap.bitshift[i] = 0;
      nbitsout = jfin - jstart + 1;
      if (nbitsout == nbits[i]) 
        codemap.bitshift[i] = -(codemap.nbitsge - nbits[i] - jstart);
      else 
        codemap.bitshift[i] = nbits[i] - nbitsout;

      /* Mark bit positions in next element of genome array (if any)
         in 'bitselect2' */

      codemap.bitselect2[i] = 0;
      for (j = 0; j < codemap.bitshift[i]; j++)
        codemap.bitselect2[i] += pow(2, codemap.nbitsge - 1 - j);

      /* Position for next parameter */

      ppos += nbits[i];
      if (codemap.bitshift[i] >= 0) n++;
    }

  /* Mark bit positions for any unused part of last element */

  i = nparm - 1;
  if (codemap.bitselect2[i] != 0) 
    jfin = jstart + nbits[i] - 1 - codemap.nbitsge;  

  codemap.njunkbits=0;
  codemap.junkbits=0;
  for (j = jfin + 1 ; j < codemap.nbitsge; j++)
   { codemap.njunkbits++;
     codemap.junkbits += pow(2, codemap.nbitsge - 1 - j);
   }

  /* Return genome length */

  return codemap.nbitsg;
}


void clear_codemap ()

{ codemap.nparm = 0;
  codemap.nbitsg = 0;
  codemap.nbitsge = 0;
  codemap.nge = 0;
  codemap.njunkbits = 0;
  codemap.junkbits = 0;
  free(codemap.bitselect);
  free(codemap.bitselect2);
  free(codemap.bitshift);
}


Population malloc_population (int n)

/* Allocate space for an array of genomes representing a population of
   size 'n' (Each individal genome is represented by an array of genome
   elements) */

{ Population p;
  int i;

  p = (Population) malloc(n * sizeof(Genome));
  if (!p) return NULL;

  p[0] = (Genome) malloc(n * codemap.nge * sizeof(Genome_Element));
  if (!p[0]) return NULL;
  for (i = 1; i < n; i++) p[i] = p[i-1] + codemap.nge;

  return p;
}


Genome malloc_genome ()

/* Allocate space for an individual genome */

{ Genome g;

  g = (Genome) malloc(codemap.nge * sizeof(Genome_Element));

  return g;
}


Parmvec_Int malloc_uparmvec ()

/* Allocate space for integer representation of a parameter vector */

{ Parmvec_Int u;

  u = (Parmvec_Int) malloc(codemap.nparm * sizeof(Genome_Element));

  return u;
}


void free_population (Population p)

{ free(p[0]);
  free(p);
}


void free_genome (Genome g)

{ free (g);
}


void free_uparmvec (Parmvec_Int u)

{ free (u);
}


Genome genomecode (Parmvec_Int uparmvec, Genome individual) 

/* Code parameter vector represented by integer array 'uparmvec' as a bit 
   string in 'individual'. Return 'individual'. */ 

{ Genome_Element uparmwk,uout; 
  int n,i,j;

  n=0;     /* initialize genome array index */
  uout=0;  /* blank out element ready to receive parameter bits */

  for (i = 0; i < codemap.nparm; i++)

    { /* Shift parameter bits to correct position for genome insertion */

      if (codemap.bitshift[i] < 0) 
        uparmwk = uparmvec[i] << -codemap.bitshift[i]; 
      else 
        uparmwk = uparmvec[i] >> codemap.bitshift[i]; 

      /* Add parameter bits into current element of genome array */

      uout = (uout & ~codemap.bitselect[i]) | (uparmwk & codemap.bitselect[i]);

      /* If current element is full then save it to genome and put any 
         remaining bits in next element */
         
      if (codemap.bitshift[i] > 0)

        { individual[n++] = uout;
          uout = 0;

          uparmvec[i] = uparmvec[i] << (codemap.nbitsge - codemap.bitshift[i]);
          uout = 
            (uout & ~codemap.bitselect2[i]) | 
            (uparmvec[i] & codemap.bitselect2[i]);
        }

      else if (codemap.bitshift[i] == 0) 

        { individual[n++] = uout;
          uout = 0;
        }
    }

  /* Assign last element of genome if not already done */
  
  if (n < codemap.nge) individual[n++] = uout;

  return individual;
}


Parmvec_Int genomedecode (Genome individual, Parmvec_Int uparmvec) 

/* Decode bit string in 'individual' to get integer representation of parameter
   vector in 'uparmvec'. Return 'uparmvec'. */ 

{ Genome_Element uparmwk;
  int n,i;

  n=0;    /* initialize genome array index */

  for (i=0; i < codemap.nparm; i++)

    { /* Get parameter bits from genome array element */ 

      uparmvec[i] = codemap.bitselect[i] & individual[n];

      /* Shift to correct position for integer representation of parameter */

      if (codemap.bitshift[i] < 0) 
        uparmvec[i] = uparmvec[i] >> -codemap.bitshift[i]; 
      else 
        uparmvec[i] = uparmvec[i] << codemap.bitshift[i]; 

      /* If current genome element is exhausted move to next and get more 
         bits if needed */

      if (codemap.bitshift[i] > 0)

        { uparmwk = codemap.bitselect2[i] & individual[++n];
          uparmwk = uparmwk >> (codemap.nbitsge - codemap.bitshift[i]);

          uparmvec[i] = uparmvec[i] | uparmwk;
        }

      else if (codemap.bitshift[i] == 0) n++;
    }

  return uparmvec;
}


Genome genomecreate (int echo, Genome individual)

/* Create a random genome in 'individual'. Return 'individual'. */

{ double x;
  int i;

  /* Assign each element to a random integer in the range 0 to 
     'genome_uparmmax' */ 

  for (i = 0; i < codemap.nge; i++)
    individual[i] = 
      (Genome_Element) ga_random_ulong(0UL, genome_uparmmax, echo);

  return individual;
}


Genome genomecopy (Genome a, Genome b)

/* Write genome of individual 'a' to individual 'b' and return 'b' */

{ int i;

  for (i = 0; i < codemap.nge; i++) b[i] = a[i]; 
  return b;
}


int genomematch (Genome a, Genome b)

/* Return 1 to indicate that genomes of individuals 'a' and 'b' are 
   identical. Otherwise return 0. */

{ int match,i;

  match = 1;
  for (i = 0; match && i < codemap.nge; i++) 
    if (b[i] != a[i]) 
      { if (i < codemap.nge - 1) match = 0;
        else 
          if ((b[i] & ~codemap.junkbits) != (a[i] & ~codemap.junkbits)) match=0;
      }

  return match;
}


int genomediff (Genome a, Genome b)

/* Return number of bits that differ between genomes of individuals 
   'a' and 'b' */

{ Genome_Element ucmp;
  int i,j,jlast;
  int count; 
 
  count = 0;

  for (i = 0; i < codemap.nge; i++) if (b[i] != a[i])

      { ucmp = a[i] ^ b[i];
 
        if (i == codemap.nge - 1 && codemap.njunkbits > 0)
          { ucmp = ucmp >> codemap.njunkbits;
            jlast = codemap.nbitsge - 1 - codemap.njunkbits;
          }
        else jlast = codemap.nbitsge - 1;

        for (j = jlast; j >= 0; j--) 
          { if (ucmp & 1) count++;
            ucmp = ucmp >> 1;
          }
      }

  return count;
}


int genomestr (Genome individual, char *s)

/* Convert bit string in 'individual' to a character representation thereof 
   in 's'. Return number of characters used. */

{ Genome_Element uout;
  int jlast,i,j,k;

  k = codemap.nbitsg;

  s[k--] = '\0';

  for (i = codemap.nge - 1; i >= 0; i--)

    { uout = individual[i];
      if (i == codemap.nge - 1 && codemap.njunkbits > 0)
        { uout = uout >> codemap.njunkbits;
          jlast = codemap.nbitsge - 1 - codemap.njunkbits;
        }
      else jlast = codemap.nbitsge - 1; 

      for (j = jlast; j >= 0; j--) 
       { s[k--] = '0' + (uout & 1);
         uout = uout >> 1; 
       }
    }

  return codemap.nbitsg + 1;
}


Genome crossover_uniform (int echo, double pcross, Genome crossbits,
                          int *ncross)

/* Create a uniform crossover map in 'crossbits', setting bits with
   probability 'pcross'. Number of bits set is recorded in 'ncross'. */

{ unsigned int a;
  int i,j,jlast;
  int valid;

  *ncross = 0;

  for (i = 0; i < codemap.nge; i++)

    { crossbits[i] = 0;
      a = 1;

      jlast = codemap.nbitsge - 1;
      if (i == codemap.nge - 1) jlast -= codemap.njunkbits;

      for (j = codemap.nbitsge - 1; j >= 0; j--)
        { valid = (j <= jlast); 
          if (randomu(echo) <= pcross && valid)
            { (*ncross)++;
              crossbits[i] += a;
            }
          a = a * 2;
        }
    }

  return crossbits;
}


Genome crossover_point (int echo, double pcross, Genome crossbits,
                        int *ncross)

/* Create a single-point crossover map in 'crossbits', setting bits from
   a randomly chosen point after the leading bit to indicate section of
   genome for crossover. Crossover occurs with probability 'pcross'.
   In the event that no crossover is indicated, all bits are zero.
   Number of crossovers 'ncross' is set set 0 or 1 accordingly.
   Return 'crossbits'. */

{ int crosspos,icross,jcross;
  unsigned int a;
  int i,j;

  for (i = 0; i < codemap.nge; i++) crossbits[i] = 0;

  if (randomu(echo) > pcross) *ncross=0;
  else

    { *ncross = 1;

      /* Choose crossover point after leading bit */

      crosspos = ga_random_int(1, codemap.nbitsg - 1, echo);

      /* Determine corresponding genome element and bit position */

      icross = (int)((crosspos + 0.5) / codemap.nbitsge);
      jcross = crosspos - icross * codemap.nbitsge;

      /* Set bits for whole section of genome from crossover point onwards */

      a = 1;
      for (j = codemap.nbitsge - 1; j >= jcross; j--)
        { crossbits[icross] += a;
          a = a * 2;
        }
      for (i = icross + 1; i < codemap.nge; i++) 
        crossbits[i] = ~ (Genome_Element) 0;
    }

  return crossbits;
}


Genome genomecrossover (Genome cross, Genome mate1, Genome mate2, Genome child)

/* Generate new genome in 'child' by combining bits from 'mate1' and 'mate2'
   according to the crossover map 'cross'. Return 'child'. */

{ int i;

  for (i = 0; i < codemap.nge; i++)
    child[i] = (mate1[i] & ~cross[i]) | (mate2[i] & cross[i]);

  return child;
}


void write_codemap (FILE *outfile)

{ int i;

  if (outfile)
    { fprintf(outfile,"codemap.nparm = %d\n",codemap.nparm);
      fprintf(outfile,"codemap.nbitsg = %d\n",codemap.nbitsg);
      fprintf(outfile,"codemap.nbitsge = %d\n",codemap.nbitsge);
      fprintf(outfile,"codemap.nge = %d\n",codemap.nge);
      fprintf(outfile,"codemap.bitselect = ");
      write_uparmvec (outfile,codemap.bitselect);
      fprintf(outfile,"codemap.bitselect2 = ");
      write_uparmvec (outfile,codemap.bitselect2);
      fprintf(outfile,"codemap.bitshift = ");
      if (codemap.nparm > 0)
        { for (i = 0; i < codemap.nparm - 1; i++) 
            fprintf(outfile,"%d ",codemap.bitshift[i]);
          fprintf(outfile,"%d\n",codemap.bitshift[i]);
        }
      else fprintf(outfile,"Null vector\n"); 
      fprintf(outfile,"codemap.njunkbits = %d\n",codemap.njunkbits);
      fprintf(outfile,"codemap.junkbits = ");
      write_genome_element(outfile,codemap.junkbits);
    }
}


void write_population (FILE *outfile, int ng, Population p, double *fitness)

{ int i;

  if (outfile)
    { if (ng > 0)
        for (i = 0; i < ng; i++) 
          { if (fitness) write_genome(outfile,p[i],fitness[i]); 
            else write_genome(outfile,p[i],0); 
          }
      else fprintf(outfile,"Null set\n");
    }
}


void write_genome (FILE *outfile, Genome g, double f)

{ Genome_Element uout;
  int i,width;

  if (outfile)
    { if (codemap.nge > 0) 
        { width = 2 * sizeof(Genome_Element);
          for (i = 0; i < codemap.nge - 1; i++) 
            fprintf(outfile,"%0*X ",width,g[i]);
          uout = g[i] & ~codemap.junkbits;  
          fprintf(outfile,"%0*X %g\n",width,uout,f);
        }
      else fprintf(outfile,"Null vector\n");
    }
}


void write_uparmvec (FILE *outfile, Parmvec_Int u)

{ int i,width;

  if (outfile)
    { if (codemap.nparm > 0) 
        { width = 2 * sizeof(Parmvec_Int_Element);
          for (i = 0; i < codemap.nparm - 1; i++) 
            fprintf(outfile,"%0*X ",width,u[i]);
          fprintf(outfile,"%0*X\n",width,u[i]);
        }
      else fprintf(outfile,"Null vector\n");
    }
}


void write_genome_element (FILE *outfile, Genome_Element uout) 

{ int width;

  if (outfile) 
    { width = 2 * sizeof(Genome_Element);
      fprintf(outfile,"%0*X\n",width,uout);
    }
}
